/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

/*
 * Filename       : TDataTypesDL.cpp
 * Description    : A sample program to add/retrieve/update/delete a document containing 
 *                  different type of attributes, that is, a string, int, Date, Time, etc
 *                  It uses an existing index class "DataType_Index".
 *
 * Preparation to run this program:
 *
 *          	    - use System Administration GUI tools to create Index-Class "DataType_Index" 
 *                    with the following attributes:
 *	  
 *          		IndexClassName : DataType_Index
 *          		 ...attribute [1] :
 *          		     name          = Data_FString
 *          		     type          = 2
 *          		     size          = 20
 *          		     nullable      = true
 *          		     precision     = 20
 *          		     scale         = 1
 *          		     stringType    = 4
 *          		 ...attribute [2] :
 *          		     name          = Data_VString
 *          		     type          = 1
 *          		     size          = 32
 *          		     nullable      = true
 *          		     precision     = 32
 *          		     scale         = 1
 *          		     stringType    = 4
 *          		 ...attribute [3] :
 *          		     name          = Data_Short
 *          		     type          = 3
 *          		     size          = 6
 *          		     nullable      = true
 *          		     precision     = 32767
 *          		     scale         = -32768
 *          		     stringType    = 0
 *          		 ...attribute [4] :
 *          		     name          = Data_Long
 *          		     type          = 4
 *          		     size          = 11
 *          		     nullable      = true
 *          		     precision     = 2147483647
 *          		     scale         = -2147483648
 *          		     stringType    = 0
 *          		 ...attribute [5] :
 *          		     name          = Data_Date
 *          		     type          = 7
 *          		     size          = 10
 *          		     nullable      = true
 *          		     precision     = 0
 *          		     scale         = 0
 *          		     stringType    = 0
 *          		 ...attribute [6] :
 *          		     name          = Data_Time
 *          		     type          = 8
 *          		     size          = 8
 *          		     nullable      = true
 *          		     precision     = 0
 *          		     scale         = 0
 *          		     stringType    = 0
 *          		 ...attribute [7] :
 *          		     name          = Data_Timestamp
 *          		     type          = 9
 *          		     size          = 26
 *          		     nullable      = true
 *          		     precision     = 0
 *          		     scale         = 0
 *          		     stringType    = 0
 *          		 ...attribute [8] :
 *          		     name          = Data_Decimal
 *          		     type          = 6
 *          		     size          = 7
 *          		     id            = 125
 *          		     nullable      = true
 *          		     precision     = 5
 *          		     scale         = 2
 *          		     stringType    = 0
 *         	   
 *          		 This index class will be used in this sample program.
 *
 * Program's steps:
 *                  - connect to DL datastore
 *                  - create a document DDO
 *                  - add different type of attributes to this DDO
 *                  - set the value of these attributes
 *                  - display the content of this DDO
 *                  - add this DDO to the DL persistent store
 *                  - retrieve it back, and display the content
 *                  - update this DDO, retrieve it back and display the content
 *                  - delete this document DDO from the persistent store
 *                  - disconnect from DL datastore
 */

#include <iostream.h>
#include <DKException.hpp>
#include <DKAny.hpp>
#include <DKDDO.hpp>
#include <DKDatastoreDL.hpp>
#include <DKFolder.hpp>
#include <DKParts.hpp>
#include <dkCollection.hpp>
#include <dkXDO.hpp>
#include <DKBlobDL.hpp>
#include <DKPidXDODL.hpp>

void displayDDO(DKDDO* ddo);

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;

   // Checking for input parameters
   if (argc < 4) {
      cout<<"Usage: " << endl;
      cout<<"    TDataTypes <LibServerName> <userId> <password>" << endl;
      return;
   }
   // input parameters
   DKString serverName = DKString(argv[1]);
   DKString userId     = DKString(argv[2]);
   DKString password   = DKString(argv[3]);

   DKString indexclass, itemId;

   indexclass = "DataType_Index";

   cout << "index class = " << indexclass << endl;
   DKDatastoreDL dl; // must be outside try/catch block

  try 
  {
    
     // connect to the datastore
     cout << "Connecting to datastore\n";
     dl.connect(serverName, userId, password);
     cout << "datastore " << dl.datastoreName() << " connected " << endl;
           
     DKPid pid;
     pid.setObjectType(indexclass);
     
     DKDDO ddo( &dl, pid);
     
     // set the item type to be a document
     ddo.addProperty(DK_CM_PROPERTY_ITEM_TYPE, DK_CM_DOCUMENT);
     
     int i;
     short dataId;
     char* attrName[] = { 
                          "Data_FString",
                          "Data_VString",
                          "Data_Short",
                          "Data_Long"
                          ,"Data_Date"
                          ,"Data_Time"
                          ,"Data_Timestamp"
                          ,"Data_Decimal"
                         };
     
     DKAny attrValue[8];
     attrValue[0] = DKString("fixed length string");
     attrValue[1] = DKString("variable length string");
     attrValue[2] = (short)10;
     attrValue[3] = 2147483647L;
     attrValue[4] = DKDate(24, DKDate::April,1990);
     attrValue[5] = DKTime(24,10,20);
     attrValue[6] = DKTimestamp(1997, 8, 16, 10, 11, 12, 999);
     attrValue[7] = DKString("5.2");
     
     short attr_prop_type[] = { 
                                DK_CM_DATAITEM_TYPE_STRING,
                                DK_CM_DATAITEM_TYPE_STRING,
                                DK_CM_DATAITEM_TYPE_SHORT,
                                DK_CM_DATAITEM_TYPE_LONG
                               ,DK_CM_DATAITEM_TYPE_DATE
                               ,DK_CM_DATAITEM_TYPE_TIME
                               ,DK_CM_DATAITEM_TYPE_TIMESTAMP
                               ,DK_CM_DATAITEM_TYPE_STRING         // DK_CM_DATAITEM_TYPE_DECIMAL inside
                               }; 
     
     short attrCount = sizeof(attrName) / sizeof(char*);
        
     for( i=0; i < attrCount; i++)
     {
           dataId = ddo.addData(attrName[i]);
           ddo.addDataProperty( dataId, DK_CM_PROPERTY_TYPE, attr_prop_type[i]);              // mandatory
           ddo.addDataProperty( dataId, DK_CM_PROPERTY_NULLABLE, DK_CM_TRUE);                    // optional
           ddo.addDataProperty( dataId, DK_CM_PROPERTY_ACCESS_RIGHT, DK_CM_ATTRIBUTE_READWRITE); // optional
           ddo.setData( dataId, attrValue[i] );
     }

     cout << "\n *** displaying the ddo before adding it *** \n";
     displayDDO(&ddo);

     // add the ddo
     cout << "\n *** adding the ddo *** \n" << endl;
     ddo.add();
     pid = ddo.getPid();
     cout << " the new added ddo itemId = " << pid.getId() << endl;     
     // retrieve it back
     cout << " retrieve the newly added ddo, with itemId = " << pid.getId() << endl;     
     
     DKDDO* nddo;
     nddo = new DKDDO(&dl, pid);
     try {
        nddo->retrieve();
     } 
     catch (DKException &e) {
        cout << " item does not exist " << e.text() << endl;
        return;
     }
     
     cout << " displaying the retrieved new ddo" << endl;
     displayDDO(nddo);
     
     cout << "\n *** updating this DDO *** \n" << endl;
           
     DKAny u_attr[8];
     u_attr[0] = DKString("updated char");
     u_attr[1] = DKString("updated vchar");
     u_attr[2] = (short) -32768;
     u_attr[3] = (long) (-2147483647L - 1);
     u_attr[4] = DKDate(15, DKDate::May,1997);
     u_attr[5] = DKTime(12, 15, 45);
     u_attr[6] = DKTimestamp(1990, 4, 24, 8, 36, 17, 250);
     u_attr[7] = DKString("10.6");
  
     short u_dataid;
     for (i = 0; i < attrCount; i++) {
         u_dataid = nddo->dataId(attrName[i]);
         cout << " original " << attrName[i] << " value = " << nddo->getData(u_dataid) << endl;
         nddo->setData(u_dataid, u_attr[i]);
         cout << "      new " << attrName[i] << " value = " << nddo->getData(u_dataid) << endl;
     }
     
     nddo->update();
     pid = nddo->getPid();
     
     cout << " retrieve the updated ddo, with itemId = " << pid.getId() << endl;
     DKDDO* uddo;
     uddo = new DKDDO(&dl, pid);
     uddo->retrieve();
     cout << " displaying the updated ddo" << endl;
     displayDDO(uddo);
          
     cout << "\n  *** deleting the ddo ***\n";
     uddo->del();
     dl.disconnect();
     cout << " datastore disconnected\n";

     // make sure there is no memory leaks
     delete nddo;
     delete uddo;

   }
   catch(DKException &exc)
   {
	 cout << "Error id " << exc.errorId() << endl;
	 for(unsigned long i = 0; i< exc.textCount();i++)
	 {
	    cout << "Error text: " << exc.text(i) << endl;
	 }
	 for (unsigned long g=0; g< exc.locationCount();g++)
	 {
	    const DKExceptionLocation* p = exc.locationAtIndex(g);
	    cout << "Filename: " << p->fileName() << endl;
	    cout << "Function: " << p->functionName() << endl;
	    cout << "LineNumber: " << p->lineNumber() << endl;
	 }
   
	 cout << "Exception Class Name: " << exc.name() << endl;
   }  

   cout << "done" << endl;
}

void displayDDO(DKDDO* ddo) {
     DKString dataName;
     unsigned short itemType;
     DKAny* element;
     DKDDO* item;
     DKBlobDL* blob;
     DKFolder* folder;
     DKParts*  parts;
     dkIterator* iter;
     DKAny any;

     itemType = ddo->getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE);
     cout << " item id[" << ddo->getPid().getId() << "]  ";
     if (itemType == DK_CM_FOLDER)      cout << " is a Folder "    << endl;
     else if (itemType == DK_CM_DOCUMENT)  cout << " is a Document "  << endl;
     else cout << " type is unknown "  << endl;

     for (unsigned short j = 1; j <= ddo->dataCount(); j++) {
         dataName = ddo->getDataName(j);
         cout << " attr[" << j << "] = "
              << dataName  << ",     \t value ["
              << ddo->getData(j) << "] " 
              << ", type = " << ddo->getDataPropertyByName(j,"type")
              << endl;

         if (dataName == DKPARTS) {
            any = ddo->getData(j);
            parts = (DKParts*) any.value();
            if (parts) {
               iter  = parts->createIterator();
               while(iter->more()) {
                    element = iter->next();   
                    blob = (DKBlobDL*) element->value();
                    cout << "  blob part id "  <<
                      ((DKPidXDODL*) (blob->getPid()))->getPartId() 
                        <<  " content class = " 
                        <<  blob->getContentClass() 
                        <<  " rep type = "
                        <<  blob->getRepType()
                        <<  " aff type = "
                        <<  blob->getAffiliatedType()
                        <<  endl;
               }   
               delete iter;
            }
         }

         if (dataName == DKFOLDER) {
            any = ddo->getData(j);
            folder = (DKFolder*) any.value();
            if (folder) {
               iter = folder->createIterator();
               while(iter->more()) {
        	   element = iter->next();   
        	   item = (DKDDO*) element->value();
        	   displayDDO(item);
               }
               delete iter;
            }   
         }
     }
}
