/*****************************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/*****************************************************************************************/

// test list servers/ list schema / list schema attributes from OnDemand datastore

/*
 * Filename       : TListAppGrpsOD.java
 * Description    : test DKDatastoreOD for list functions
 * Change History :
 *
 * Date        Name  Description
 * ---------- -----  ----------------------------------------------------------
 * 1999/04/13  BKH   Initial creation, based on TListCatalogV4.java
 * 1999/10/10  nnc   Ported to C++
 *
 */

#include <iostream.h>
#include <stdio.h>
#include <DKConstant.h>
#include <DKDatastoreOD.hpp>
#include <DKServerDefOD.hpp>
#include <DKAppGrpDefOD.hpp>
#include <DKFieldDefOD.hpp>
#include <DKDatastoreDefOD.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void ListAppGrps(DKDatastoreOD& dsOD);

void main(int argc, char *argv[])
{
  cout << "begin..." << endl;
  // Indicate this is a Console Application on Windows NT/95 for error
  // message code page conversion to OEM code page. If this was a Windows
  // GUI (Graphic User Interface) Application this would not be necessary
  // because DK_CM_SS_WINDOWS is the default.
  DKEnvironment::setSubSystem(DK_CM_SS_CONSOLE);

  //--------------------------------------------------------------
  // Checking for input parameters
  //--------------------------------------------------------------
  if (argc < 4)
  {
    cout<<"Usage: "<<endl;
    cout<<"  TListAppGrpsOD <libsrv> <userid> <pw> <connect_string> <configuration_string>" << endl;
    return;
  }
  // ### get input parameters
  DKString libsrv = DKString(argv[1]);
  DKString userid = DKString(argv[2]);
  DKString pw = DKString(argv[3]);
  DKString connect_string = "";
  DKString configuration_string = "";
  if (argc > 4)
    connect_string = argv[4];
  if (argc > 5 )
    configuration_string = argv[5];


  // must be outside try/catcg block
  DKDatastoreOD dsOD;

  DKServerDefOD * pSV = NULL;
  DKSequentialCollection * pCol = NULL;
  dkIterator * pIter = NULL;

  try
  {
    DKString strServerName;
    DKString strServerType;
    int i = 0;
    int j = 0;
    int k = 0;

    cout << "show servers..." << endl;
    pCol = (DKSequentialCollection*)dsOD.listDataSources();
    pIter = pCol->createIterator();
    while (pIter->more() == TRUE)
    {
      i++;
      pSV = (DKServerDefOD*)((void*)(*pIter->next()));
      strServerName = pSV->getName();
      strServerType = pSV->getServerType();
      cout << "Server Name [" << i << "] - " << strServerName << " Server Type - " << strServerType << endl;
      // cleanup
      delete pSV;
    }
    cout << "  " << i << " server(s) listed" << endl;

    cout << "\n********connecting to datastore...#1***********" << endl;
    dsOD.connect(libsrv,userid,pw,connect_string);
    cout << "datastore connected, libsrv: " << libsrv << ", userid: " << userid << endl;
    cout << "connect_string=" << connect_string << endl;
    ListAppGrps(dsOD);
    dsOD.disconnect();
    cout << "***********datastore disconnected #1***********\n" << endl;

    // enable this section to test re-entrancy
    /*
    cout << "\n********connecting to datastore...#2***********" << endl;
    dsOD.connect(libsrv,userid,pw,connect_string);
    cout << "datastore connected, libsrv: " << libsrv << ", userid: " << userid << endl;
    cout << "connect_string=" << connect_string << endl;
    ListAppGrps(dsOD);
    cout << " " << i << " app group(s) listed" << endl;
    dsOD.disconnect();
    cout << "***********datastore disconnected #2***********\n" << endl;
    */

  }
  catch (DKException &exc)
  {
    cout << "Error id " << exc.errorId() << endl;
    for (unsigned long i = 0; i< exc.textCount();i++)
    {
      cout << "Error text: " << exc.text(i) << endl;
    }
    for (unsigned long g=0; g< exc.locationCount();g++)
    {
      const DKExceptionLocation* p = exc.locationAtIndex(g);
      cout << "Filename: " << p->fileName() << endl;
      cout << "Function: " << p->functionName() << endl;
      cout << "LineNumber: " << p->lineNumber() << endl;
    }

    cout << "Exception Class Name: " << exc.name() << endl;
  }


  // clean up
  if (pCol)
    delete pCol;
  if ( pIter )
    delete pIter;

  cout << "done ..." << endl;
}

void ListAppGrps(DKDatastoreOD& dsOD)
{
  DKString strAppGrp;
  DKSequentialCollection * pCol = NULL;
  dkIterator * pIter = NULL;
  DKSequentialCollection * pCol2 = NULL;
  dkIterator * pIter2 = NULL;
  DKSequentialCollection * pCol3 = NULL;
  dkIterator * pIter3 = NULL;
  DKAppGrpDefOD * agDef = NULL;
  DKFieldDefOD * attrDef = NULL;

  cout << "\nshow app grps..." << endl;

  DKTimestamp timeBefore;
  cerr << "\nBefore calling listEntities, time is " << timeBefore.asString(NULL) << endl;
  pCol = (DKSequentialCollection*)dsOD.listSearchableEntities();  // get app grps
  DKTimestamp timeAfter;
  cerr << "\nAfter calling listEntities, time is " << timeAfter.asString(NULL) << endl;
  pIter = pCol->createIterator();
  int i = 0;

  DKTimestamp timeBefore2;
  cerr << "\nBefore getting attributes, time is " << timeBefore2.asString(NULL) << endl;
  while (pIter->more() == TRUE)
  {
    i++;
    agDef = (DKAppGrpDefOD*)((void*)(*pIter->next()));
    strAppGrp = agDef->getName();
    cout << "  app grp name[" << i << "]: ==>" << strAppGrp << "<==" << endl;
    cout << "  show attributes for " << strAppGrp << " app grp - " << endl;

    // get attributes
    //pCol2 = (DKSequentialCollection*)dsOD.listEntityAttrs(strAppGrp);
    // a much faster way!
    pCol2 = (DKSequentialCollection*)agDef->listAttrs();
    pIter2 = pCol2->createIterator();
    int j = 0;

    while (pIter2->more() == TRUE)
    {
      j++;
      attrDef = (DKFieldDefOD*)(void*)(*pIter2->next());
      cout << "    Attribute name[" << j << "]: ==>" << attrDef->getName() << "<==" << endl;
      cout << "      datastoreType: " << attrDef->datastoreType() << endl;
      cout << "      attributeOf: " << attrDef->getEntityName() << endl;
      cout << "      type: " << attrDef->getType() << endl;
      cout << "      size: " << attrDef->getSize() << endl;
      cout << "      id: " << attrDef->getId() << endl;
      cout << "      precision: " << attrDef->getPrecision() << endl;
      cout << "      scale: " << attrDef->getScale() << endl;
      cout << "      stringType: " << attrDef->getStringType() << endl;
      cout << "      NULLable: " << attrDef->isNullable() << endl;
      cout << "      Queryable: " << attrDef->isQueryable() << endl;
      cout << "      Updatable: " << attrDef->isUpdatable() << endl;
      // cleanup
      delete attrDef;
    }
    cout << "  " << j << " attribute(s) listed for ==>" << strAppGrp << "<==" << " app grp\n" << endl;
    // cleanup
    if ( pIter2 )
      delete pIter2;
    if ( pCol2 )
      delete pCol2;
    if (agDef)
      delete agDef;
  }
  DKTimestamp timeAfter2;
  cerr << "\nAfter getting attributes, time is " << timeAfter2.asString(NULL) << endl;
  cout << " " << i << " app group(s) listed" << endl;

  // cleanup
  if (pCol)
    delete pCol;
  if ( pIter )
    delete pIter;
}
