/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Retrieving an Attribute Group Definition
    Attribute Definitions exist in two forms, the general Attribute Group Definition and
    the Attribute Definition's belonging to a Group, tailored for a specific Component Type. 
    Once an Item or Component Type is created using an Attribute Group Definition, a 
    seperate instance of the Attribute Group Definition and each Attribute Definition of 
    that group will be stored within the Component Type definition that will not be affected
    by any modification of the general Attribute Group Definition or those Attribute 
    Definitions within.  This prevents breaking, destroying, or corrupting existing Data 
    Models and existing Items of those Data Models.

    Naming Convention
        The retrieved definition's list of attributes have attribute names of the following
        format, which will be needed to access and set attribute group information.

            Grouped Attribute's Name Format:  <Attribute Group Name>.<Attribute Name>

    Retrieving Attributes within a Group Directly
        Attribute Definitions within a Group may be retrieved directly without first 
        retrieving the Attribute Group Definition.  This may easily be done by simply
        retrieving a Attribute Definition directly from the Datastore Definition or 
        Component Type Definition using the Nameing Conventioned outlined above.  Please
        refer to SAttributeDefinitionRetrieval for an example of how to retrieve individual
        Attribute Definitions directly.

    Detecting Non-Existant Attribute Group Definitions
        When retrieving Attribute Group Definitions, if it does not exist, null is returned.
        Usage of the null object will result in a NullPointerException.

    Datastore Definition's Attribute Groups
        To obtain the general Attribute Group Definition that the instances added to Item
        Types are based on, retrieve the definition object from the Datastore Definition.
        
    Component Type Definition
        To obtain the specific tailored version of an Attribute Group Definition or any 
        Attribute Definition within for a perticular Item or Component Type, it must be
        retrieved from the Component Type Definition.
        
        Hint:  Use DKComponentTypeDef.retrieveAttrGroup() or retrieveAttr().  Please refer
               to SItemTypeRetrievalICM for more information on how to retrieve an existing
               Component Type Definition.

This Sample will retrieve general Attribute Group Definitions from the Datastore Definition.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKAttrGroupDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SAttributeGroupDefRetrievalICM.hpp"

/************************************************************************************************
 *          FILENAME: SAttributeGroupDefRetrievalICM
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Retrieving & Printing Attribute Group Definitions, Listing Attributes 
 *                    Belonging to Attribute Group, Grouped Attribute Name Format
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Retrieving Attribute Group Definitions
 *                    Listing Attributes Belonging to Attribute Group
 *                    Grouped Attribute Name Format
 *                    Printing Attribute Group Definitions
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SAttributeGroupDefRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Attribute Groups used here must be already defined.  If not, please run
 *                    SAttributeGroupDefCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SAttributeGroupDefRetrievalICM_launch.cpp
 *                    SAttributeGroupDefRetrievalICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SAttributeGroupDefRetrievalICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                            << endl;
            cout << "  SAttributeGroupDefRetrievalICM <database> <userName> <password>"  << endl;
            cout << "  *** Some parameters not specified, using defaults..."             << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==============================================="     << endl;
        cout << "IBM Information Integrator for Content     v" << ver << endl;
        cout << "Sample Program:  SAttributeGroupDefRetrievalICM"     << endl;
        cout << "-----------------------------------------------"     << endl;
        cout << " Database: " << database                             << endl;
        cout << " UserName: " << userName                             << endl;
        cout << "===============================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //--------------------------------------------------------------
            // Retrieving an Attribute Group Definition
            //--------------------------------------------------------------
            // Please refer to the header documentation regarding the choice
            // between sources of the instance desired for the Definition.  
            // This example uses the Datastore Definition's general Attr.
            // Group Definition.
            //
            // Retrieving the "S_carAccident" attribute Group.
            cout << "Retrieving Attribute Group 'S_carAccident'..." << endl;

                DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKAttrGroupDefICM* attrGroupDef = (DKAttrGroupDefICM *) dsDefICM->retrieveAttrGroup("S_carAccident"); // Retrieve the definition.

            cout << "Retrieved Attribute Group 'S_carAccident'." << endl;

            //--------------------------------------------------------------
            // List Attributes of this Group
            //--------------------------------------------------------------
            cout << "Listing the Attributes Belonging to Group 'S_carAccident'..." << endl;

                DKSequentialCollection* attrs = (DKSequentialCollection*) attrGroupDef->listAttrs(); // Get list of attribute definitions
				if (attrs && (attrs->cardinality()>0) )
				{
                
					cout << "\nAttributes Belonging to Group 'S_carAccident':" << endl;
					dkIterator* iter = attrs->createIterator();
					while(iter->more()){  // while there are attributes, continue listing
						DKAttrDefICM* attr = (DKAttrDefICM *) iter->next()->value(); // Move pointer to next element & return that object.
						cout << "     - " << attr->getName() << ":  " << attr->getDescription() << endl;
						delete(attr);   // Free Memory
					}
					cout << "" << endl;

					delete(iter);       // Free Memory
					delete(attrs);      // Free Memory.  Deleteing a Sequental Colection does not Delete Contents.
				}
            
            cout << "Listing the Attributes Belonging to Group 'S_carAccident'." << endl;

            //--------------------------------------------------------------
            // Access & Print Attribute Group Definition Information
            //--------------------------------------------------------------
            cout << "Printing the Attribute Group Definition of 'S_carAccident'..." << endl;

                printAttrGroupDef(attrGroupDef); // Use wrapper funciton from this Sample.
                delete(attrGroupDef);            // Free Memory

            cout << "Printed the Attribute Group Definition of 'S_carAccident'." << endl;

            //--------------------------------------------------------------
            // Retrieving another Attribute Group Definition
            //--------------------------------------------------------------
            // Retrieving the "S_address" attribute Group.
            cout << "Retrieving Attribute Group 'S_address'..." << endl;

                attrGroupDef = (DKAttrGroupDefICM*) dsDefICM->retrieveAttrGroup("S_address"); // Retrieve the definition.

            cout << "Retrieved Attribute Group 'S_address'." << endl;

            //--------------------------------------------------------------
            // Access & Print Attribute Group Definition Information
            //--------------------------------------------------------------
            cout << "Printing the Attribute Group Definition of 'S_address'..." << endl;

                printAttrGroupDef(attrGroupDef); // Use wrapper funciton from this Sample.
                delete(attrGroupDef);            // Free Memory

            cout << "Printed the Attribute Group Definition of 'S_address'." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Print an Attribute Group Definition
    * @param attrGroupDef  Attribute Group Definition, DKAttrGroupDefICM.
    **/
    void SAttributeGroupDefRetrievalICM::printAttrGroupDef(DKAttrGroupDefICM * attrGroupDef){
        cout << "---------------------------------------------"     << endl;
        cout << "         Attribute Group Definition"               << endl;
        cout << "---------------------------------------------"     << endl;
        cout << "       Name:  " <<  attrGroupDef->getName()        << endl;
        cout << "Description:  " << attrGroupDef->getDescription()  << endl;
        cout << " Attributes:   -"                                  << endl;
        DKSequentialCollection* attrs = (DKSequentialCollection*) attrGroupDef->listAttrs(); // Get list of attribute definitions
		if (attrs && (attrs->cardinality()>0) )
		{
			dkIterator* iter = attrs->createIterator();
			while(iter->more()){  // while there are attributes, continue listing
				DKAttrDefICM* attr = (DKAttrDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
				cout << "              " << attr->getName() << ":  " << attr->getDescription() << endl;
				delete(attr);     // Free Memory
			}
			delete(iter);         // Free Memory
			delete(attrs);        // Free Memory
			cout << "---------------------------------------------" << endl;
		}

    }

   /**
    * Print an Attribute Group Definition
    * @param dsICM         Connected DKDatastoreICM object.
    * @param attrGroupName Attribute Group Name.
    **/
    void SAttributeGroupDefRetrievalICM::printAttrGroupDef(DKDatastoreICM * dsICM, char * attrGroupName){
        // First, we need acces to the Datastore Definition object.
        DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
        DKAttrGroupDefICM* attrGroupDef = (DKAttrGroupDefICM*) dsDefICM->retrieveAttrGroup(attrGroupName); // Retrieve the definition.
        SAttributeGroupDefRetrievalICM::printAttrGroupDef(attrGroupDef);

        delete(attrGroupDef);   // Free Memory
    }

   /**
    * Obtain the Definition of the specified Attribute within the Attribute Group.  This
    * function is useful for cases such as in Item Type Creation when only one of a number
    * of Attributes within need to be accessed to set Item Type Design-Time Properties.
    * @param attrGroupDef  Attribute Group Definition object.
    * @param attrName      Name of the Attribute within the Attribute Group to obtain the Definition for.  Use name format "<Group Name>.<Attr Name>".
    * @return  Returns the Attribute Group Definition found within the Attribute group.  Null is returned if it is not found.
    **/
    DKAttrDefICM* SAttributeGroupDefRetrievalICM::getAttrDefWithinGroup(DKAttrGroupDefICM * attrGroupDef, char * attrName){

        DKAttrDefICM* retVal = NULL;

        DKSequentialCollection* attrs = (DKSequentialCollection*) attrGroupDef->listAttrs(); // Get list of attribute definitions
                
		if (attrs && (attrs->cardinality()>0) )
		{
			dkIterator* iter = attrs->createIterator();
			while(iter->more()){  // while there are attributes, continue looking
				DKAttrDefICM* attr = (DKAttrDefICM*) iter->next()->value(); // Move pointer to next element & return that object.
				if(strcmp(attr->getName(),attrName)==0) // if the name matches the one specified, return this definition
					retVal = attr;  // This is the only object we will not delete
				else // Delete all others in the collection, return only the one that the user-application was looking for.
					delete(attr);   // Free Memory
			}


			delete(iter);           // Free Memory
			delete(attrs);          // Free Memory.  Deleteing a Sequental Colection does not Delete Contents.
		}

        return(retVal);  // If it was not found, the value will still be NULL.
    }
