/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Pre-requisit Informaiton:
    This sample pulls together all concepts learned so far, using the IBM COntent Management
    Data Modeling Concepts in a business model example.  The Insurance Model is defined and
    explained in SItemTypeCreationICM.
    
    Minimum Required Reading & Samples Run
        SConnectDisconnectICM                   SFolderICM
        SAttributeDefinitionCreationICM         SLinksICM
        SAttributeGroupDefinitionCreationICM    SResourceMgrDefCreationICM
        SReferenceAttrDefCreationICM            SResourceMgrSetDefaultICM
        SItemTypeCreationICM                    SSMSCollectionDefCreationICM
        SItemCreationICM                        SSMSCollectionDefSetDefaultICM
        SItemRetrievalICM                       SResourceItemCreaitonICM
        SItemUpdateICM                          SResourceItemRetrievalICM
        SItemDeletionICM

For a view of a real-world business model example, please refer to the specified section for
each funcitonality:
     
    Functionality   Section(s)
    --------------  -------------------------------
    Attributes      Auto Claim, All Sections
    Attr Groups     Adjuster Report 
    Child Component Police Report, Insurance Policy
    Folders         Adjuster Report
    Image XDO       Auto Photo
    Inbound Links   Police Report
    Item Creation   Auto Claim, All Sections
    Linking         Insurance Policy, Police Reports
    MIME Types      Auto Claim, All Sections
    Multivalue Attr Police Report, Insurance Policy
    Outbound Links  Insurance Policy
    Printing Items  Printing Section
    Reference Attr  Adjuster Report 
    Resource XDO    Auto Claim, All Sections

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKLobICM.hpp>
#include <DKImageICM.hpp>
#include <DKChildCollection.hpp>
#include <DKLinkCollection.hpp>
#include <DKFolder.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SLinksICM.hpp"
#include "SItemRetrievalICM.hpp"
#include "SInsuranceModelItemCreationICM.hpp"

/************************************************************************************************
 *          FILENAME: SInsuranceModelItemCreaitonICM.cp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Putting everything together into a real-world example, the Sample 
 *                    Insurance Model.  Creates & Prints Items for Insurance Model.  
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Putting everything together into a real-world example
 *                    Creating Insurance Model's Auto Claims
 *                    Creating Insurance Model's Adjustment Reports
 *                    Creating Insurance Model's Police Reports
 *                    Creating Insurance Model's Insurance Policies
 *                    Creating Insurance Model's Auto Photos
 *                    Using Folders
 *                    Creating Links
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SInsuranceModelItemCreationICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                         - SAttributeDefinitionCreationICM
 *                         - SAttributeGroupDefCreationICM
 *                         - SReferenceAttrDefCreationICM
 *                         - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                         - SResourceMgrDefSetDefaultICM
 *                         - SSMSCollectionDefSetDefaultICM
 *                    User-Defined Link Types must be defined.  Please run the following:
 *                         - SLinkTypeDefinitionCreationICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It 
 *                           must be edited first for the RM Confiuration information.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SItemCreationICM.hpp/cpp
 *                    SLinksICM.hpp/cpp
 *                    SItemRetrievalICM.hpp/cpp
 *                    SInsuranceModelItemCreationICM_launch.cpp
 *                    SInsuranceModelItemCreationICM.hpp
 *                    SInsuranceModelICM_AccidenReport_0120200130.tif
 *                    SInsuranceModelICM_AccidenReport_0506200110.tif
 *                    SInsuranceModelICM_AccidenReport_729200173.tif
 *                    SInsuranceModelICM_Adjustment_4-852369.tif
 *                    SInsuranceModelICM_Adjustment_5-654123.tif
 *                    SInsuranceModelICM_Adjustment_6-987654.tif
 *                    SInsuranceModelICM_Claim_4-852369.tif
 *                    SInsuranceModelICM_Claim_5-654123.tif
 *                    SInsuranceModelICM_Claim_6-987654.tif
 *                    SInsuranceModelICM_Photo_1ABC089.jpg
 *                    SInsuranceModelICM_Photo_4IBM134-1.tif
 *                    SInsuranceModelICM_Photo_4IBM134-2.tif
 *                    SInsuranceModelICM_Photo_6usa676-1.tif
 *                    SInsuranceModelICM_Photo_6usa676-2.tif
 *                    SInsuranceModelICM_Policy_10285265410.tif
 *                    SInsuranceModelICM_Policy_10315975308.tif
 *                    SInsuranceModelICM_Policy_10452136412.tif
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SInsuranceModelItemCreationICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                            << endl;
            cout << "  SInsuranceModelItemCreationICM <database> <userName> <password>"  << endl;
            cout << "  *** Some parameters not specified, using defaults..."             << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==============================================="     << endl;
        cout << "IBM Information Integrator for Content     v" << ver << endl;
        cout << "Sample Program:  SInsuranceModelItemCreationICM"     << endl;
        cout << "-----------------------------------------------"     << endl;
        cout << " Database: " << database                             << endl;
        cout << " UserName: " << userName                             << endl;
        cout << "===============================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create Insurance Model Items
            //-------------------------------------------------------------
            // Uses Function provided below.
            cout << "Creating Insurance Model Items..." << endl;

                dkCollection* ddoColl = createInsuranceModelItems(dsICM);  // Use function provided below.

            cout << "Created Insurance Model Items." << endl;

            //-------------------------------------------------------------
            // Free Memory
            //-------------------------------------------------------------
            // Will delete all DDOs created in the funciton above.  May not
            // have to delete DDOs linked by other DDOs.  They will be deleted
            // by the one that links to it.
            cout << "Cleaning Up Memory..." << endl;

                deleteMemoryObjectsReturnedByCreateInsuranceModelItems(ddoColl);

            cout << "Cleaned Up Memory." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Create Items for the Insurance Model.
    * @param dsICM  Connected DKDatastoreICM object.
    * @return  Returns a java.util.Vector containing all DDOs created.
    **/
    dkCollection* SInsuranceModelItemCreationICM::createInsuranceModelItems(DKDatastoreICM* dsICM){

            //------------------------------------------------------------------
            // Insurance Model
            //------------------------------------------------------------------
            // Please refer to the header documentation in SItemTypeCreationICM
            // for descriptions of each Item Type, example data, and the overall
            // Insurance Model.
            //
            // The foillowing sections will assume knowledge of all areas covered.
            // Each have been covered in the prerequisit Samples outlined in the
            // Getting Started section of the Java Sample Readme.
            
            //-------------------------------------------------------------
            // Auto Claim #6-987654 (Gregory Twain)
            //-------------------------------------------------------------
            // Features:  - Item Creation
            //            - Attributes
            //            - XDO
            cout << "Creating Auto Claim #6-987654..." << endl;

                // Create new DDO/XDO Object.
                DKLobICM* autoClaim1 = (DKLobICM*) dsICM->createDDO("S_autoClaim",DK_CM_DOCUMENT);
                
                // Set Attributes
                autoClaim1->setData(autoClaim1->dataId(DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   DKString("6-987654")); 
                autoClaim1->setData(autoClaim1->dataId(DK_CM_NAMESPACE_ATTR,"S_accidentDate"),  DKDate("2001-04-20")); 
                autoClaim1->setData(autoClaim1->dataId(DK_CM_NAMESPACE_ATTR,"S_claimantFName"), DKString("Gregory")); 
                autoClaim1->setData(autoClaim1->dataId(DK_CM_NAMESPACE_ATTR,"S_claimantLName"), DKString("Twain")); 
                autoClaim1->setData(autoClaim1->dataId(DK_CM_NAMESPACE_ATTR,"S_driverLicense"), DKString("B12345696")); 
                autoClaim1->setData(autoClaim1->dataId(DK_CM_NAMESPACE_ATTR,"S_insuredFName"),  DKString("Gregory")); 
                autoClaim1->setData(autoClaim1->dataId(DK_CM_NAMESPACE_ATTR,"S_insuredLName"),  DKString("Twain")); 
                autoClaim1->setData(autoClaim1->dataId(DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  DKString("10452136412")); 

                // Store the File
                autoClaim1->setContentFromClientFile("SInsuranceModelICM_Claim_6-987654.tif"); // Load the file into memory.
                autoClaim1->setMimeType("image/tiff");                                         // Describe the Content Type (MIME Type)
                
                // Save New Item In Datastore
                autoClaim1->add();

            cout << "Created Auto Claim #6-987654." << endl;

            //-------------------------------------------------------------
            // Auto Claim #5-654123 (Jessica Fitzgerald)
            //-------------------------------------------------------------
            cout << "Creating Auto Claim #5-654123..." << endl;

                // Create new DDO/XDO Object.
                DKLobICM* autoClaim2 = (DKLobICM*) dsICM->createDDO("S_autoClaim",DK_CM_DOCUMENT);
                
                // Set Attributes
                autoClaim2->setData(autoClaim2->dataId(DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   DKString("5-654123")); 
                autoClaim2->setData(autoClaim2->dataId(DK_CM_NAMESPACE_ATTR,"S_accidentDate"),  DKDate("2001-05-04")); 
                autoClaim2->setData(autoClaim2->dataId(DK_CM_NAMESPACE_ATTR,"S_claimantFName"), DKString("Jessica")); 
                autoClaim2->setData(autoClaim2->dataId(DK_CM_NAMESPACE_ATTR,"S_claimantLName"), DKString("Fitzgerald")); 
                autoClaim2->setData(autoClaim2->dataId(DK_CM_NAMESPACE_ATTR,"S_driverLicense"), DKString("B98755678")); 
                autoClaim2->setData(autoClaim2->dataId(DK_CM_NAMESPACE_ATTR,"S_insuredFName"),  DKString("Alexander")); 
                autoClaim2->setData(autoClaim2->dataId(DK_CM_NAMESPACE_ATTR,"S_insuredLName"),  DKString("Churhill"));
                autoClaim2->setData(autoClaim2->dataId(DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  DKString("10315975308"));

                // Store the File
                autoClaim2->setContentFromClientFile("SInsuranceModelICM_Claim_5-654123.tif"); // Load the file into memory.
                autoClaim2->setMimeType("image/tiff");                                         // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                autoClaim2->add();

            cout << "Created Auto Claim #5-654123." << endl;

            //-------------------------------------------------------------
            // Auto Claim #4-852369 (George Tolstoy)
            //-------------------------------------------------------------
            cout << "Creating Auto Claim #5-654123..." << endl;

                // Create new DDO/XDO Object.
                DKLobICM* autoClaim3 = (DKLobICM*) dsICM->createDDO("S_autoClaim",DK_CM_DOCUMENT);
                
                // Set Attributes
                autoClaim3->setData(autoClaim3->dataId(DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   DKString("4-852369")); 
                autoClaim3->setData(autoClaim3->dataId(DK_CM_NAMESPACE_ATTR,"S_accidentDate"),  DKDate("2001-07-29")); 
                autoClaim3->setData(autoClaim3->dataId(DK_CM_NAMESPACE_ATTR,"S_claimantFName"), DKString("George")); 
                autoClaim3->setData(autoClaim3->dataId(DK_CM_NAMESPACE_ATTR,"S_claimantLName"), DKString("Tolstoy")); 
                autoClaim3->setData(autoClaim3->dataId(DK_CM_NAMESPACE_ATTR,"S_driverLicense"), DKString("B25878914")); 
                autoClaim3->setData(autoClaim3->dataId(DK_CM_NAMESPACE_ATTR,"S_insuredFName"),  DKString("Gabriela")); 
                autoClaim3->setData(autoClaim3->dataId(DK_CM_NAMESPACE_ATTR,"S_insuredLName"),  DKString("Neruda")); 
                autoClaim3->setData(autoClaim3->dataId(DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  DKString("10285265410")); 

                // Store the File
                autoClaim3->setContentFromClientFile("SInsuranceModelICM_Claim_4-852369.tif"); // Load the file into memory.
                autoClaim3->setMimeType("image/tiff");                                         // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                autoClaim3->add();

            cout << "Created Auto Claim #4-852369." << endl;

            //-------------------------------------------------------------
            // Auto Photo 1 for Vin #1HOME10R4KL987258
            //-------------------------------------------------------------
            // Features:  - Image XDO
            cout << "Creating Auto Photo for Vin #1HOME10R4KL987258..." << endl;

                // Create new DDO/XDO Object.
                DKImageICM* photo1a = (DKImageICM*) dsICM->createDDO("S_autoPhoto",DK_CM_ITEM);
                
                // Set Attributes
                photo1a->setData(photo1a->dataId(DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   DKString("6-987654")); 
                photo1a->setData(photo1a->dataId(DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  DKString("10452136412")); 
                photo1a->setData(photo1a->dataId(DK_CM_NAMESPACE_ATTR,"S_vin"),           DKString("1HOME10R4KL987258")); 
                photo1a->setData(photo1a->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), DKString("Michael")); 
                photo1a->setData(photo1a->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), DKString("Reed")); 
                photo1a->setData(photo1a->dataId(DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), DKDate("2001-02-01")); 

                // Store the File
                photo1a->setContentFromClientFile("SInsuranceModelICM_Photo_4IBM134-1.tif"); // Load the file into memory.
                photo1a->setMimeType("image/tiff");                                          // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                photo1a->add();

            cout << "Created Auto Photo for Vin #1HOME10R4KL987258." << endl;

            //-------------------------------------------------------------
            // Auto Photo 2 for Vin #1HOME10R4KL987258
            //-------------------------------------------------------------
            cout << "Creating Auto Photo for Vin #1HOME10R4KL987258..." << endl;

                // Create new DDO/XDO Object.
                DKImageICM* photo1b = (DKImageICM*) dsICM->createDDO("S_autoPhoto",DK_CM_ITEM);
                
                // Set Attributes
                photo1b->setData(photo1b->dataId(DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   DKString("6-987654")); 
                photo1b->setData(photo1b->dataId(DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  DKString("10452136412")); 
                photo1b->setData(photo1b->dataId(DK_CM_NAMESPACE_ATTR,"S_vin"),           DKString("1HOME10R4KL987258")); 
                photo1b->setData(photo1b->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), DKString("Michael")); 
                photo1b->setData(photo1b->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), DKString("Reed")); 
                photo1b->setData(photo1b->dataId(DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), DKDate("2001-02-01")); 

                // Store the File
                photo1b->setContentFromClientFile("SInsuranceModelICM_Photo_4IBM134-2.tif"); // Load the file into memory.
                photo1b->setMimeType("image/tiff");                                          // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                photo1b->add();

            cout << "Created Auto Photo for Vin #1HOME10R4KL987258." << endl;

            //-------------------------------------------------------------
            // Auto Photo for Vin #1HOME10R4KL987258
            //-------------------------------------------------------------
            cout << "Creating Auto Photo for Vin #1HOME10R4KL987258..." << endl;

                // Create new DDO/XDO Object.
                DKImageICM* photo2 = (DKImageICM*) dsICM->createDDO("S_autoPhoto",DK_CM_ITEM);
                
                // Set Attributes
                photo2->setData(photo2->dataId(DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   DKString("5-654123")); 
                photo2->setData(photo2->dataId(DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  DKString("10315975308")); 
                photo2->setData(photo2->dataId(DK_CM_NAMESPACE_ATTR,"S_vin"),           DKString("5CALL78T6PU743358")); 
                photo2->setData(photo2->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), DKString("John")); 
                photo2->setData(photo2->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), DKString("Johnson")); 
                photo2->setData(photo2->dataId(DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), DKDate("2001-05-08")); 

                // Store the File
                photo2->setContentFromClientFile("SInsuranceModelICM_Photo_1ABC089.jpg");   // Load the file into memory.
                photo2->setMimeType("image/jpeg");                                          // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                photo2->add();

            cout << "Created Auto Photo for Vin #1HOME10R4KL987258." << endl;

            //-------------------------------------------------------------
            // Auto Photo 1 for Vin #1HOME10R4KL987258
            //-------------------------------------------------------------
            cout << "Creating Auto Photo for Vin #1HOME10R4KL987258..." << endl;

                // Create new DDO/XDO Object.
                DKImageICM* photo3a = (DKImageICM*) dsICM->createDDO("S_autoPhoto",DK_CM_ITEM);
                
                // Set Attributes
                photo3a->setData(photo3a->dataId(DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   DKString("4-852369")); 
                photo3a->setData(photo3a->dataId(DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  DKString("10285265410")); 
                photo3a->setData(photo3a->dataId(DK_CM_NAMESPACE_ATTR,"S_vin"),           DKString("9GOAL56H3LI552314")); 
                photo3a->setData(photo3a->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), DKString("Catherine")); 
                photo3a->setData(photo3a->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), DKString("Adjuster")); 
                photo3a->setData(photo3a->dataId(DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), DKDate("2001-08-02")); 

                // Store the File
                photo3a->setContentFromClientFile("SInsuranceModelICM_Photo_6usa676-1.tif"); // Load the file into memory.
                photo3a->setMimeType("image/tiff");                                          // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                photo3a->add();

            cout << "Created Auto Photo for Vin #1HOME10R4KL987258." << endl;

            //-------------------------------------------------------------
            // Auto Photo 2 for Vin #1HOME10R4KL987258
            //-------------------------------------------------------------
            cout << "Creating Auto Photo for Vin #1HOME10R4KL987258..." << endl;

                // Create new DDO/XDO Object.
                DKImageICM* photo3b = (DKImageICM*) dsICM->createDDO("S_autoPhoto",DK_CM_ITEM);
                
                // Set Attributes
                photo3b->setData(photo3b->dataId(DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   DKString("4-852369")); 
                photo3b->setData(photo3b->dataId(DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  DKString("10285265410")); 
                photo3b->setData(photo3b->dataId(DK_CM_NAMESPACE_ATTR,"S_vin"),           DKString("9GOAL56H3LI552314")); 
                photo3b->setData(photo3b->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), DKString("Catherine")); 
                photo3b->setData(photo3b->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), DKString("Adjuster")); 
                photo3b->setData(photo3b->dataId(DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), DKDate("2001-08-02")); 

                // Store the File
                photo3b->setContentFromClientFile("SInsuranceModelICM_Photo_6usa676-2.tif"); // Load the file into memory.
                photo3b->setMimeType("image/tiff");                                          // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                photo3b->add();

            cout << "Created Auto Photo for Vin #1HOME10R4KL987258." << endl;

            //-------------------------------------------------------------
            // Adjuster Report for Claim #6-987654
            //-------------------------------------------------------------
            // Features:  - Attribute Group
            //            - Reference Attribute
            //            - Folder
            cout << "Creating Adjuster Report for Claim #6-987654..." << endl;

                // Create new DDO/XDO Object.
                DKLobICM* adjusterReport1 = (DKLobICM*) dsICM->createDDO("S_adjustrReport",DK_CM_FOLDER);
                
                // Set Attributes (Including Attribute Groups & Reference Attribute)
                adjusterReport1->setData(adjusterReport1->dataId(DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   DKString("6-987654"));              // Attributes
                adjusterReport1->setData(adjusterReport1->dataId(DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  DKString("10452136412")); 
                adjusterReport1->setData(adjusterReport1->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), DKString("Michael")); 
                adjusterReport1->setData(adjusterReport1->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), DKString("Reed")); 
                adjusterReport1->setData(adjusterReport1->dataId(DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), DKDate("2001-02-01")); 
                adjusterReport1->setData(adjusterReport1->dataId(DK_CM_NAMESPACE_ATTR,"S_claimantFName"), DKString("Gregory")); 
                adjusterReport1->setData(adjusterReport1->dataId(DK_CM_NAMESPACE_ATTR,"S_claimantLName"), DKString("Twain")); 
                DKDDO* autoClaim1Copy = dsICM->createDDO(((DKPidICM*)autoClaim1->getPidObject())->pidString());  // Create copy to give to ddo's reference attribute.
                adjusterReport1->setData(adjusterReport1->dataId(DK_CM_NAMESPACE_ATTR,"S_autoClaim"),     (dkDataObjectBase*)(DKDDO*) autoClaim1Copy);  // Reference Attribute
                adjusterReport1->setData(adjusterReport1->dataId(DK_CM_NAMESPACE_ATTR,"S_carAccident.S_licensePlate"), DKString("4IBM134"));            // Attribute Group
                adjusterReport1->setData(adjusterReport1->dataId(DK_CM_NAMESPACE_ATTR,"S_carAccident.S_accidentDate"), DKDate("2001-04-20"));
            
                // Add Any Related Pictures to Folder   (Any number (0 - n) of items may be stored here.)
                DKFolder* dkFolder = (DKFolder*)(dkCollection*) adjusterReport1->getData(adjusterReport1->dataId(DK_CM_NAMESPACE_ATTR,DK_CM_DKFOLDER));
                dkFolder->addElement((dkDataObjectBase*)(DKDDO*)photo1a); 
                dkFolder->addElement((dkDataObjectBase*)(DKDDO*)photo1b); 

                // Store the File
                adjusterReport1->setContentFromClientFile("SInsuranceModelICM_Adjustment_6-987654.tif"); // Load the file into memory.
                adjusterReport1->setMimeType("image/tiff");                                              // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                adjusterReport1->add();

            cout << "Created Adjuster Report for Claim #6-987654." << endl;

            //-------------------------------------------------------------
            // Adjuster Report for Claim #5-654123
            //-------------------------------------------------------------
            cout << "Creating Adjuster Report for Claim #5-654123..." << endl;

                // Create new DDO/XDO Object.
                DKLobICM* adjusterReport2 = (DKLobICM*) dsICM->createDDO("S_adjustrReport",DK_CM_FOLDER);
                
                // Set Attributes (Including Attribute Groups & Reference Attribute)
                adjusterReport2->setData(adjusterReport2->dataId(DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   DKString("5-654123"));                    // Attributes
                adjusterReport2->setData(adjusterReport2->dataId(DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  DKString("10315975308")); 
                adjusterReport2->setData(adjusterReport2->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), DKString("John")); 
                adjusterReport2->setData(adjusterReport2->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), DKString("Johnson")); 
                adjusterReport2->setData(adjusterReport2->dataId(DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), DKDate("2001-05-08")); 
                adjusterReport2->setData(adjusterReport2->dataId(DK_CM_NAMESPACE_ATTR,"S_claimantFName"), DKString("Jessica")); 
                adjusterReport2->setData(adjusterReport2->dataId(DK_CM_NAMESPACE_ATTR,"S_claimantLName"), DKString("Fitsgerald")); 
                DKDDO* autoClaim2Copy = dsICM->createDDO(((DKPidICM*)autoClaim2->getPidObject())->pidString());  // Create copy to give to ddo's reference attribute.
                adjusterReport2->setData(adjusterReport2->dataId(DK_CM_NAMESPACE_ATTR,"S_autoClaim"),     (dkDataObjectBase*)(DKDDO*) autoClaim2Copy);  // Reference Attribute
                adjusterReport2->setData(adjusterReport2->dataId(DK_CM_NAMESPACE_ATTR,"S_carAccident.S_licensePlate"), DKString("1ABC089"));            // Attribute Group
                adjusterReport2->setData(adjusterReport2->dataId(DK_CM_NAMESPACE_ATTR,"S_carAccident.S_accidentDate"), DKDate("2001-05-04"));

                // Add Any Related Pictures to Folder   (Any number (0 - n) of items may be stored here.)
                dkFolder = (DKFolder*)(dkCollection*) adjusterReport2->getData(adjusterReport2->dataId(DK_CM_NAMESPACE_ATTR,DK_CM_DKFOLDER));
                dkFolder->addElement((dkDataObjectBase*)(DKDDO*)photo2); 

                // Store the File
                adjusterReport2->setContentFromClientFile("SInsuranceModelICM_Adjustment_5-654123.tif"); // Load the file into memory.
                adjusterReport2->setMimeType("image/tiff");                                              // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                adjusterReport2->add();

            cout << "Created Adjuster Report for Claim #5-654123." << endl;

            //-------------------------------------------------------------
            // Adjuster Report for Claim #4-852369
            //-------------------------------------------------------------
            cout << "Creating Adjuster Report for Claim #4-852369..." << endl;

                // Create new DDO/XDO Object.
                DKLobICM* adjusterReport3 = (DKLobICM*) dsICM->createDDO("S_adjustrReport",DK_CM_FOLDER);
                
                // Set Attributes (Including Attribute Groups & Reference Attribute)
                adjusterReport3->setData(adjusterReport3->dataId(DK_CM_NAMESPACE_ATTR,"S_claimNumber"),   DKString("4-852369"));                    // Attrib8utes
                adjusterReport3->setData(adjusterReport3->dataId(DK_CM_NAMESPACE_ATTR,"S_policyNumber"),  DKString("10285265410")); 
                adjusterReport3->setData(adjusterReport3->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterFName"), DKString("Catherine")); 
                adjusterReport3->setData(adjusterReport3->dataId(DK_CM_NAMESPACE_ATTR,"S_adjusterLName"), DKString("Green")); 
                adjusterReport3->setData(adjusterReport3->dataId(DK_CM_NAMESPACE_ATTR,"S_adjustmntDate"), DKDate("2001-08-02")); 
                adjusterReport3->setData(adjusterReport3->dataId(DK_CM_NAMESPACE_ATTR,"S_claimantFName"), DKString("George"));
                adjusterReport3->setData(adjusterReport3->dataId(DK_CM_NAMESPACE_ATTR,"S_claimantLName"), DKString("Tolstoy"));
                DKDDO* autoClaim3Copy = dsICM->createDDO(((DKPidICM*)autoClaim3->getPidObject())->pidString());  // Create copy to give to ddo's reference attribute.
                adjusterReport3->setData(adjusterReport3->dataId(DK_CM_NAMESPACE_ATTR,"S_autoClaim"),     (dkDataObjectBase*)(DKDDO*) autoClaim3Copy); // Reference Attribute
                adjusterReport3->setData(adjusterReport3->dataId(DK_CM_NAMESPACE_ATTR,"S_carAccident.S_licensePlate"), DKString("6USA676"));           // Attribute Group
                adjusterReport3->setData(adjusterReport3->dataId(DK_CM_NAMESPACE_ATTR,"S_carAccident.S_accidentDate"), DKDate("2001-07-29"));

                // Add Any Related Pictures to Folder   (Any number (0 - n) of items may be stored here.)
                dkFolder = (DKFolder*)(dkCollection*) adjusterReport3->getData(adjusterReport3->dataId(DK_CM_NAMESPACE_ATTR,DK_CM_DKFOLDER));
                dkFolder->addElement((dkDataObjectBase*)(DKDDO*)photo3a); 
                dkFolder->addElement((dkDataObjectBase*)(DKDDO*)photo3b); 

                // Store the File
                adjusterReport3->setContentFromClientFile("SInsuranceModelICM_Adjustment_4-852369.tif"); // Load the file into memory.
                adjusterReport3->setMimeType("image/tiff");                                              // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                adjusterReport3->add();

            cout << "Created Adjuster Report for Claim #4-852369." << endl;

            //-------------------------------------------------------------
            // Police Report #01202001-30
            //-------------------------------------------------------------
            // Features:  - Multivalue Attributes
            //            - Child Components
            //            - Inbound Links
            cout << "Creating Police Report #01202001-30..." << endl;

                // Create new DDO/XDO Object.
                DKLobICM* policeReport1 = (DKLobICM*) dsICM->createDDO("S_policeReport",DK_CM_DOCUMENT);
                
                // Set Attributes
                policeReport1->setData(policeReport1->dataId(DK_CM_NAMESPACE_ATTR,"S_reportNumber"), DKString("01202001-30"));
                policeReport1->setData(policeReport1->dataId(DK_CM_NAMESPACE_ATTR,"S_accidentDate"), DKDate("2001-01-20")); 
                
                // Set Multi-Value Attributes / Child Components
                DKDDO* victimInfo = dsICM->createChildDDO("S_policeReport", "S_victimInfo");
                victimInfo->setData(victimInfo->dataId(DK_CM_NAMESPACE_ATTR,"S_licensePlate"),  DKString("4IBM134"));
                victimInfo->setData(victimInfo->dataId(DK_CM_NAMESPACE_ATTR,"S_driverLicense"), DKString("B12345696"));

                // Get Child Component Collection
                short dataid = policeReport1->dataId(DK_CM_NAMESPACE_CHILD,"S_victimInfo");
                DKChildCollection* children = (DKChildCollection*)(dkCollection*) policeReport1->getData(dataid);
               
                // If none exists, create One
                if(children==NULL){
                    children = new DKChildCollection();
                    policeReport1->setData(policeReport1->dataId(DK_CM_NAMESPACE_CHILD, "S_victimInfo"), (dkCollection*) children);
                }
                
                // Add Child Components / Multivalue Attributes to Parent
                children->addElement((dkDataObjectBase*)(DKDDO*)victimInfo); 

                // Store the File
                policeReport1->setContentFromClientFile("SInsuranceModelICM_AccidenReport_0120200130.tif"); // Load the file into memory.
                policeReport1->setMimeType("image/tiff");                                                   // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                policeReport1->add();

                // Add Links Must be Added After DKDDO->add() is called.
                dsICM->checkOut(policeReport1); // Check Out Item.

                // Link Adjuster Report to Police Report (Inbound Link)
                DKDDO* adjusterReport1copy = dsICM->createDDO(((DKPidICM*)adjusterReport1->getPidObject())->pidString());
                DKDDO* policeReport1copy   = dsICM->createDDO(((DKPidICM*)policeReport1->getPidObject())->pidString());
                DKLink* dkLink = new DKLink("S_policeReports",adjusterReport1copy,policeReport1copy);
                SLinksICM::addLinkToDDO(dkLink,policeReport1);

                // Must Reset Resource Content In Order to Update a Resource Item
                policeReport1->setContentFromClientFile("SInsuranceModelICM_AccidenReport_0120200130.tif"); // Load the file into memory.
                policeReport1->setMimeType("image/tiff");                                                   // Describe the Content Type (MIME Type)

                // Save Changes In Datastore
                policeReport1->update(DK_CM_CHECKIN);
                
            cout << "Created Police Report #01202001-30." << endl;

            //-------------------------------------------------------------
            // Police Report #05062001-10
            //-------------------------------------------------------------
            cout << "Creating Police Report #05062001-10..." << endl;

                // Create new DDO/XDO Object.
                DKLobICM* policeReport2 = (DKLobICM*) dsICM->createDDO("S_policeReport",DK_CM_DOCUMENT);
                
                // Set Attributes
                policeReport2->setData(policeReport2->dataId(DK_CM_NAMESPACE_ATTR,"S_reportNumber"), DKString("05062001-10")); 
                policeReport2->setData(policeReport2->dataId(DK_CM_NAMESPACE_ATTR,"S_accidentDate"), DKDate("2001-05-04")); 
                
                // Set Multi-Value Attributes / Child Components
                DKDDO* victimInfo1 = dsICM->createChildDDO("S_policeReport", "S_victimInfo");
                victimInfo1->setData(victimInfo1->dataId(DK_CM_NAMESPACE_ATTR,"S_licensePlate"),  DKString("1ABC089")); 
                victimInfo1->setData(victimInfo1->dataId(DK_CM_NAMESPACE_ATTR,"S_driverLicense"), DKString("B98755678")); 
                DKDDO* victimInfo2 = dsICM->createChildDDO("S_policeReport", "S_victimInfo");
                victimInfo2->setData(victimInfo2->dataId(DK_CM_NAMESPACE_ATTR,"S_licensePlate"),  DKString("5JJJ064"));
                victimInfo2->setData(victimInfo2->dataId(DK_CM_NAMESPACE_ATTR,"S_driverLicense"), DKString("C99851876"));

                // Get Child Component Collection
                dataid   = policeReport2->dataId(DK_CM_NAMESPACE_CHILD,"S_victimInfo");
                children = (DKChildCollection*)(dkCollection*) policeReport2->getData(dataid);
               
                // If none exists, create One
                if(children==NULL){
                    children = new DKChildCollection();
                    policeReport2->setData(policeReport2->dataId(DK_CM_NAMESPACE_CHILD, "S_victimInfo"), (dkCollection*) children);
                }
                
                // Add Child Components / Multivalue Attributes to Parent
                children->addElement((dkDataObjectBase*)(DKDDO*)victimInfo1); 
                children->addElement((dkDataObjectBase*)(DKDDO*)victimInfo2); 

                // Store the File
                policeReport2->setContentFromClientFile("SInsuranceModelICM_AccidenReport_0506200110.tif"); // Load the file into memory.
                policeReport2->setMimeType("image/tiff");                                                   // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                policeReport2->add();

                // Add Links Must be Added After DKDDO->add() is called.
                dsICM->checkOut(policeReport2); // Check Out Item.

                // Link Adjuster Report to Police Report (Inbound Link)
                DKDDO* adjusterReport2copy = dsICM->createDDO(((DKPidICM*)adjusterReport2->getPidObject())->pidString());
                DKDDO* policeReport2copy   = dsICM->createDDO(((DKPidICM*)policeReport2->getPidObject())->pidString());
                dkLink = new DKLink("S_policeReports",adjusterReport2copy,policeReport2copy);
                SLinksICM::addLinkToDDO(dkLink,policeReport2);

                // Must Reset Resource Content In Order to Update a Resource Item
                policeReport2->setContentFromClientFile("SInsuranceModelICM_AccidenReport_0506200110.tif"); // Load the file into memory.
                policeReport2->setMimeType("image/tiff");                                                   // Describe the Content Type (MIME Type)
                
                // Save Changes In Datastore
                policeReport2->update(DK_CM_CHECKIN);
                
            cout << "Created Police Report #05062001-10." << endl;

            //-------------------------------------------------------------
            // Police Report #07292001-00
            //-------------------------------------------------------------
            cout << "Creating Police Report #07292001-00..." << endl;

                // Create new DDO/XDO Object.
                DKLobICM* policeReport3 = (DKLobICM*) dsICM->createDDO("S_policeReport",DK_CM_DOCUMENT);
                
                // Set Attributes
                policeReport3->setData(policeReport3->dataId(DK_CM_NAMESPACE_ATTR,"S_reportNumber"), DKString("07292001-00")); 
                policeReport3->setData(policeReport3->dataId(DK_CM_NAMESPACE_ATTR,"S_accidentDate"), DKDate("2001-07-29")); 
                
                // Set Multi-Value Attributes / Child Components
                victimInfo1 = dsICM->createChildDDO("S_policeReport", "S_victimInfo");
                victimInfo1->setData(victimInfo1->dataId(DK_CM_NAMESPACE_ATTR,"S_licensePlate"),  DKString("1ZZZ687"));
                victimInfo1->setData(victimInfo1->dataId(DK_CM_NAMESPACE_ATTR,"S_driverLicense"), DKString("G88468714"));
                victimInfo2 = dsICM->createChildDDO("S_policeReport", "S_victimInfo");
                victimInfo2->setData(victimInfo2->dataId(DK_CM_NAMESPACE_ATTR,"S_licensePlate"),  DKString("5SEG684"));
                victimInfo2->setData(victimInfo2->dataId(DK_CM_NAMESPACE_ATTR,"S_driverLicense"), DKString("G58166871"));
                DKDDO* victimInfo3 = dsICM->createChildDDO("S_policeReport", "S_victimInfo");
                victimInfo3->setData(victimInfo3->dataId(DK_CM_NAMESPACE_ATTR,"S_licensePlate"),  DKString("5CCC876"));
                victimInfo3->setData(victimInfo3->dataId(DK_CM_NAMESPACE_ATTR,"S_driverLicense"), DKString("A87163587"));

                // Get Child Component Collection
                dataid   = policeReport3->dataId(DK_CM_NAMESPACE_CHILD,"S_victimInfo");
                children = (DKChildCollection*)(dkCollection*) policeReport3->getData(dataid);
               
                // If none exists, create One
                if(children==NULL){
                    children = new DKChildCollection();
                    policeReport3->setData(policeReport3->dataId(DK_CM_NAMESPACE_CHILD, "S_victimInfo"), (dkCollection*) children);
                }
                
                // Add Child Components / Multivalue Attributes to Parent
                children->addElement((dkDataObjectBase*)(DKDDO*)victimInfo1); 
                children->addElement((dkDataObjectBase*)(DKDDO*)victimInfo2); 
                children->addElement((dkDataObjectBase*)(DKDDO*)victimInfo3); 

                // Link Adjuster Report to Police Report (Inbound Link)
                // No Adjustment Reports Exist for this report.

                // Store the File
                // No File Available for this report.

                // Save New Item In Datastore
                policeReport3->add();
                
            cout << "Created Police Report #07292001-00." << endl;

            //-------------------------------------------------------------
            // Police Report #07292001-73
            //-------------------------------------------------------------
            cout << "Creating Police Report #07292001-73..." << endl;

                // Create new DDO/XDO Object.
                DKLobICM* policeReport4 = (DKLobICM*) dsICM->createDDO("S_policeReport",DK_CM_DOCUMENT);
                
                // Set Attributes
                policeReport4->setData(policeReport4->dataId(DK_CM_NAMESPACE_ATTR,"S_reportNumber"), DKString("07292001-73")); 
                policeReport4->setData(policeReport4->dataId(DK_CM_NAMESPACE_ATTR,"S_accidentDate"), DKDate("2001-07-29")); 
                
                // Set Multi-Value Attributes / Child Components
                victimInfo1 = dsICM->createChildDDO("S_policeReport", "S_victimInfo");
                victimInfo1->setData(victimInfo1->dataId(DK_CM_NAMESPACE_ATTR,"S_licensePlate"),  DKString("6USA676"));
                victimInfo1->setData(victimInfo1->dataId(DK_CM_NAMESPACE_ATTR,"S_driverLicense"), DKString("B25878914"));

                // Get Child Component Collection
                dataid   = policeReport4->dataId(DK_CM_NAMESPACE_CHILD,"S_victimInfo");
                children = (DKChildCollection*)(dkCollection*) policeReport4->getData(dataid);
               
                // If none exists, create One
                if(children==NULL){
                    children = new DKChildCollection();
                    policeReport4->setData(policeReport4->dataId(DK_CM_NAMESPACE_CHILD, "S_victimInfo"), (dkCollection*) children);
                }
                
                // Add Child Components / Multivalue Attributes to Parent
                children->addElement((dkDataObjectBase*)(DKDDO*)victimInfo1); 

                // Store the File
                policeReport4->setContentFromClientFile("SInsuranceModelICM_AccidenReport_729200173.tif"); // Load the file into memory.
                policeReport4->setMimeType("image/tiff");                                                  // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                policeReport4->add();

                // Add Links Must be Added After DKDDO->add() is called.
                dsICM->checkOut(policeReport4); // Check Out Item.

                // Link Adjuster Report to Police Report (Inbound Link)
                DKDDO* adjusterReport3copy = dsICM->createDDO(((DKPidICM*)adjusterReport3->getPidObject())->pidString());
                DKDDO* policeReport4copy   = dsICM->createDDO(((DKPidICM*)policeReport4->getPidObject())->pidString());
                dkLink = new DKLink("S_policeReports",adjusterReport3copy,policeReport4copy);
                SLinksICM::addLinkToDDO(dkLink,policeReport4);

                // Must Reset Resource Content In Order to Update a Resource Item
                policeReport4->setContentFromClientFile("SInsuranceModelICM_AccidenReport_729200173.tif"); // Load the file into memory.
                policeReport4->setMimeType("image/tiff");                                                  // Describe the Content Type (MIME Type)
                
                // Save Changes In Datastore
                policeReport4->update(DK_CM_CHECKIN);
                
            cout << "Created Police Report #07292001-73." << endl;

            //-------------------------------------------------------------
            // Insurance Policy #10452136412
            //-------------------------------------------------------------
            // Features:  - Linking
            cout << "Creating Insurance Policy #10452136412..." << endl;

                // Create new DDO/XDO Object.
                DKLobICM* policy1 = (DKLobICM*) dsICM->createDDO("S_insurePolicy",DK_CM_DOCUMENT);
                
                // Set Attributes
                policy1->setData(policy1->dataId(DK_CM_NAMESPACE_ATTR,"S_street"),       DKString("258 Green Rd.")); 
                policy1->setData(policy1->dataId(DK_CM_NAMESPACE_ATTR,"S_city"),         DKString("Ocean"));
                policy1->setData(policy1->dataId(DK_CM_NAMESPACE_ATTR,"S_state"),        DKString("CA"));
                policy1->setData(policy1->dataId(DK_CM_NAMESPACE_ATTR,"S_zipCode"),      DKString("90060"));
                policy1->setData(policy1->dataId(DK_CM_NAMESPACE_ATTR,"S_policyNumber"), DKString("10452136412"));
                
                // Set Multi-Value Attributes / Child Components
                DKDDO* insured1 = dsICM->createChildDDO("S_insurePolicy", "S_insured");
                insured1->setData(insured1->dataId(DK_CM_NAMESPACE_ATTR,"S_firstName"), DKString("Gregory"));
                insured1->setData(insured1->dataId(DK_CM_NAMESPACE_ATTR,"S_lastName"),  DKString("Twain"));
                DKDDO* vin1 = dsICM->createChildDDO("S_insurePolicy", "S_vin");
                vin1->setData(vin1->dataId(DK_CM_NAMESPACE_ATTR,"S_vin"), DKString("1HOME10R4KL987258"));
                DKDDO* vin2 = dsICM->createChildDDO("S_insurePolicy", "S_vin");
                vin2->setData(vin2->dataId(DK_CM_NAMESPACE_ATTR,"S_vin"), DKString("3CASA55R4KL987500"));

                // Get Insured Child Component Collection
                dataid  = policy1->dataId(DK_CM_NAMESPACE_CHILD,"S_insured");
                DKChildCollection* insured = (DKChildCollection*)(dkCollection*) policy1->getData(dataid);
               
                // If none exists, create One
                if(insured==NULL){
                    insured = new DKChildCollection();
                    policy1->setData(policy1->dataId(DK_CM_NAMESPACE_CHILD, "S_insured"), (dkCollection*) insured);
                }

                // Get Vin Child Component Collection
                dataid  = policy1->dataId(DK_CM_NAMESPACE_CHILD,"S_vin");
                DKChildCollection* vin = (DKChildCollection*)(dkCollection*) policy1->getData(dataid);
               
                // If none exists, create One
                if(vin==NULL){
                    vin = new DKChildCollection();
                    policy1->setData(policy1->dataId(DK_CM_NAMESPACE_CHILD, "S_vin"), (dkCollection*) vin);
                }
                
                // Add Child Components / Multivalue Attributes to Parent
                insured->addElement((dkDataObjectBase*)(DKDDO*)insured1); 
                vin->addElement((dkDataObjectBase*)(DKDDO*)vin1); 
                vin->addElement((dkDataObjectBase*)(DKDDO*)vin2); 

                // Store the File
                policy1->setContentFromClientFile("SInsuranceModelICM_Policy_10452136412.tif"); // Load the file into memory.
                policy1->setMimeType("image/tiff");                                             // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                policy1->add();

                // Add Links Must be Added After DKDDO->add() is called.
                dsICM->checkOut(policy1); // Check Out Item.

                // Link Polocy to Police Reports, Auto Claim, & Adjustments (Outbound Links)
                DKDDO* policy1copy       = dsICM->createDDO(((DKPidICM*)policy1->getPidObject())->pidString());
                       policeReport1copy = dsICM->createDDO(((DKPidICM*)policeReport1->getPidObject())->pidString());
                dkLink = new DKLink("S_policeReports",policy1copy,policeReport1copy);
                SLinksICM::addLinkToDDO(dkLink,policy1);
                       policy1copy    = dsICM->createDDO(((DKPidICM*)policy1->getPidObject())->pidString());
                DKDDO* autoClaim1copy = dsICM->createDDO(((DKPidICM*)autoClaim1->getPidObject())->pidString());
                dkLink = new DKLink("S_autoClaims",policy1copy,autoClaim1copy);
                SLinksICM::addLinkToDDO(dkLink,policy1);
                       policy1copy         = dsICM->createDDO(((DKPidICM*)policy1->getPidObject())->pidString());
                       adjusterReport1copy = dsICM->createDDO(((DKPidICM*)adjusterReport1->getPidObject())->pidString());
                dkLink = new DKLink("S_adjustments",policy1copy,adjusterReport1copy);
                SLinksICM::addLinkToDDO(dkLink,policy1);

                // Must Reset Resource Content In Order to Update a Resource Item
                policy1->setContentFromClientFile("SInsuranceModelICM_Policy_10452136412.tif"); // Load the file into memory.
                policy1->setMimeType("image/tiff");                                             // Describe the Content Type (MIME Type)
                
                // Save Changes In Datastore
                policy1->update(DK_CM_CHECKIN);

            cout << "Created Insurance Policy #10452136412." << endl;

            //-------------------------------------------------------------
            // Insurance Policy #10315975308
            //-------------------------------------------------------------
            cout << "Creating Insurance Policy #10315975308..." << endl;

                // Create new DDO/XDO Object.
                DKLobICM* policy2 = (DKLobICM*) dsICM->createDDO("S_insurePolicy",DK_CM_DOCUMENT);
                
                // Set Attributes
                policy2->setData(policy2->dataId(DK_CM_NAMESPACE_ATTR,"S_street"),       DKString("456 First St."));
                policy2->setData(policy2->dataId(DK_CM_NAMESPACE_ATTR,"S_city"),         DKString("Santa"));
                policy2->setData(policy2->dataId(DK_CM_NAMESPACE_ATTR,"S_state"),        DKString("CA"));
                policy2->setData(policy2->dataId(DK_CM_NAMESPACE_ATTR,"S_zipCode"),      DKString("90123"));
                policy2->setData(policy2->dataId(DK_CM_NAMESPACE_ATTR,"S_policyNumber"), DKString("10315975308"));

                // Set Multi-Value Attributes / Child Components
                insured1 = dsICM->createChildDDO("S_insurePolicy", "S_insured");
                insured1->setData(insured1->dataId(DK_CM_NAMESPACE_ATTR,"S_firstName"), DKString("Alexand"));
                insured1->setData(insured1->dataId(DK_CM_NAMESPACE_ATTR,"S_lastName"),  DKString("Churchill"));
                DKDDO* insured2 = dsICM->createChildDDO("S_insurePolicy", "S_insured");
                insured2->setData(insured2->dataId(DK_CM_NAMESPACE_ATTR,"S_firstName"), DKString("Joan"));
                insured2->setData(insured2->dataId(DK_CM_NAMESPACE_ATTR,"S_lastName"),  DKString("Churchill"));
                DKDDO* insured3 = dsICM->createChildDDO("S_insurePolicy", "S_insured");
                insured3->setData(insured3->dataId(DK_CM_NAMESPACE_ATTR,"S_firstName"), DKString("Marty"));
                insured3->setData(insured3->dataId(DK_CM_NAMESPACE_ATTR,"S_lastName"),  DKString("Churchill"));
                vin1 = dsICM->createChildDDO("S_insurePolicy", "S_vin");
                vin1->setData(vin1->dataId(DK_CM_NAMESPACE_ATTR,"S_vin"), DKString("5CALL78T6PU743358"));

                // Get Insured Child Component Collection
                dataid  = policy2->dataId(DK_CM_NAMESPACE_CHILD,"S_insured");
                insured = (DKChildCollection*)(dkCollection*) policy2->getData(dataid);
               
                // If none exists, create One
                if(insured==NULL){
                    insured = new DKChildCollection();
                    policy2->setData(policy2->dataId(DK_CM_NAMESPACE_CHILD, "S_insured"), (dkCollection*) insured);
                }

                // Get Vin Child Component Collection
                dataid  = policy2->dataId(DK_CM_NAMESPACE_CHILD,"S_vin");
                vin     = (DKChildCollection*)(dkCollection*) policy2->getData(dataid);
               
                // If none exists, create One
                if(vin==NULL){
                    vin = new DKChildCollection();
                    policy2->setData(policy2->dataId(DK_CM_NAMESPACE_CHILD, "S_vin"), (dkCollection*) vin);
                }
                
                // Add Child Components / Multivalue Attributes to Parent
                insured->addElement((dkDataObjectBase*)(DKDDO*)insured1); 
                insured->addElement((dkDataObjectBase*)(DKDDO*)insured2); 
                insured->addElement((dkDataObjectBase*)(DKDDO*)insured3); 
                vin->addElement((dkDataObjectBase*)(DKDDO*)vin1); 

                // Store the File
                policy2->setContentFromClientFile("SInsuranceModelICM_Policy_10315975308.tif"); // Load the file into memory.
                policy2->setMimeType("image/tiff");                                             // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                policy2->add();

                // Add Links Must be Added After DKDDO->add() is called.
                dsICM->checkOut(policy2); // Check Out Item.

                // Link Polocy to Police Reports, Auto Claim, & Adjustments (Outbound Links)
                DKDDO* policy2copy       = dsICM->createDDO(((DKPidICM*)policy2->getPidObject())->pidString());
                       policeReport2copy = dsICM->createDDO(((DKPidICM*)policeReport2->getPidObject())->pidString());
                dkLink = new DKLink("S_policeReports",policy2copy,policeReport2copy);
                SLinksICM::addLinkToDDO(dkLink,policy2);
                       policy2copy    = dsICM->createDDO(((DKPidICM*)policy2->getPidObject())->pidString());
                DKDDO* autoClaim2copy = dsICM->createDDO(((DKPidICM*)autoClaim2->getPidObject())->pidString());
                dkLink = new DKLink("S_autoClaims",policy2copy,autoClaim2copy);
                SLinksICM::addLinkToDDO(dkLink,policy2);
                       policy2copy         = dsICM->createDDO(((DKPidICM*)policy2->getPidObject())->pidString());
                       adjusterReport2copy = dsICM->createDDO(((DKPidICM*)adjusterReport2->getPidObject())->pidString());
                dkLink = new DKLink("S_adjustments",policy2copy,adjusterReport2copy);
                SLinksICM::addLinkToDDO(dkLink,policy2);

                // Must Reset Resource Content In Order to Update a Resource Item
                policy2->setContentFromClientFile("SInsuranceModelICM_Policy_10315975308.tif"); // Load the file into memory.
                policy2->setMimeType("image/tiff");                                             // Describe the Content Type (MIME Type)
                
                // Save Changes In Datastore
                policy2->update(DK_CM_CHECKIN);

            cout << "Created Insurance Policy #10315975308." << endl;

            //-------------------------------------------------------------
            // Insurance Policy #10285265410
            //-------------------------------------------------------------
            cout << "Creating Insurance Policy #10285265410..." << endl; 

                // Create new DDO/XDO Object.
                DKLobICM* policy3 = (DKLobICM*) dsICM->createDDO("S_insurePolicy",DK_CM_DOCUMENT);
                
                // Set Attributes
                policy3->setData(policy3->dataId(DK_CM_NAMESPACE_ATTR,"S_street"),       DKString("333 Karenin"));
                policy3->setData(policy3->dataId(DK_CM_NAMESPACE_ATTR,"S_city"),         DKString("Marina"));
                policy3->setData(policy3->dataId(DK_CM_NAMESPACE_ATTR,"S_state"),        DKString("CA"));
                policy3->setData(policy3->dataId(DK_CM_NAMESPACE_ATTR,"S_zipCode"),      DKString("90546"));
                policy3->setData(policy3->dataId(DK_CM_NAMESPACE_ATTR,"S_policyNumber"), DKString("10285265410"));

                // Set Multi-Value Attributes / Child Components
                insured1 = dsICM->createChildDDO("S_insurePolicy", "S_insured");
                insured1->setData(insured1->dataId(DK_CM_NAMESPACE_ATTR,"S_firstName"), DKString("Gabriela"));
                insured1->setData(insured1->dataId(DK_CM_NAMESPACE_ATTR,"S_lastName"),  DKString("Neruda"));
                insured2 = dsICM->createChildDDO("S_insurePolicy", "S_insured");
                insured2->setData(insured2->dataId(DK_CM_NAMESPACE_ATTR,"S_firstName"), DKString("Francisco"));
                insured2->setData(insured2->dataId(DK_CM_NAMESPACE_ATTR,"S_lastName"),  DKString("Neruda"));
                vin1 = dsICM->createChildDDO("S_insurePolicy", "S_vin");
                vin1->setData(vin1->dataId(DK_CM_NAMESPACE_ATTR,"S_vin"), DKString("9GOAL56H3LI552314"));

                // Get Insured Child Component Collection
                dataid  = policy3->dataId(DK_CM_NAMESPACE_CHILD,"S_insured");
                insured = (DKChildCollection*)(dkCollection*) policy3->getData(dataid);
               
                // If none exists, create One
                if(insured==NULL){
                    insured = new DKChildCollection();
                    policy3->setData(policy3->dataId(DK_CM_NAMESPACE_CHILD, "S_insured"), (dkCollection*) insured);
                }

                // Get Vin Child Component Collection
                dataid  = policy3->dataId(DK_CM_NAMESPACE_CHILD,"S_vin");
                vin     = (DKChildCollection*)(dkCollection*) policy3->getData(dataid);
               
                // If none exists, create One
                if(vin==NULL){
                    vin = new DKChildCollection();
                    policy3->setData(policy3->dataId(DK_CM_NAMESPACE_CHILD, "S_vin"), (dkCollection*) vin);
                }
                
                // Add Child Components / Multivalue Attributes to Parent
                insured->addElement((dkDataObjectBase*)(DKDDO*)insured1); 
                insured->addElement((dkDataObjectBase*)(DKDDO*)insured2); 
                vin->addElement((dkDataObjectBase*)(DKDDO*)vin1); 

                // Store the File
                policy3->setContentFromClientFile("SInsuranceModelICM_Policy_10285265410.tif"); // Load the file into memory.
                policy3->setMimeType("image/tiff");                                             // Describe the Content Type (MIME Type)

                // Save New Item In Datastore
                policy3->add();

                // Add Links Must be Added After DKDDO->add() is called.
                dsICM->checkOut(policy3); // Check Out Item.

                // Link Polocy to Police Reports, Auto Claim, & Adjustments (Outbound Links)
                DKDDO* policy3copy       = dsICM->createDDO(((DKPidICM*)policy3->getPidObject())->pidString());
                       policeReport4copy = dsICM->createDDO(((DKPidICM*)policeReport4->getPidObject())->pidString());
                dkLink = new DKLink("S_policeReports",policy3copy,policeReport4copy);
                SLinksICM::addLinkToDDO(dkLink,policy3);
                       policy3copy    = dsICM->createDDO(((DKPidICM*)policy3->getPidObject())->pidString());
                DKDDO* autoClaim3copy = dsICM->createDDO(((DKPidICM*)autoClaim3->getPidObject())->pidString());
                dkLink = new DKLink("S_autoClaims",policy3copy,autoClaim3copy);
                SLinksICM::addLinkToDDO(dkLink,policy3);
                       policy3copy         = dsICM->createDDO(((DKPidICM*)policy3->getPidObject())->pidString());
                       adjusterReport3copy = dsICM->createDDO(((DKPidICM*)adjusterReport3->getPidObject())->pidString());
                dkLink = new DKLink("S_adjustments",policy3copy,adjusterReport3copy);
                SLinksICM::addLinkToDDO(dkLink,policy3);

                // Must Reset Resource Content In Order to Update a Resource Item
                policy3->setContentFromClientFile("SInsuranceModelICM_Policy_10285265410.tif"); // Load the file into memory.
                policy3->setMimeType("image/tiff");                                             // Describe the Content Type (MIME Type)
                
                // Save Changes In Datastore
                policy3->update(DK_CM_CHECKIN);

            cout << "Created Insurance Policy #10285265410." << endl;

            //-------------------------------------------------------------
            // Print Each
            //-------------------------------------------------------------
            // Using the SItemRetrievalICM's printDDO funciton, this will
            // demonstrate what each DDO looks like now.
            cout << "Printing Items..." << endl;

                SItemRetrievalICM::printDDO(autoClaim2);
                SItemRetrievalICM::printDDO(photo2);
                SItemRetrievalICM::printDDO(adjusterReport2);
                SItemRetrievalICM::printDDO(policeReport2);
                SItemRetrievalICM::printDDO(policy2);

            cout << "Printed Items." << endl;
            
            //-------------------------------------------------------------
            // Package up DDOs 
            //-------------------------------------------------------------
            dkCollection* ddoColl = new DKSequentialCollection();
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)autoClaim1);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)autoClaim2);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)autoClaim3);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)photo1a);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)photo1b);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)photo2);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)photo3a);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)photo3b);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)adjusterReport1);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)adjusterReport2);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)adjusterReport3);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)policeReport1);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)policeReport2);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)policeReport3);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)policeReport4);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)policy1);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)policy2);
            ddoColl->addElement((dkDataObjectBase*)(DKDDO*)policy3);
            
            return(ddoColl);
            
    }//end createInsuranceModelItems

   /**
    * Delete Objects in Memory for all objects returned by createInsuranceModelItems(...).
    * Deletes the DDOs and collection passed in.
    * @param ddoColl  dkCollection returned by SInsuranceModelItemCreationICM::createInsuranceModelItems().
    **/
    void SInsuranceModelItemCreationICM::deleteMemoryObjectsReturnedByCreateInsuranceModelItems(dkCollection* ddoColl){
    
        dkIterator* iter = ddoColl->createIterator();  // Create iterator to go through the DDOs returned.

        DKLobICM* autoClaim1      = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKLobICM* autoClaim2      = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKLobICM* autoClaim3      = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKImageICM* photo1a       = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKImageICM* photo1b       = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKImageICM* photo2        = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKImageICM* photo3a       = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKImageICM* photo3b       = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKLobICM* adjusterReport1 = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKLobICM* adjusterReport2 = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKLobICM* adjusterReport3 = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKLobICM* policeReport1   = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKLobICM* policeReport2   = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKLobICM* policeReport3   = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKLobICM* policeReport4   = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKLobICM* policy1         = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKLobICM* policy2         = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
        DKLobICM* policy3         = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();

        // Note, in this sample, photos were added to folder without making copies.
        // Folder will handle deletion of these.
        delete(autoClaim1);
        delete(autoClaim2);
        delete(autoClaim3);
        delete(adjusterReport1);
        delete(adjusterReport2);
        delete(adjusterReport3);
        delete(policeReport1);
        delete(policeReport2);
        delete(policeReport3);
        delete(policeReport4);
        delete(policy1);
        delete(policy2);
        delete(policy3);
     
        delete(iter);
        delete(ddoColl);
    }
