/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Updating Resource Items
    Updating Resource Items for Resource Content is almost identical to creating them, 
    except the "update()" method is used instead of "add()".  Updating Resource Items
    for non-resource Content may be found in SItemRetrievalICM.
    
    Resource Content may be updated using any of the methods outlined in 
    SResourceItemCreationICM or SResourceItemRetrievalICM, regardless of the 
    original method used.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKLobICM.hpp>
#include <DKTextICM.hpp>
#include <DKImageICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SResourceItemRetrievalICM.hpp"
#include "SResourceItemUpdateICM.hpp"

/************************************************************************************************
 *          FILENAME: SResourceItemUpdateICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Updating Resource Items.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Updating Resource Items
 *                    Updating, Storing Directly from File
 *                    Updating, Storing Directly from Stream
 *                    Updating Using Content in Local Memory
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SResourceItemUpdateICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                         - SAttributeDefinitionCreationICM
 *                         - SAttributeGroupDefCreationICM
 *                         - SReferenceAttrDefCreationICM
 *                         - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                         - SResourceMgrDefSetDefaultICM
 *                         - SSMSCollectionDefSetDefaultICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It 
 *                           must be edited first for the RM Confiuration information.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SResourceItemRetrieveICM.hpp/cpp
 *                    SResourceItemUpdateICM_launch.cpp
 *                    SResourceItemUpdateICM.hpp
 *                    SResourceItemICM_Document1.doc
 *                    SResourceItemICM_Document2.doc
 *                    SResourceItemICM_Text1.txt
 *                    SResourceItemICM_Text2.txt
 *                    SResourceItemICM_Image1.bmp
 *                    SResourceItemICM_Image2.bmp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SResourceItemUpdateICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                        << endl;
            cout << "  SResourceItemUpdateICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."         << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SResourceItemUpdateICM"         << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create Resource Object for Retrieval Demonstration
            //-------------------------------------------------------------
            // In order to demonstrate Updating of Resource Items, a few
            // Resource Items must first be created.
            cout << "Creating Resource Objects..." << endl;

                DKLobICM*    lob = (DKLobICM*)   dsICM->createDDO("S_lob",   DK_CM_DOCUMENT); // May be any semantic type.
                DKTextICM*  text = (DKTextICM*)  dsICM->createDDO("S_text",  DK_CM_ITEM);     // Resource Type determined by XDO Classification
                DKImageICM*image = (DKImageICM*) dsICM->createDDO("S_image", DK_CM_ITEM);     // in the Item Type Definition.

                // Set Mime Type / Type of Content
                lob->setMimeType("application/msword");
                text->setMimeType("text/plain");
                image->setMimeType("image/bmp");
                
                // Store Resource Content & Save Resource Item
                lob->add("SResourceItemICM_Document1.doc");  // Add it to the datastore,
                text->add("SResourceItemICM_Text1.txt");     // storing the content from
                image->add("SResourceItemICM_Picture1.bmp"); // the specified file.

            cout << "Created Resource Objects." << endl;

            //-------------------------------------------------------------
            // Checking Out / Locking for Update
            //-------------------------------------------------------------
            cout << "Checking Out / Locking Resource Item for Update..." << endl;

                dsICM->checkOut(lob);
                dsICM->checkOut(text);
                dsICM->checkOut(image);
        
            cout << "Checked Out / Locked Resource Item for Update." << endl;

            //-------------------------------------------------------------
            // Store Content Directly From a File
            //-------------------------------------------------------------
            // In one operation, update the item along with the specified conent.
            cout << "Updating Item, Storing Content Directly From File..." << endl;

                lob->update("SResourceItemICM_Document2.doc");  // Update it in the datastore,
                text->update("SResourceItemICM_Text2.txt");     // storing the content from
                image->update("SResourceItemICM_Picture2.bmp"); // the specified file.

            cout << "Updated Item, Storing Content Directly From File." << endl;

            //-------------------------------------------------------------
            // Create Streams
            //-------------------------------------------------------------
            // In order to store from a stream, we must first create a stream
            // for this sample to use.  The following will create streams from
            // the content files.
            cout << "Creating Streams..." << endl;

                // First, let's get a stream that we can use
                istream* iStream1 = new ifstream("SResourceItemICM_Document1.doc");
                istream* iStream2 = new ifstream("SResourceItemICM_Text1.txt");
                istream* iStream3 = new ifstream("SResourceItemICM_Picture1.bmp");
                // Determine the length of the file
                int stream1Length = SResourceItemRetrievalICM::getFileLength("SResourceItemICM_Document1.doc");;
                int stream2Length = SResourceItemRetrievalICM::getFileLength("SResourceItemICM_Text1.txt");
                int stream3Length = SResourceItemRetrievalICM::getFileLength("SResourceItemICM_Picture1.bmp");

            cout << "Created Streams." << endl;
            
            //-------------------------------------------------------------
            // Update Content Directly From a Stream
            //-------------------------------------------------------------
            // In one operation, store content from a stream while saving 
            // to the persistent datastore
            cout << "Updating Item, Storing Content Directly From a Stream..." << endl;

                lob->update(iStream1,stream1Length);  // Now add to datastore,
                text->update(iStream2,stream2Length); // storing the content from
                image->update(iStream3,stream3Length);// the specified stream

            cout << "Updated Item, Stored Content Directly From a Stream." << endl;

            //-------------------------------------------------------------
            // Load Content into Item's Local Memory.
            //-------------------------------------------------------------
            // Content may be loaded directly into the XDO's local memory.
            // When all modification is complete, it may then be updated in
            // the datastore.
            cout << "Loading Content into Item's Local Memory..." << endl;
    
                lob->setContentFromClientFile("SResourceItemICM_Document2.doc"); // Load the file into memory.
                text->setContentFromClientFile("SResourceItemICM_Text2.txt");
                image->setContentFromClientFile("SResourceItemICM_Picture2.bmp");

            cout << "Loaded Content into Item's Local Memory." << endl;

            //-------------------------------------------------------------
            // Update in Datastore With Content Already in Memory.
            //-------------------------------------------------------------
            // When all modification is complete, it may be updated in the
            // datastore.
            cout << "Updating in Datastore with Content Already in Memory..." << endl;

                lob->update();      // The content was already loaded into memory
                text->update();     // using the setContentFromClientFile(filename)
                image->update();    // function

            cout << "Updated in Datastore with Content Already in Memory." << endl;

            //-------------------------------------------------------------
            // Checking In / Unlocking Items
            //-------------------------------------------------------------
            cout << "Checking In / Unlocking Items..." << endl;

                dsICM->checkIn(lob);
                dsICM->checkIn(text);
                dsICM->checkIn(image);

                delete(lob);        // Free Memory since it is no longer needed in this sample.
                delete(text);
                delete(image);

            cout << "Checked In / Unlocked Items." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
