/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TdkCallback.hpp
 * Description    : Test utility methods used in FVT
 *
 * Change History
 * Date        Name  Description     
 * ----------  ----- -----------------------------------------------------
 * 2000/07/15  CSP   created
 *------------------------------------------------------------------------
 */

#include "TLogUtils.hpp"

TLogUtils::TLogUtils()
{
	init();
}
TLogUtils::TLogUtils(int argc, const char* args[])
{
	init();
	processCommandLineArgs(argc, args);
}
TLogUtils::TLogUtils(TLogUtils* aParent)
{
	init();
	setParent(aParent);
}
void TLogUtils::setParent(TLogUtils* aParent)
{
	if(aParent != NULL) {
		_fChild     = true;

		_strMsgBuff = aParent->getMsgBuff();
		_nCurIndent = aParent->getCurIndent();
		_strIndent  = aParent->getStrIndent();
		_strInFile  = aParent->getInFile();
		_strOutFile = aParent->getOutFile();
		_out        = aParent->getOutStream();
		_err        = aParent->getErrStream();
		_fDualOut   = aParent->isDualOut();
		_fAppend    = aParent->isAppend();
		_collVars   = aParent->getCollVars();
	}
}
void TLogUtils::init()
{
	nErrNumber  = 0;

	_strMsgBuff = NULL;
	_strIndent  = NULL;

	_strInFile  = "stdin";
	_strOutFile = "stdout";
	_out        = stdout;
	_err        = NULL;

	_fDualOut   = true;
	_fAppend    = false;
	_fChild     = false;

	_collVars   = new DKSequentialCollection();

	getStrIndent();
	getMsgBuff();
}
TLogUtils::~TLogUtils(){
	if(!_fChild) {
		if(_collVars != NULL) {
			deletedkCollection(_collVars);
			_collVars  = NULL;
		}
	
		if(_strMsgBuff != NULL) {
			delete _strMsgBuff;
			_strMsgBuff = NULL;
		}
	
		if(_strIndent != NULL) {
			delete _strIndent;
			_strIndent  = NULL;
		}

		if(_out != NULL && _out != stdout) {
			fclose(_out);
		}
		if(_err != NULL && _err != stderr) {
			fclose(_err);
		}
	}
} 
void TLogUtils::resetErrorInfo()
{
	strErrDesc  = "";
	strErrLoc   = "";
	strErrInput = "";
}
void TLogUtils::deletedkCollection(dkCollection* aCol)
{
	// when DKNVPair object is deleted it has no knowledge of it's
	// internal elements and it won't delete memory in them!
	// so we need to delete those elements
	if(aCol != NULL) {
		DKNVPair*   ptrNVPair = NULL;
		dkIterator* lIter     = aCol->createIterator();
		char*       lName;
		DKAny       lValue;

		for(; lIter->more();) {
			ptrNVPair = (DKNVPair*)(void*)(*(lIter->next()));
			if(ptrNVPair != NULL) {
				lName   = (char*)ptrNVPair->getName();
				lValue  = ptrNVPair->getValue();

				aCol->removeElementAt(*lIter);

				if(lName  != NULL) delete[] lName;
				//delete   lValue;
			}
		}
		delete aCol;
	}
}
void TLogUtils::logmsg(const DKString& aMsg, bool fAppend)
{
	// fAppend true means this line is continuation of previous line
	// so should not be prefixed with indent
	char* strAppend = (fAppend) ? "" : getStrIndent();
	fprintf(_out, "%s%s\n", strAppend, (const char*)aMsg);
	fflush(_out);

	if(_out != stdout && _fDualOut) {
		printf("%s%s\n", strAppend, (const char*)aMsg);
	}
}
void TLogUtils::logmsg(const DKString& aMsg1, const DKString& aMsg2, bool fAppend)
{
	logmsg((aMsg1+aMsg2), fAppend);
}
void TLogUtils::logerr(const DKString& anErr1, const DKString& anErr2)
{
	if(_err != NULL && _err != stderr) {
		fprintf(_err, "%s\n", (const char*)(anErr1 + anErr2));
		fflush(_err);
	}
}
void TLogUtils::logmsgBuff(bool fAppend)
{
	// fAppend means this line is continuation of previous line
	// so should not be prefixed with indent
	char* strAppend = (fAppend) ? "" : getStrIndent();
	fprintf(_out, "%s%s", strAppend, getMsgBuff());
	fflush(_out);

	if(_out != stdout && _fDualOut) {
		printf("%s%s", strAppend, getMsgBuff());
	}
}
void TLogUtils::logmsgDual(const DKString& aMsg1, const DKString& aMsg2)
{
	sprintf(getMsgBuff(), "%s%s", (const char*)aMsg1, (const char*)aMsg2);

	logmsg(getMsgBuff());
	logerr(getMsgBuff());
}

char* TLogUtils::getMsgBuff()
{
	if(_strMsgBuff == NULL) {
		_strMsgBuff = new char[_ID_STR_LEN];
		_strMsgBuff[0] = 0;
	}
	return _strMsgBuff;
}
void TLogUtils::logErrorInfo()
{
	logerr("");
	logerr("-------------------------------------------------------------------------");
	logerr("Test Case   : ", _strInFile);
	logerr("Error #     : ", ++nErrNumber);
	logerr("Test Date   : ", "cur date");
	logerr("Code Base   : ", strCodeBase);
	logerr("Description : ", strErrDesc);
	logerr("Where       : ", strErrLoc);
	logerr("Input       : ", strErrInput);
}
void TLogUtils::logmsgError(const DKString& anErrMsg)
{
	sprintf(getMsgBuff(), "2s\n", _ID_STR_ERROR, (const char*)anErrMsg);
	logmsgBuff();
	strErrDesc = anErrMsg;
	logErrorInfo();
}
void TLogUtils::logmsgDKException(DKException& aDKExp)
{
	// print diagnostic information to err file
	logErrorInfo();
	logerr("Got following Exception...");

	sprintf(getMsgBuff(), "%sDKException occurred\n", _ID_STR_ERROR);
	logmsgBuff();

	logmsgDual("    Name        : ", aDKExp.name());
	logmsgDual("    Error code  : ", DKString(aDKExp.errorCode()));
	logmsgDual("    Error state : ", DKString(aDKExp.errorState()));

	unsigned long nInd;

	unsigned long nTextCount = aDKExp.textCount();
	for(nInd=0; nInd < nTextCount; nInd++) {
		logmsgDual(aDKExp.text(nInd));
	}

	unsigned long nLocCount = aDKExp.locationCount();
	const DKExceptionLocation* ptrLoc;

	for(nInd=0; nInd < nLocCount; nInd++) {
		ptrLoc = aDKExp.locationAtIndex(nInd);
		logmsgDual(DKString(ptrLoc->fileName()) + " " +
				DKString(ptrLoc->functionName())    + "(" +
				DKString(ptrLoc->lineNumber())      + ")"  );
	}
}
void TLogUtils::logmsgdkResultSetCursor(dkResultSetCursor* aRSCursor)
{
	if(aRSCursor != NULL) {
		sprintf(getMsgBuff(), "# of Items     : %2d\n", aRSCursor->cardinality());
		logmsgBuff();
		DKDDO* lDKDDO = NULL;

		changeIndentTab(true);
		try {
			while(aRSCursor->isValid()) {
				DKDDO* lDKDDO = aRSCursor->fetchNext();
				if(lDKDDO != NULL) {
					sprintf(getMsgBuff(), "Item #         : %2d\n", aRSCursor->getPosition());
					logmsgBuff();
					logmsgDKDDO(lDKDDO);
				}
			}
		}
		catch(...) {
			changeIndentTab(false);
			throw;
		}
		changeIndentTab(false);
	}
}
void TLogUtils::logmsgDKDDO(DKDDO* aDKDDO) {
	if(aDKDDO != NULL) {
		DKDDO*            lDKDDO        = aDKDDO;
		DKString          lstrDataName  = "";
		DKAny             lObjAny;

		// DL has this property but not TS! don't know about QBIC
		ushort   shItemType  = lDKDDO->getProperty(lDKDDO->propertyId(DK_PROPERTY_ITEM_TYPE));
		DKString strItemName = getItemName(shItemType);
		logmsg("DDO Item is    : ", strItemName);

		int    lDDOCount   = lDKDDO->dataCount();
		logmsg("DDO pid string : ", ((DKPid*)lDKDDO->getPidObject())->pidString());
		sprintf(getMsgBuff(), "# of Items     : %2d\n",  lDDOCount);
		logmsgBuff();

		changeIndentTab(true);
		try {
			for(short i = 1; i <= lDDOCount; i++) {
				lstrDataName = lDKDDO->getDataName(i);
				lObjAny      = lDKDDO->getData(i);
	
				sprintf(getMsgBuff(), "% 3d. Name : %-20s", i, (char*)lstrDataName);
				logmsgBuff();
				logmsgObject(lObjAny);
			}
		}
		catch(...) {
			changeIndentTab(false);
			throw;
		}
		changeIndentTab(false);
	}
}
void TLogUtils::logmsgdkXDO(dkXDO* adkXDO)
{
	if(adkXDO != NULL) {
		DKPidXDO* lDKPidXDO = (DKPidXDO*)(adkXDO->getPid());
		logmsg("XDO pid string : ", lDKPidXDO->pidString(), true);

		/* if DKPidXDODL
		logmsg("XDO pid id              : " + lDKPidXDO->getId()            );
		logmsg("XDO pid id string       : " + lDKPidXDO->getIdString()      );
		logmsg("XDO pid id string count : " + lDKPidXDO->getIdStringCount() );
		logmsg("XDO pid primary id      : " + lDKPidXDO->getPrimaryId()     );
		logmsg("XDO pid item id         : " + lDKPidXDO->getIdString(0)     );
		logmsg("XDO pid part no         : " + lDKPidXDO->getIdString(1)     );
		logmsg("XDO pid rep type        : " + lDKPidXDO->getIdString(2)     );
		*/
	}
}
void TLogUtils::logmsgdkCollection(dkCollection* adkCollection)
{
	if(adkCollection != NULL) {
		int              lCount    = adkCollection->cardinality();
		dkIterator*      lIter     = adkCollection->createIterator();
		DKAny*           ptrAny   = NULL;
		sprintf(getMsgBuff(), " Count : %-3d\n", lCount);
		logmsgBuff(true);
	
		changeIndentTab(true);
		try {
			for(short nInd = 1; lIter->more(); nInd++) {
				ptrAny   = lIter->next();
	
				switch(ptrAny->typeCode()) {
				case DKAny::tc_string     :
				case DKAny::tc_collection :
				case DKAny::tc_dobase     :
					// DKParts collection has tc_voidpointer and don't know how to
					// print values corresponding to that
					sprintf(getMsgBuff(), "% 3d.", nInd);
					logmsgBuff();
					break;
				}
	
				logmsgObject(*ptrAny);
			}
		}
		catch(...) {
			changeIndentTab(false);
			throw;
		}
		changeIndentTab(false);
	}
}
void TLogUtils::logmsgObject(DKAny& anObject)
{
	DKAny             lObjAny = anObject;
	dkDataObjectBase* lDOBase = NULL;

	switch(lObjAny.typeCode()) {
	case DKAny::tc_string :
		// DKString
		sprintf(getMsgBuff(), " Value : %s\n", (char*)((DKString)lObjAny));
		logmsgBuff(true);
		break;

	case DKAny::tc_collection :
		// dkCollection
		logmsgdkCollection((dkCollection*)lObjAny);
		break;

	case DKAny::tc_dobase :
		lDOBase = (dkDataObjectBase*)lObjAny;

		// DKDDO
		if(     lDOBase->protocol() == DK_PDDO) {
			logmsgDKDDO((DKDDO*)lDOBase);
		}
		// dkXDO
		else if(lDOBase->protocol() == DK_XDO) {
			logmsgdkXDO((dkXDO*)lDOBase);
		}
		break;

	case DKAny::tc_null:
		logmsg("", true);
		break;

	default:
		break;
	}
}
const DKString TLogUtils::getItemName(const ushort shItemType)
{
	DKString lName = "Unknown";

	switch (shItemType) {
		case DK_DOCUMENT :
			lName = "Document";
			break;
		case DK_FOLDER :
			lName = "Folder";
			break;
		case DK_WORKBASKET :
			lName = "Workbasket";
			break;
		case DK_WORKFLOW :
			lName = "Workflow";
			break;
		default:
			lName = "Unknown";
			break;
	}
	return lName;
}
void TLogUtils::changeIndentTab(bool fIncrement)
{
	if(_nCurIndent >= 0) {
		getStrIndent()[((_nCurIndent * 4))] = ' ';
	}

	if(fIncrement) {
		_nCurIndent++;
	}
	else {
		_nCurIndent--;
	}

	if(_nCurIndent >= 0) {
		_strIndent[((_nCurIndent * 4))] = 0;
	}
}
int TLogUtils::getCurIndent()
{
	return _nCurIndent;
}
void TLogUtils::setCurIndent(int nIndent)
{
	_nCurIndent = nIndent;
}
char* TLogUtils::getStrIndent()
{
	if(_strIndent == NULL) {
		_strIndent = new char[_ID_STR_LEN];
		memset(_strIndent, ' ', _ID_STR_LEN);
		_nCurIndent   = 0;
		_strIndent[0] = 0;
	}

	return _strIndent;
}
FILE* TLogUtils::getOutStream()
{
	return _out;
}
FILE* TLogUtils::getErrStream()
{
	return _err;
}
void TLogUtils::enableDualOut(bool fEnable)
{
	_fDualOut = fEnable;
}
bool TLogUtils::isDualOut()
{
	return(_fDualOut);
}
void TLogUtils::enableAppend(bool fEnable)
{
	_fAppend = fEnable;
}
bool TLogUtils::isAppend()
{
	return(_fAppend);
}
void TLogUtils::setOutStream(FILE* anOutStream)
{
	if(getOutStream() != NULL && getOutStream() != stdout) {
		fclose(getOutStream());
	}

	_out = anOutStream;
	if(_out == NULL) {
		_out        = stdout;
		_strOutFile = "stdout";
	}
}
const char* TLogUtils::getOutFile()
{
	return _strOutFile;
}
const char* TLogUtils::getInFile()
{
	return _strInFile;
}
dkCollection* TLogUtils::getCollVars()
{
	return _collVars;
}
void TLogUtils::setOutFile(const DKString& aFile)
{
	const char* lFile = (const char*)aFile;
	if(_strOutFile == NULL || (lFile != NULL && !equalsIgnoreCase(_strOutFile, lFile))) {
		if(!equalsIgnoreCase(lFile, "stdout")) {
			char* strAppend = (isAppend()) ? "a" : "w";
			FILE* lOut      = fopen(lFile, strAppend);
			_strOutFile     = lFile;

			setOutStream(lOut);

			char strErrFile[64];
			sprintf(strErrFile, "%s%s", lFile, _ID_ERRFILE_EXT);
			if(_err != NULL && _err != stderr) {
				fclose(_err);
			}
			_err = fopen(strErrFile, strAppend);
		}


		if(_strInFile != NULL) {
			sprintf(getMsgBuff(), "Reading input from  : %s\n", _strInFile);
			logmsgBuff();
		}
		if(getOutStream() != stdout) {
			printf("Writing output to   : %s\n", _strOutFile );
			time_t lCurTime = time(NULL);
			logmsg(ctime(&lCurTime));
		}
	}
}
void TLogUtils::processCommand(const DKString& aCmd)
{
	dkCollection* ldkCollection = getTokens(aCmd);
	dkIterator*   lIter         = ldkCollection->createIterator();
	int           nArgc         = ldkCollection->cardinality();
	char**        args          = new char*[nArgc];
	DKAny         lObjAny;

	try {
		for(short nInd = 0; lIter->more(); nInd++) {
			lObjAny = *(lIter->next());
			args[nInd] = (char*)((DKString)lObjAny);
		}
	
		if(     equalsIgnoreCase(args[0], "help")) {
			printCommandHelp();
		}
		else if((args[0][0] == '~') && (strchr(aCmd, '=') != NULL)) {
			// its a ~variableName = _StringValue_ form
			DKString lstrRHS = getRHS(aCmd);
			setVariableValue(args[0], lstrRHS);
		}
		else if(equalsIgnoreCase(args[0], "listVariables")) {
			listVariables();
		}
		else {
			logmsgError(DKString("Unknown command : ") + DKString(args[0][0]));
		}
	}
	catch(DKException aDKExp) {
		logmsgDKException(aDKExp);
	}
	catch(char* strExp) {
		logmsgError(strExp);
	}
	catch(...) {
		delete[] args;
		delete   ldkCollection;
		logmsgError("Got unknown exception");

		throw;
	}

	delete[] args;
	delete   ldkCollection;
}
void TLogUtils::readFromStdIn()
{
	char lstrCmd[_ID_STR_LEN];

	for(;;) {
		logmsg("-->Please enter command:");

		gets(lstrCmd);

		if(equalsIgnoreCase(lstrCmd, "exit") ||
		   equalsIgnoreCase(lstrCmd, "quit") ||
		   equalsIgnoreCase(lstrCmd, "q")    ||
		   equalsIgnoreCase(lstrCmd, "e")) {
			 break;
		}
		processCommand(lstrCmd);
	}
}
void TLogUtils::readInputFile(const DKString& anInFile)
{
	if(equalsIgnoreCase(anInFile, "stdin")) {
		readFromStdIn();
	}
	else {
		const char* lInFile = (const char*)anInFile;
		char*       lstrCmd = "";

		_strInFile = _ID_DEFAULT_INFILE;
	
		if(lInFile != NULL && strlen(lInFile) > 0) {
			_strInFile  = lInFile;
		}
		sprintf(getMsgBuff(), "-->Input file  : %s\n", _strInFile);
		logmsgBuff();

		FILE* fpIn = fopen(_strInFile, "r");


		if(fpIn != NULL) {

			char   lstrCmd[_ID_STR_LEN];
			size_t nPos    = -1;
			DKString lstrCmdTrimmed;

			while((lstrCmd[++nPos] = fgetc(fpIn)) != EOF) {
				if(lstrCmd[nPos] == 0 || lstrCmd[nPos] == '\n') {
					lstrCmd[nPos] = 0;

					lstrCmdTrimmed = trim(lstrCmd);
					if(isCommand(lstrCmdTrimmed)) {
						takeCommand(lstrCmdTrimmed);
					}
					nPos = -1;
				}
			}
			fclose(fpIn);
		}
		else {
			sprintf(getMsgBuff(), "%sopening input file : %s\n", _ID_STR_ERROR, _strInFile);
			logmsgDual(getMsgBuff());
		}
	}
}
void TLogUtils::printOtherHelp()
{
	// override this method if you want to print some extra help for
	// your derived class.
}
void TLogUtils::printUsage()
{
	char* lstrMsg = 
	"usage: exeName [-options] [[Command], [Command]...]\n\n"
	"where options include:\n"
	"  -i{InputFile}     Read input from specified file\n"
	"  -o{OutputFile}    write output to specified file and stdout\n"
	"  -O{OutputFile}    same as -o but donot write to stdout\n"
	"  -a{OutputFile}    same as -o but append to OutputFile\n"
	"  -A{OutputFile}    same as -O but append to OutputFile\n"
	"  -v{Version#}      Version/CodeBase information\n"
	"  -h                Print this help\n"
	"  -?                Print this help\n\n"
	"  e.g. exeName -iTest1 -oTest1.log \n"
	"       which will use Test1 as input file and Test1.log as out put file\n";

	logmsg(lstrMsg);

	printCommandHelp();
	printOtherHelp();
}
const DKString TLogUtils::getAvailableCommands()
{
	char* rc = 
			"    TLogUtils-----------------------------------------------------\n\t" 
			"help                          \n\t" 
			"exit/quit                     \n\t"
			"~VariableName = _StringValue_ \n\t"
			"listVariables                 \n"
			"";
	return rc;
}
void TLogUtils::printCommandHelp()
{
	logmsg("Available commands:");
	DKString lstrCmds = getAvailableCommands();
	logmsg(lstrCmds);
}

void TLogUtils::processCommandLineArgs(int argc, const char* args[])
{
	const char*  strCurArg      = "";
	bool   fCmdSpecifiedOnCmdln = false;
	int    nArgLen              = 0;

	for(int i=1; i<argc; i++ ) {
		strCurArg     = args[i];
		nArgLen       = strlen(strCurArg);
		if(nArgLen <= 0) {
			continue;
		}

		char chFirst  = strCurArg[0];

		if(nArgLen >= 2 && chFirst == '-' || chFirst == '/') {
			char chSecond = toupper(strCurArg[1]);
			const char* strArgValue = "";

			if(nArgLen > 2) {
				strArgValue = strCurArg + 2;
			}

			if(chSecond == 'H' || chSecond == '?') {
				printUsage();
			}
			else if(strlen(strArgValue) > 0) {
				if(chSecond == 'I') {
					_strInFile  = strArgValue;
				}
				else if(chSecond == 'O' ) {
					enableDualOut(strCurArg[1] == 'o');
					enableAppend(false);
					setOutFile(strArgValue);
				}
				else if(chSecond == 'A' ) {
					enableDualOut(strCurArg[1] == 'a');
					enableAppend(true);
					setOutFile(strArgValue);
				}
				else if(chSecond == 'V' ) {
					strCodeBase = stringValue(strArgValue);
				}
			}
		}
		else {
			fCmdSpecifiedOnCmdln = true;
			takeCommand(strCurArg);
		}
	}
	
	// if only infile is specified then set out file based on
	// infile name
	if(strlen(_strInFile) > 0) {
		if(!equalsIgnoreCase(_strInFile, "stdin") && 
				(_strOutFile == NULL || strlen(_strOutFile) <= 0 ||
				equalsIgnoreCase(_strOutFile, "stdout"))) {

			char strOutFile[64];
			sprintf(strOutFile, "%s%s", _strInFile, _ID_OUTFILE_EXT);
			setOutFile(strOutFile);
		}
		readInputFile(_strInFile);
	}
	else if(!fCmdSpecifiedOnCmdln) {
		readFromStdIn();
	}
}
void TLogUtils::takeCommand(const DKString& aCmd)
{
	const char* lCmd = (const char*)aCmd;

	if(lCmd != NULL) {
		DKString lstrCmd = stringValue(aCmd);
		sprintf(getMsgBuff(), "-->Processing  : %s\n", lCmd);
		logmsgBuff();

		if(lCmd[0] != '~' && !equalsIgnoreCase(lCmd, lstrCmd)) {
			sprintf(getMsgBuff(), "                 %s\n", (const char*)lstrCmd);
			logmsgBuff();
		}

		strErrLoc = aCmd;
		processCommand(aCmd);
	}
}
bool TLogUtils::isCommand(const DKString& aString)
{
	bool        rc      = false;
	const char* lString = (const char*) aString;

	if(lString != NULL) {
		// if line is a blank line or starts with ; or # or // then 
		// then it's not a command line
		rc = (strlen(lString) > 0  &&
			!(lString[0] == '#')     &&
			!(strlen(lString) >= 2 && lString[0] == '/'  && lString[1] == '/'));
	}

	return rc;
}
bool TLogUtils::equalsIgnoreCase(const DKString& aS1, const DKString& aS2)
{
	const char* lS1 = (const char*)aS1;
	const char* lS2 = (const char*)aS2;

	bool   rc = (lS1 != NULL && lS2 != NULL);
	if(rc) {
		size_t nLenS1 = strlen(lS1);
		size_t nLenS2 = strlen(lS2);

		if(rc = (nLenS1 == nLenS2)) {
			for(size_t nInd=0; nInd < nLenS1; nInd++) {
				if(toupper(lS1[nInd]) != toupper(lS2[nInd])) {
					rc = false;
					break;
				}
			}
		}
	}
	return rc;
}


dkCollection* TLogUtils::getTokens(const DKString& aCmdLine, int numSkip)
{
	dkCollection* ldkCollection = new DKSequentialCollection();
	const char*   lCmdLine      = (const char*)aCmdLine;

	if(lCmdLine != NULL && isCommand(lCmdLine)) {
		const char*  strCmd       = lCmdLine;
		int          nTokenS      = -1;
		unsigned int nTokenE      = -1;
		char         chCur        = ' ';
		char         chPrev       = ' ';
		bool         fStrStart    = false;
		char         strCurToken[_ID_STR_LEN] = "";
		int          nDelCount    = numSkip;        // to skip first n arguments
		DKString     lstrTmp      = NULL;

		for(nTokenE=0; nTokenE<strlen(strCmd); nTokenE++) {
			chCur = strCmd[nTokenE];
			if(chCur == '/' && chPrev == '/') {
				// skip C++ type comment's
				if(nTokenE > 1) {
					nTokenE--;
				}
				break;
			}
			else if(strchr(_ID_STR_COMMAND_DELIMITER, chCur) == NULL) {
				if(chCur == '"' && chPrev != '\\') {

					int nIncludeThisChar = (fStrStart) ? 1 : 0;

					strncpy(strCurToken, (strCmd + nTokenS), (nTokenE - nTokenS + nIncludeThisChar));
					strCurToken[nTokenE - nTokenS + nIncludeThisChar] = 0;
					nTokenS   = -1;

					if(fStrStart) {
						fStrStart = false;
					}
					else {
						fStrStart = true;
						nTokenS   = nTokenE;
					}
				}
				else {
					if(nTokenS < 0) {
						nTokenS = nTokenE;
					}
				}
			}
			else {
				if(nTokenS >= 0 && !fStrStart) {
					strncpy(strCurToken, (strCmd + nTokenS), (nTokenE - nTokenS));
					strCurToken[nTokenE - nTokenS] = 0;
					nTokenS = -1;
				}
			}
	
			if(strlen(strCurToken) > 0) {
				if(nDelCount <= 0) {
					lstrTmp = stringValue(strCurToken);
					ldkCollection->addElement(DKAny(lstrTmp));
				}
				else {
					nDelCount--;
				}
				strCurToken[0] = 0;
			}
			chPrev = chCur;
		}
		if(nTokenS >= 0 && nDelCount <= 0 && nTokenS < (int)nTokenE) {
			// get the last token
			strncpy(strCurToken, (strCmd + nTokenS), (nTokenE - nTokenS));
			strCurToken[nTokenE - nTokenS] = 0;

			lstrTmp = stringValue(strCurToken);
			ldkCollection->addElement(DKAny(lstrTmp));
		}

		/*
		dkIterator* lIter   = ldkCollection->createIterator();
		DKAny lObjAny;
		for(short nInd = 1; lIter->more(); nInd++) {
			lObjAny = *(lIter->next());
			printf("%3d. ->%s<-\n", nInd, (char*)((DKString)lObjAny));
		}
		*/
	}
	return ldkCollection;
}
const DKString TLogUtils::stringValue(const DKString& aString)
{
	// take out backslashes and double-quote marks from string
	// literals and replace variable names with string values
	// except if variable name starts at column 0 
	// e.g. in ~var1 = ~var2 OR ~var3 we replace only ~var2 and ~var3
	// values but not ~var1 value.

	const char* lString = (const char*)aString;
	char        chPrev = ' ';
	char        chCur;
	int         nStart;
	char        lVarName[_ID_STR_LEN];
	bool        fEOS = false;
	char        rc[_ID_STR_LEN];

	memset(rc, 0, _ID_STR_LEN);

	if(lString != NULL) {
		int  nLen          = strlen(lString);
		bool fHasEqualOptr = (strchr(lString, '=') != NULL);

		for(int nInd=0; nInd<nLen; nInd++) {
			chCur = lString[nInd];
			// skip if variable starts at column 0
			if((!fHasEqualOptr || nInd != 0) && (chCur == '~') && (chPrev != '\\')) {
				//it's a script variable
				for(nStart=nInd; nInd<nLen; nInd++) {
					chCur = lString[nInd];
					if(strchr("() \t,/;\\", chCur) != NULL) {
						break;
					}
				}
				fEOS = (nInd>=nLen);
				strncpy(lVarName, (lString + nStart), (nInd - nStart));
				lVarName[nInd - nStart] = 0;

				DKAny  lObjAny   = getVariableValue(lVarName);
				char*  lVarValue = lVarName;

				// attempt to replace variable name with string value
				if(lObjAny.typeCode() == DKAny::tc_string) {
					lVarValue  = (char*)((DKString)lObjAny);
				}
				strcpy((rc+strlen(rc)), lVarValue);
			}

			if(!fEOS && ((chCur != '\\' && chCur != '"') || chPrev == '\\')) {
				rc[strlen(rc)] = chCur;
			}
			chPrev = chCur;
		}
	}
	char* lnewString = new char[strlen(rc) + 1];
	strcpy(lnewString, rc);

	return lnewString;
}
const DKString TLogUtils::getRHS(const DKString& aCmd)
{
	const char* lCmd     = (const char*)aCmd;
	DKString lstrLine = "";

	if(lCmd != NULL && strlen(lCmd) > 0) {
		char* lRHS = strchr(lCmd, '=');
		if( lRHS != NULL) {
			// take out comment
			char* lComment = strstr(lRHS, "//");
			if(lComment != NULL) {
				lComment[0] = 0;
			}
		}

		if(lRHS != NULL && strlen(lRHS) > 0) {
			lstrLine = stringValue(trim(lRHS + 1));
		}
		else {
			logmsgError("in script syntax.  \n\tIt should be of form ~variableName = _StringValue_");
		}
	}
	return lstrLine;
}

void TLogUtils::listVariables()
{
	dkIterator* lIter     = _collVars->createIterator();
	DKNVPair*   ptrNVPair = NULL;
	char*       lName     = NULL;
	char*       lValue    = NULL;

	for(; lIter->more();) {
		ptrNVPair = (DKNVPair*)(void*)(*(lIter->next()));

		if(ptrNVPair != NULL) {
			lName         = (char*)ptrNVPair->getName();
			DKAny lObjAny = ptrNVPair->getValue();
			if(lObjAny.typeCode() == DKAny::tc_string) {
				lValue  = (char*)((DKString)lObjAny);
			}
			else {
				lValue = "???";
			}

			sprintf(getMsgBuff(), "%-15s = %s\n", lName, lValue);
			logmsgBuff();
		}
	}
}

char* TLogUtils::newCopy(const DKString& aStr, int nSize)
{
	char*       lNewStr = NULL;
	const char* lStr    = (const char*)aStr;

	if(lStr != NULL) {
		int nLen = strlen(lStr) + 1;

		lNewStr = (nSize > nLen) ? new char[nSize] : new char[nLen];
		strcpy(lNewStr, lStr);
	}
	return lNewStr;
}
void TLogUtils::setVariableValue(const DKString& aName, const DKString& aValue)
{
	setVariableValue(aName, DKAny(DKString(aValue)));
}
void TLogUtils::setVariableValue(const DKString& aName, const DKAny& aValue)
{
	if((const char*)aName != NULL) {
		DKNVPair* ptrNVPair = getValuePairFor(aName);
		DKAny*    ptrValue  = new DKAny(aValue);
		if(ptrNVPair == NULL) {
			ptrNVPair = new DKNVPair(newCopy(aName), *ptrValue);
			_collVars->addElement(ptrNVPair);
			// ***
			//delete ptrNVPair;
		}
		else {
			ptrNVPair->setValue(*ptrValue);
		}
	}
}
DKAny TLogUtils::getVariableValue(const DKString& aName)
{
	DKNVPair* ptrNVPair = getValuePairFor(aName);
	DKAny     rc        = (ptrNVPair != NULL) ? (ptrNVPair->getValue()) : DKAny(NULL);
	return rc;
}
DKNVPair* TLogUtils::getValuePairFor(const DKString& aName)
{
	DKNVPair*   ptrNVPair = NULL;
	dkIterator* lIter     = _collVars->createIterator();

	for(; ptrNVPair == NULL && lIter->more();) {
		ptrNVPair = (DKNVPair*)(void*)(*(lIter->next()));

		DKString strName = ptrNVPair->getName();
		
		if(!equalsIgnoreCase((char*)ptrNVPair->getName(), aName)) {
			ptrNVPair = NULL;
		}
	}

	return ptrNVPair;
}
const DKString TLogUtils::trim(const DKString& aString)
{
	// trim leading and trailing balnks
	char        rc[_ID_STR_LEN];
	const char* lString = (const char*)aString;
	int         nS      = 0;
	int         nE      = -1;

	if(lString != NULL) {
		int nLen = strlen(lString);
		if(nLen > _ID_STR_LEN) {
			nLen = _ID_STR_LEN;
		}

		for(nS = 0; lString[nS] == ' ' && nS < nLen; nS++);
		for(nE = nLen - 1; lString[nE] == ' ' && nE >= nS; nE--);
		if(nS <= nE) {
			strncpy(rc, (lString+nS), (nE-nS+1));
		}
	}
	rc[nE - nS + 1] = 0;
	//printf("source->%s<- \nDest  ->%s<-", lString, rc);
	
	//char* lnewString = new char[strlen(rc) + 1];
	//strcpy(lnewString, rc);

	return rc;
}
char* TLogUtils::toUpperCase(char* aString)
{
	if(aString != NULL) {
		int nLen = strlen(aString);
		for(int nInd=0; nInd < nLen; nInd++) {
			aString[nInd] = toupper(aString[nInd]);
		}
	}
	return aString;
}

