/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;
import java.util.*;
import java.io.*;

/**
 * Imports a Document. import a document is not supported by DB2, JDBC, DataJointer and OD.  
 * This handles setting the attributes as well as the content.
 * For ICM, child component attributes can be specified (only a single child component
 * of a particular component type can be specified).
 */
public class TImport {

    public static void main(String[] args) throws Exception {
        if (args.length < 6) {
            System.out.println(
                "Imports a document to the server. \n"
                    + "Syntax: \n"
                    + "  java TImport <dstype> <server> <userid> <password> <entity> <filename> <attrs> \n"
                    + "where: \n"
                    + "  <dstype> is the type of server (Fed, ICM, DL, OD, DB2) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n"
                    + "  <entity> the type of entity (item type, table) to create the item as \n"
                    + "  <filename> the name of the file containing the document to be imported. \n"
                    + "  <attrs>  the attributes for the newly created item.  These are \n"
                    + "           in the form of <attrname>=<value>. \n"
                    + "Special values: \n"
                    + "  timestamp -- can be used as a value and is substituted with real timestamp. \n"
                    + "Examples: \n"
                    + "  java TImport ICM icmnlsdb icmadmin password Book login.gif Title='A Small GIF' BkAuthor.LastName='Anonymous' \n"
                    + "  java TImport DL libsrvrn frnadmin password NOINDEX c:\\temp\\login.gif Source='gif' Userid='user1' 'Time Stamp'=timestamp \n");
            System.exit(0);
        }

        // Read args
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = args[4];
        String filename = args[5];
        String attrsString = "";
        for (int i = 6; i < args.length; i++) {
            attrsString += " " + args[i];
        }

        // Create beans
        CMBConnection connection = new CMBConnection();
        CMBSchemaManagement schemaManagement = connection.getSchemaManagement();
        CMBDataManagement dataManagement = connection.getDataManagement();

        // Connect to server
        TConnect.connect(connection, dstype, server, userid, password);

		// Import the item
        importItem(schemaManagement, dataManagement, entity, filename, attrsString);

		// Disconnect from server.
        TConnect.disconnect(connection);
    }

    /**
     * Creates an item on the server.
     * @param schemaManagement an instance of CMBSchemaManagement bean, to get
     *        information on possible attributes
     * @param dataManagement an instance of CMBDataManagement bean, to perform
     *        the item add.
     * @param entity the name of the entity to create the item under
     * @param attrsString the attributes to give the item, in the form name=value
     */
    public static void importItem(
        CMBSchemaManagement schemaManagement,
        CMBDataManagement dataManagement,
        String entity,
        String filename,
        String attrsString)
        throws CMBException, FileNotFoundException, IOException {

		// Construct a CMBItem object to represent the new item
        System.out.println("Creating CMBItem and setting attributes");
        CMBItem item = new CMBItem();
        item.setConnection(dataManagement.getConnection());
        item.setItemType(CMBBaseConstant.CMB_TYPE_DOCUMENT);
        item.setServerType(dataManagement.getConnection().getDsType());
        item.setServerName(dataManagement.getConnection().getServerName());

        // Look up the entity, to make sure it exists and to get attribute metadata
        System.out.println("  Entity: " + entity);
        CMBEntity entityBean = schemaManagement.getEntity(entity);
        if (entityBean == null) {
            throw new IllegalArgumentException("Entity not found: " + entity);
        }
        item.setEntityName(entity);

		// Set the attributes on the new item
        TCreateItem.setItemAttrs(entityBean, item, attrsString);

		// Construct a CMBObject object for the content part
        CMBObject obj = new CMBObject();
        obj.setMimeType(getMimeType(filename));

		// Read the document data into the CMBObject
        System.out.println("Reading content file");
        File file = new File(filename);
        FileInputStream fileStream = new FileInputStream(file);
        byte[] data = new byte[(int) file.length()];
        int dataread = 0;
        while (dataread < data.length) {
            dataread += fileStream.read(data, dataread, data.length - dataread);
        }
        System.out.println(" Size of input data " + data.length);
        fileStream.close();
        obj.setData(data);
        if (dataManagement.getConnection().getDsType().equals("ICM")) {
            obj.setPartType("ICMBASE");
        }

		// Add a content part to the item.  
		// Note: CMBDataManagement.addContentBeforeCreate
		// method is used rather than addContent because the item
		// does not exist yet.  addContent will throw an exception
		// if the item does not exist.
        System.out.println("Adding content part to item.");
        dataManagement.setDataObject(item);
        dataManagement.addContent(obj);

		// Create the item (including the content part) on the server
        System.out.println("Creating item on server.");
        dataManagement.createItem(entity);

    }

    /**
     * Returns MIME content type for typical file extensions.
     */
    public static String getMimeType(String filename) {
        filename = filename.toLowerCase();
        if (filename.endsWith(".gif")) {
            return "image/gif";
        } else if (filename.endsWith(".jpg") || filename.endsWith(".jpeg")) {
            return "image/jpeg";
        } else if (filename.endsWith(".tif") || filename.endsWith(".tiff")) {
            return "image/tiff";
        } else if (filename.endsWith(".pdf")) {
            return "application/pdf";
        } else if (filename.endsWith(".mda")) {
            return "application/vnd.ibm.modcap";
        } else if (filename.endsWith(".afp")) {
            return "application/afp";
        } else if (filename.endsWith(".txt")) {
            return "text/plain";
        } else if (filename.endsWith(".htm") || filename.endsWith(".html")) {
            return "text/html";
        } else if (filename.endsWith(".rtf")) {
            return "text/richtext";
        } else {
            return "application/binary";
        }
    }

}
