/*
* @copyright(disclaimer) 
*Licensed Materials - Property of IBM
*IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
*(c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.
*
*US Government Users Restricted Rights                                  
*Use, duplication or disclosure restricted by GSA ADP Schedule          
*Contract with IBM Corp.     
*
*DISCLAIMER OF WARRANTIES :                                             
*                                                                       
*Permission is granted to copy and modify this  Sample code, and to           
*distribute modified versions provided that both the copyright        
*notice,- and this permission notice and warranty disclaimer  appear in all copies and modified  
*versions. 
*
*THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. 
*IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM     
*ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                                     
*                                                                 
*@endCopyright       
*       
*/
//*************************************************************************************
// File Name   = TDocumentRouting.java
// 
// Description: This sample shows how to use the
//             CMBDocRoutingQueryServiceICM bean and CMBDocRoutingMgtICM bean.
//
//PREREQUISITS: 
//  TDocumentRouting calls TCreateItemTypeICM. To run TDocumentRouting.java you will need to 
//  compile both TDocumentRouting.java and TCreateItemTypeICM.java.
//  
// Worknodes, worklists, and processes must be created before running this program.
//***************************************************************************************
import  com.ibm.mm.beans.*;
import  com.ibm.mm.beans.util.*;
import  com.ibm.mm.beans.workflow.*;
import com.ibm.mm.beans.workflow.event.*;
import java.lang.*;
import java.io.*;
import java.util.*;
import java.beans.*;
import com.ibm.mm.sdk.common.*;

public  class TDocumentRouting
{
    CMBConnection       connBean   = null;
    CMBDocRoutingQueryServiceICM wfqs     = null;
    CMBDocRoutingDataManagementICM  wfdataBean   = null;
    String          line  = "";
    BufferedReader  in    = new BufferedReader(new InputStreamReader(System.in));
    String itemPid="";
    public TDocumentRouting()
    {
        try {
            connBean = new CMBConnection();  // instantiate the connection bean
            wfqs=connBean.getDocRoutingQueryServiceICM(); // get the document routing query service bean
            wfqs.setTraceEnabled(true);// set the trace on for the document routing query service bean
            wfdataBean = connBean.getDocRoutingDataManagementICM();// get the document routing data management bean
            wfdataBean.setTraceEnabled(true);// set the trace on for the document routing data management bean 
            connBean.setTraceEnabled(true);// set the trace on for the connection bean
        }
        catch (Exception exc) {
            exc.printStackTrace();
        }
    }

/******************************************/
/*  Validate connection bean properties   */
/******************************************/
void  checkConnBeanProperty(){
    try {
        if (connBean.getServerName().length() == 0) {
            connBean.setServerName("ICMNLSDB");
            System.out.println("server name: " + connBean.getServerName());
        }
        if (connBean.getUserid().length() == 0) {
            connBean.setUserid("ICMADMIN");
            System.out.println("userid: " +  connBean.getUserid());
        }
        if (connBean.getPassword().length() == 0) {
           connBean.setPassword("password");
           System.out.println("password: " +  connBean.getPassword());

        }
        connBean.setConfigString("DBAUTH=SERVER");
        connBean.setDsType("ICM");
        short conntype=0;
        connBean.setConnectionType(conntype);
        System.out.println("connection type for datastore: " +  connBean.getConnectionType());

    }
    catch (Exception exc) {
        exc.printStackTrace();
    }
}



/******************************************/
/*  Connect to datastore                  */
/******************************************/
 void connect()
 {
     checkConnBeanProperty();
     try {
         System.out.println("Connecting...");
         connBean.connect();
         System.out.println("OK, connected\n");
     }
     catch (Exception exc) {
         exc.printStackTrace();
     }                                            
 }



/******************************************/
/*  Set the datastore type                */
/******************************************/
public void setDsType(String value) {
try {
   connBean.setDsType(value); 
   }
  catch (Exception ex) {
   ex.printStackTrace();
 }
}


/******************************************/
/*  Create an item type                   */
/******************************************/
public void createItemType()
{
  try{
   System.out.println("We are going to create an item type.\n" +
                      "The pid of the item type will be used to start a process (option 2).");
   String it= getString("Enter a unique item type name");
   String child= getString("Enter a unique child component name");
   String attr1= getString("Enter a unique attribute  name");
   String attr2= getString("Enter a second unique attribute name");
   TCreateItemTypeICM createItemType=new TCreateItemTypeICM(it,child,attr1,attr2);
   itemPid=createItemType.getPid();
   System.out.println("The pid for the item type:\n" + itemPid);
   }
   catch (Exception e) {
    e.printStackTrace();
    }
}


/********************************************************************************************/
/*                                                                                          */
/* Following are examples of the methods the CMBDocumentRoutingQueryServiceICM bean provides*/
/*                                                                                          */
/********************************************************************************************/


/******************************************/
/*  Get a process for a given process name*/
/******************************************/
public void getProcess(){
    CMBProcessICM process;
 try {
     String name= getString("Enter a process name ");
     process=wfqs.getProcess(name);

     if (process==null){
        System.out.println("The processs did not exist for the criteria. Null value returned");
     }
     else listProcessAttributes(process); // list the process attributes
   }catch (Exception ex) {
       ex.printStackTrace();
   }
}


/******************************************/
/*  Get the users for a given ACL name    */
/******************************************/
public void getUsers(){
 try {
    String name= getString("Enter acl name");
    Collection col=wfqs.getUsers(name);
    if (col==null) {
       System.out.println("Users do not exist. Null value returned");
       return;
    }
    if (col.size()==0) {
       System.out.println("There are no users in this acl. Zero returned.");
       return;
    }
    Iterator itemsIter=(Iterator)col.iterator();
    while (itemsIter.hasNext()) {
     System.out.println("User " + itemsIter.next());   
    }
   }catch (Exception ex) {
       ex.printStackTrace();
   }
}


/******************************************/
/*  Get the worklists                     */
/******************************************/
public void getWorkLists(){
 try {
    Collection col=wfqs.getWorkLists();
    if (col==null) {
       System.out.println("Worklists do not exist, null value returned");
       return;
    }
    if (col.size()==0) {
       System.out.println("There are no worklists in the datastore, zero objects returned.");
       return;
    }
    Iterator itemsIter=(Iterator)col.iterator();
    while (itemsIter.hasNext()) {
     listWorkListAttributes((CMBWorkListICM)(itemsIter.next()));   
    }
   }catch (Exception ex) {
       ex.printStackTrace();
   }
}


/******************************************/
/*  Get the processes                     */
/******************************************/
public void getProcesses(){
 try {
    Collection col=wfqs.getProcesses();
    if (col==null) {
       System.out.println("Processes do not exist, null value returned");
       return;
    }
    if (col.size()==0) {
       System.out.println("There are no processes in the datastore, zero objects returned.");
       return;
    }
    Iterator itemsIter=(Iterator)col.iterator();
    while (itemsIter.hasNext()) {
     listProcessAttributes((CMBProcessICM)(itemsIter.next()));   
    }
   }catch (Exception ex) {
       ex.printStackTrace();
   }
}

/**********************************************************/
/*  Get the next work package pid string for a given owner*/ 
/*  and given worklist name.                             */
/*********************************************************/
public void getNextWorkPackagePidString(){
   String value;
 try {
     String name= getString("Enter worklist name ");
     String owner=getString("Enter owner ");
     value=wfqs.getNextWorkPackagePidString(name,owner);
     if (value==null){
        System.out.println("The a work package did not exist for the criteria, null value returned");
     }
     else System.out.println("The next work package pid string: \n" + value);
   }catch (Exception ex) {
       ex.printStackTrace();
   }
}

/*********************************************************************/
/*  Get the work package pid strings for the specified work list name*/ 
/*  and owner name.                                                 */ 
/********************************************************************/
public void getWorkPackagePidStrings(){
   String[] value;
 try {
     String name= getString("Enter worklist name ");
     String owner=getString("Enter owner ");
     value=wfqs.getWorkPackagePidStrings(name,owner);
     if (value==null){
        System.out.println("Pids did not exist for the criteria, null value returned");
     }
     else listPidStrings(value);      
   }
    catch (Exception ex) {
       ex.printStackTrace();
   }
}


/***********************************************************************/
/* Get the work package pid strings which contain the specifed item id.*/
/***********************************************************************/
 public void getWorkPackagePidStringsWithItem(){
    String[] pids;
  try {
      String itemid= getString("Enter item id ");
      pids=wfqs.getWorkPackagePidStringsWithItem(itemid);
      if (pids==null){
         System.out.println("A work package does not exist for the given item id, null value returned");
      } else listPidStrings(pids);      
    }catch (Exception ex) {
        ex.printStackTrace();
    }
 }


/***********************************/
/*  Get the work node names.       */ 
/***********************************/
 public void getWorkNodeNames(){
  try {
      String[] value=wfqs.getWorkNodeNames();
      if (value==null){
         System.out.println("Work nodes do not exist, null value returned");
      }
      if (value.length==0) {
         System.out.println("There are no work nodes in the datastore, zero work node names returned.");
         return;
      }
      listNames(value, "node");      
    }catch (Exception ex) {
        ex.printStackTrace();
    }
 }

/***********************************/
/*  Get the work list names.       */ 
/***********************************/
 public void getWorkListNames(){
  try {
      String[] value=wfqs.getWorkListNames();
      if (value==null){
         System.out.println("There are no worklists, a null value was returned");
      }

      if (value.length==0) {
         System.out.println("There are no worklists, zero names returned.");
         return;
      }
       listNames(value, "worklist");      
    }
     catch (Exception ex) {
        ex.printStackTrace();
    }
  }


/**********************************/
/*  Get the process names .       */ 
/**********************************/
public void getProcessNames(){
try {
     String[] value=wfqs.getProcessNames();
     if (value==null){
        System.out.println("There are no processes, a null value was returned");
     } 
     if (value.length==0) {
        System.out.println("There are no processes, zero processes returned.");
        return;
     }
     listNames(value,"process");      
   }
    catch (Exception ex) {
       ex.printStackTrace();
   }
 }



/**************************************************************************/
/*  Get the work package specified by the work package pid and            */  
/*  check out the referenced item.                                        */
/**************************************************************************/
public void getWorkPackage(){
 try {
     String wpid= getString("Enter work package pid ");
     CMBWorkPackageICM wp=wfqs.getWorkPackage(wpid,true);
     if (wp==null){
        System.out.println("A work package did not exist for the criteria, a null value was returned");
     }else listWorkPackageAttributes(wp);      
   }
    catch (Exception ex) {
       ex.printStackTrace();
   }
}
/**************************************************************************/
/*  Get the next work package in the specified worklist for the specified */ 
/*  owner and check out the referenced item.                              */
/**************************************************************************/
public void getNextWorkPackage(){
 try {

     CMBWorkPackageICM value=null;
     String name= getString("Enter worklist name ");
     String empty= getString("Enter owner: O for owner, A for empty string, N for null. Enter O, A, or N");
     if (empty.equalsIgnoreCase("A")) {
        value=wfqs.getNextWorkPackage(name,"");
     }
     else if (empty.equalsIgnoreCase("N")) {
        value=wfqs.getNextWorkPackage(name,null);
     }
     else {
      String owner= getString("Enter owner.");
      System.out.println("Retrieving the work package for the owner: " + owner);
       value=wfqs.getNextWorkPackage(name,owner);
     }
     if (value==null){
        System.out.println("A work package did not exist for the criteria, a null value returned");
     }else listWorkPackageAttributes(value);
   }
    catch (Exception ex) {
       ex.printStackTrace();
   }
}


/**************************************************************************/
/*  Get the number of work packages in a worklist for a given owner.      */ 
/**************************************************************************/
public void getCount(){
 try {
     String name= getString("Enter worklist name ");
     String empty= getString("You can search by: O for owner, A get all (empty string), N null. Enter O , A, or N");
     int value=0;
     if (empty.equalsIgnoreCase("A")) {
        value=wfqs.getCount(name,"");
     }
     else if (empty.equalsIgnoreCase("N")) {
        value=wfqs.getCount(name,null);
     }
     else {
      String owner= getString("Enter owner.");
       value=wfqs.getCount(name,owner);
     }
     System.out.println("Number of work packages in the worklist : " + value);
   } catch (Exception ex) {
       ex.printStackTrace();
   }
}


/**************************************************************************/
/*  Get the owner of the work package specified by the work package pid   */ 
/**************************************************************************/
public void getWorkPackageOwner(){
 try {
     String pid= getString("Enter work package pid ");
     String owner=wfqs.getWorkPackageOwner(pid);
     System.out.println("Owner:  " + owner);
   } catch (Exception ex) {
       ex.printStackTrace();
   }
}


/**************************************************/
/* Get a work node for a specified work node name */
/**************************************************/
public void getWorkNode(){
 try {
     String name=getString("Enter work node name ");
     CMBWorkNodeICM value=wfqs.getWorkNode(name);
     if (value==null){
        System.out.println("A work node did not exist for the criteria, null value returned");
     }else listWorkNodeAttributes(value);      
    } catch (Exception ex) {
       ex.printStackTrace();
   }
}


/**************************************************/
/* Get a work list for a specified work list name */
/**************************************************/
public void getWorkList(){
 try {
     String name=getString("Enter worklist name ");
     CMBWorkListICM value=wfqs.getWorkList(name);
     if (value==null){
        System.out.println("A worklist did not exist for the criteria, null value returned");
     }
     else listWorkListAttributes(value);      
    }
    catch (Exception ex) {
       ex.printStackTrace();
   }
}


/*****************************************************/
/* Get the priority of a specified work package pid  */
/*****************************************************/
public void getWorkPackagePriority(){
 try {
     String value= getString("Enter work package pid ");
     int priority=wfqs.getWorkPackagePriority(value);
     System.out.println("The work package priority: " + priority);
   }
    catch (Exception ex) {
       ex.printStackTrace();
   }
}

/*****************************************************************************************/
/*                                                                                       */
/* Following are examples of the methods the CMBDocRoutingDataManagementICM bean provides*/
/*                                                                                       */
/****************************************************************************************/


/****************************************************************************************/
/* Continue the work package to the next work node. The specified work package
/* will be removed from the library server and a new work package is created for 
/* the specifed owner. 
/* The item referenced by the item PID will be checked in if it has been
/* checked out.  The PID of the new work package is returned unless the process has ended. 
/* If the process has ended then a null is returned.
/***************************************************************************************/

public void continueProcess() {
try {
   String id= getString("Enter work package pid");
   String route= getString("Enter  route selected");
   String owner= getString("Enter  owner");
   String wpid=wfdataBean.continueProcess(id,route,owner);
   System.out.println("New work package pid returned: \n" + wpid);
   System.out.println("If the work package pid returned is null the process has ended. " );
  }
 catch (Exception ex) {
    ex.printStackTrace();
 }
}
       
/*********************************************************************/
/* Suspend a process for a specifed work package pid 
/* for the specified minutes for the given resume list
/*********************************************************************/
public void suspendProcess() {
try {
   String wpid= getString("Enter work package pid");
   int minutes=getInteger("Enter the number of minutes that the process is to be suspended");

   // create the collection of the resume list objects
    String value= getString("Do you want to create a resume list? Enter Y or N. (N=null resumelist) " );
    ArrayList resumeList = null;
    if (value.equalsIgnoreCase("Y") ) {
     resumeList = new ArrayList();
     resumeList.add(createCMBCollectionResumeListEntryICM());
     resumeList.add(createCMBResumeListEntryICM());
    }
    wfdataBean.suspendProcess(wpid,minutes,resumeList);
  }
 catch (Exception ex) {
   ex.printStackTrace();
}
}


/****************************************************************************/
/* Set the work package owner for the specified work package pid  and owner.
/****************************************************************************/
public void setWorkPackageOwner() {
try {
   String id= getString("Enter work package pid");
   String owner= getString("Enter owner");
    wfdataBean.setWorkPackageOwner(id,owner);
  }
 catch (Exception ex) {
   ex.printStackTrace();
 }
}


/****************************************************************************/
/* Set the work package priority for the specified work package pid.
/****************************************************************************/
public void setWorkPackagePriority() {
try {
   String id= getString("Enter work package pid");
    int priority=(int)getInteger("Enter the wp priority");
    wfdataBean.setWorkPackagePriority(id,priority);
  }
 catch (Exception ex) {
   ex.printStackTrace();
  }
}


/****************************************************************************/
/* Resume a process for the specified work package pid.
/****************************************************************************/
public void resumeProcess() {
try {
   String id= getString("Enter work package pid");
    wfdataBean.resumeProcess(id);
 }
 catch (Exception ex) {
   ex.printStackTrace();
 }
}

/*********************************************************************************/
/* Start a process with the specified process name, item id, priority , and owner.
/* A work package pid is returned. 
/********************************************************************************/
public void startProcess() {
try {
   if (itemPid==null || itemPid.trim().length()<=0 ) {
      System.out.println("You must create an item type (option 1) before creating a process.");
      return;
   }
   String pname= getString("Enter process name");
   //String itemPid= getString("Enter the itemid");
   int priority=(int)getInteger("Enter the priority");
    String owner;
   String empty= getString("Is owner an empty string. Y or N");
     if (empty.equalsIgnoreCase("Y")) {
        owner="";
     }
     else {
       owner= getString("Enter owner ");
     }
   String wpid= wfdataBean.startProcess(pname,itemPid,priority,owner);
   System.out.println("Work package pid:\n" + wpid);
  }
  catch (Exception ex) {
   ex.printStackTrace();
 }
}


/******************************************************************/
/* Check out the item id in the specified work package. 
/******************************************************************/
public void checkOutItemInWorkPackage() {
try {
   String wpid= getString("Enter work package id");
   boolean value= wfdataBean.checkOutItemInWorkPackage(wpid);
   if (value==false) 
      System.out.println("The check out was not successful.");
   else
       System.out.println("The check out was successful."); 
  }catch (Exception ex) {
   ex.printStackTrace();
 }
}



/**********************************************************/
/* Terminate the process specified by the work package pid 
/**********************************************************/
public void terminateProcess() {
try {
   String wpid= getString("Enter work package id");
   wfdataBean.terminateProcess(wpid);
  }
  catch (Exception ex) {
   ex.printStackTrace();
 }
}




/********************************************************************************************/
/*                                                                                          */
/* Following are the helper methods used by this class                                      */
/*                                                                                          */
/********************************************************************************************/

 private CMBResumeListEntryICM createCMBResumeListEntryICM()
   throws DKException, Exception {
    CMBResumeListEntryICM crl = new CMBResumeListEntryICM();
    System.out.println("Creating a CMBResumeListEntryICM object");
    int quantity=(int)getInteger("Enter the quantity needed");
    crl.setQuantityNeeded(quantity);

    String requiredItemTypeName=getString("Enter the required item type name");
    crl.setRequiredItemTypeName(requiredItemTypeName);
    return crl;
  }


 private CMBCollectionResumeListEntryICM createCMBCollectionResumeListEntryICM()
   throws DKException, Exception {
    CMBCollectionResumeListEntryICM crl = new  CMBCollectionResumeListEntryICM();

    System.out.println("Creating a CollectionResumeListEntryICM object");
    int quantity=(int)getInteger("Enter the quantity needed");
    crl.setQuantityNeeded(quantity);

    String folderItemTypeName=getString("Enter the folder item type name");
    crl.setFolderItemTypeName(folderItemTypeName);

    String requiredItemTypeName=getString("Enter the required item type name");
    crl.setRequiredItemTypeName(requiredItemTypeName);
    
    return crl;
  }




private void listPidStrings(String[] pids){
 System.out.println(" \n*******  Listing Work Package Pid Strings ******* ");
 System.out.println("Number of pid strings returned: " + pids.length);
 for (int i=0;i<pids.length;i++) {
   System.out.println("\nThe work package pid string : " + pids[i]);
 }
}


private void listNames(String[] value, String which){
 System.out.println(" \n*******  Listing the "+which +" names ******* ");
 System.out.println("Number of " + which + " names returned: " + value.length);
 for (int i=0;i<value.length;i++) {
   System.out.println("The " + which + " name : " + value[i]);
 }
}


private void listWorkNodeAttributes(CMBWorkNodeICM value){
 System.out.println(" \n*******  Listing Work Node Attributes ******* ");
 String ans= getString("When your ready to view the next set of attributes type any char and hit enter.");
 System.out.println("Type : " + value.getType());
 System.out.println("Time limit :  " + value.getTimeLimit());
 System.out.println("Overload limit :  " + value.getOverloadLimit());
 System.out.println("Work node pid:  " + value.getPidString());
 System.out.println("Leave user function :  " + value.getLeaveUserFunction());
 System.out.println("Overload function  :  " + value.getOverloadUserFunction());
 System.out.println("Enter user function :  " + value.getEnterUserFunction());
 System.out.println("Overload user dll :  " + value.getOverloadUserDll());
 System.out.println("Leave user :  " + value.getLeaveUserDll());
 System.out.println("Description :  " + value.getDescription());
 System.out.println("Collection resume list :  " + value.getCollectionResumeList());
 System.out.println("ACL name :  " + value.getACLName());
 System.out.println("Work node name :  " + value.getName());
}


private void listWorkListAttributes(CMBWorkListICM value){
 System.out.println(" \n*******  Listing WorkList Attributes ******* ");
 String ans= getString("When your ready to view the next set of attributes type any char and hit enter.");
 if (value.getSelectionOrder()==CMBWorkFlowConstants.CMB_ICM_DR_SELECTION_ORDER_PRIORITY) {
    System.out.println("Selection order is 0: ORDER PRIORITY");
 }
 else if (value.getSelectionOrder()==CMBWorkFlowConstants.CMB_ICM_DR_SELECTION_ORDER_TIME) {
       System.out.println("Selection order is 1: ORDER TIME");
    }

 System.out.println("Selection Filter on Notify: " + value.getSelectionFilterOnNotify());
 if (value.getSelectionFilterOnNotify()==CMBWorkFlowConstants.CMB_ICM_DR_SELECTION_FILTER_NO) {
    System.out.println("Selection Filter on Notify 0: FILTER NO");
 }
 else if (value.getSelectionFilterOnNotify()==CMBWorkFlowConstants.CMB_ICM_DR_SELECTION_FILTER_YES) {
       System.out.println("Selection  Filter on Notify 1: FILTER YES");
 }
 else if (value.getSelectionFilterOnNotify()==CMBWorkFlowConstants.CMB_ICM_DR_SELECTION_FILTER_EITHER) {
       System.out.println("Selection Filter on Notify 2: FILTER EITHER");
 }

 System.out.println("Selection Filter on Suspend: " + value.getSelectionFilterOnSuspend());
 if (value.getSelectionFilterOnSuspend()==CMBWorkFlowConstants.CMB_ICM_DR_SELECTION_FILTER_NO) {
    System.out.println("Selection Filter on Suspend 0: FILTER NO");
 }
 else if (value.getSelectionFilterOnSuspend()==CMBWorkFlowConstants.CMB_ICM_DR_SELECTION_FILTER_YES) {
       System.out.println("Selection Filter on Suspend 1: FILTER YES");
 }
 else if (value.getSelectionFilterOnSuspend()==CMBWorkFlowConstants.CMB_ICM_DR_SELECTION_FILTER_EITHER) {
       System.out.println("Selection Filter on Suspend 2: FILTER EITHER");
 }

 System.out.println("Max results : " + value.getMaxResult());
 System.out.println("Pid :  " + value.getPidString());
 System.out.println("Description :  " + value.getDescription() );
 System.out.println("ACL name :  " + value.getACLName());
 System.out.println("Worklist name :  " + value.getName());
 for (int i=0;i<value.getWorkNodeNames().length; i++) {
  System.out.println("Work node name :  " + (value.getWorkNodeNames())[i] );
 }
}



private void listWorkPackageAttributes(CMBWorkPackageICM value){
 System.out.println(" \n*******  Listing Work Package Attributes ******* ");
 String ans= getString("When your ready to view the next set of attributes type any char and hit enter.");
 System.out.println("Resume list size :  " +value.getResumeList().size());
 Collection col=value.getResumeList();
 if (col==null) {
    System.out.println("There are no resume list entries for this work package, null value returned");
 }
 else {
  Iterator itemsIter=(Iterator)col.iterator();
  while (itemsIter.hasNext()) {
   listResumeListEntryAttributes((CMBCollectionResumeListEntryICM)(itemsIter.next()));   
  }
 }
 System.out.println("Process name :  " + value.getProcessName() + "  : length " +  (value.getProcessName()).length());
 System.out.println("Work node name :  " + value.getWorkNodeName() + "  : length " +  (value.getWorkNodeName()).length());
 System.out.println("Owner :  " + value.getOwner() + "  : length " +  (value.getOwner()).length());
 System.out.println("Work package pid :  \n" + value.getPidString());
 System.out.println("Item pid: \n " + value.getItemPidString());
 System.out.println("User last moved:  " + value.getUserLastMoved());
 System.out.println("Priority:  " + value.getPriority());
 System.out.println("Notify state:  " + value.getNotifyState());
 System.out.println("Instance:  " + value.getInstance());
 System.out.println("Suspend state:  " + value.getSuspendState());
 System.out.println("Resume time:  " + value.getResumeTime());
 System.out.println("Notify time :  " + value.getNotifyTime());
 System.out.println("Time last moved :  " + value.getTimeLastMoved());
}

private void listResumeListEntryAttributes(CMBCollectionResumeListEntryICM value){
 System.out.println(" \n*******  Listing Resume List Entry Attributes ******* ");
 String ans= getString("When your ready to view the next set of attributes type any char and hit enter.");
 System.out.println("Required item type name :  " + value.getRequiredItemTypeName()   + "  : length " +  (value.getRequiredItemTypeName()).length());
 System.out.println("Quantity needed :  " + value.getQuantityNeeded());
 if (value instanceof  CMBCollectionResumeListEntryICM) {
  System.out.println("Folder item type name :  " + value.getFolderItemTypeName()  + "  : length " +  (value.getFolderItemTypeName()).length()) ;
 }
}

private void listRouteListEntryAttributes(CMBRouteListEntryICM value){
 System.out.println(" \n*******  Listing Route List Entry Attributes ******* ");
 String ans= getString("When your ready to view the next set of attributes type any char and hit enter.");
 System.out.println("From :  " + value.getFrom()  + "  : length " +  (value.getFrom()).length() );
 System.out.println("To :  " + value.getTo() + "  : length " +  (value.getTo()).length()  ) ;
 System.out.println("getSelection :  " + value.getSelection()    + "  : length " +  (value.getSelection()).length());
}




private void listProcessAttributes(CMBProcessICM value){
 System.out.println(" \n*******  Listing Process Attributes ******* ");
 String ans= getString("When your ready to view the next set of attributes type any char and hit enter.");
 System.out.println("Process name :  " + value.getName() + "  : length " +  (value.getName()).length() );
 System.out.println("ACL name :  " + value.getACLName()    + "  : length " +  (value.getACLName()).length() );
 System.out.println("Description:  " + value.getDescription()    + "  : length " +  (value.getDescription()).length());
 System.out.println("Process pid :  " + value.getPidString()   );
 System.out.println("Last changed time:  " + value.getLastChangedTime());
 System.out.println("Route list entries size:  " + value.getRoute().size());
 Collection col=value.getRoute();
  if (col==null) {
   System.out.println("There are no route  list entries for this process, null value returned");
  }
  else {
   Iterator itemsIter=(Iterator)col.iterator();
   while (itemsIter.hasNext()) {
    listRouteListEntryAttributes((CMBRouteListEntryICM)(itemsIter.next()));   
   }
  }
}
 

public  boolean getBoolean(String prompt) {
boolean ret = false;

while (true) {
  System.out.println("");
  System.out.println(prompt + " (Y/N or y/n) ");
  try {
    line = in.readLine();
    if (line.length() != 0) {
      if      (line.equalsIgnoreCase("y")) ret = true;
      else if (line.equalsIgnoreCase("n")) ret = false;
      else    continue;

      System.out.println("");
      return ret;
      }
    System.out.println("Empty value is not allowed");
    }
  catch (IOException e) {
    e.printStackTrace();
    System.out.println("");
    return false;
    }
  }
}

public  int getInteger(String prompt){
Integer ret = null;

while (true) {
  System.out.println("");
  System.out.println(prompt + " (integer only) ");
  try {
    line = in.readLine();
    if (line.length() != 0) {
      ret = new Integer(line);
      System.out.println("");
      return ret.intValue();
      }
    System.out.println("Empty value is not allowed");
    }
  catch (NumberFormatException e) {
    System.out.println(e.getMessage());
    }
  catch (IOException e) {
    e.printStackTrace();
    System.out.println("");
    return 0;
    }
  }
}

public  String getString(String prompt)
{
while (true) {
  System.out.println("");
  System.out.println(prompt);
  try {
    line = in.readLine();
    if (line.length() != 0) {
      System.out.println("");
      return line;
      }
    System.out.println("Empty value is not allowed");
    }
  catch (IOException e) {
    e.printStackTrace();
    System.out.println("");
    return "";
    }
  }
}



public  String getStringAllowEmpty(String prompt)
{
System.out.println("");
System.out.println(prompt);
try {
  System.out.println("");
  return in.readLine();
  }
catch (IOException e) {
  e.printStackTrace();
  System.out.println("");
  return "";
  }
}




/******************************************/
/*  Main driver for this class            */
/******************************************/

 public  static void main(String argv[])
 {
     String  testName  ="IBM non-visual beans document routing sample program: TDocumentRouting";
     String  copyright ="(c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.";
     System.out.println("");
     System.out.println(testName);
     System.out.println(copyright);
     System.out.println("");

     TDocumentRouting    psObj       = new TDocumentRouting();
     boolean        loopControl = true;

     System.out.println("Connecting to the server...");
     psObj.connect();

   while (loopControl) {
     System.out.println("");
     System.out.println("Selections:");
     System.out.println("0. Exit");
     System.out.println("1. Create an item type. You must create an item type before you can start a process.");

     System.out.println("\nNote: The following selections will use the CMBDocRoutingDataManagementICM bean.");
     System.out.println("2. start a process");
     System.out.println("3. continue a process - a new work package pid is returned.");
     System.out.println("4. suspend a process ");
     System.out.println("5. set the priority of a work package");
     System.out.println("6. set the owner of a work package");
     System.out.println("7. resume a process");
     System.out.println("8. check out the item associated with a work package");
     System.out.println("9. terminate a process");

     System.out.println("\nNote: The following selections will use the CMBDocRoutingQueryServiceICM bean.");

     System.out.println("10. get the next work package for a specified worklist name and owner" );
     System.out.println("11. get the next work package pid string for specified worklist name and owner" );
     System.out.println("12. get the work package pid strings for a specified worklist name and owner" );
     System.out.println("13. get the work package pid strings for a specified item id" );
     System.out.println("14. get the work package priority for a specified work package pid" );
     System.out.println("15. get the owner of the work package for a specified work package pid" );
     System.out.println("16. get the number of work packages for a specified worklist name and owner" );
     System.out.println("17. get the work node names" );
     System.out.println("18. get the work list names" );
     System.out.println("19. get the process names" );
     System.out.println("20. get the processes" );
     System.out.println("21. get the worklists" );
     System.out.println("22. get a worklist for a specified worklist name" );
     System.out.println("23. get a process for a specified process name" );
     System.out.println("24. get a work package for a specified work package pid" );
     System.out.println("25. get a work node for specified work node name" );
     System.out.println("26. get the users defined for a specified acl name" );

     switch (psObj.getInteger("Enter your selection:" )) {
     case 0: loopControl = false;     break;
     case 1: psObj.createItemType();   break; 
     case 2: psObj.startProcess();   break; 
     case 3: psObj.continueProcess();   break; 
     case 4: psObj.suspendProcess();    break; 
     case 5: psObj.setWorkPackagePriority();   break; 
     case 6: psObj.setWorkPackageOwner();   break; 
     case 7: psObj.resumeProcess();   break; 
     case 8: psObj.checkOutItemInWorkPackage();   break; 
     case 9: psObj.terminateProcess();   break; 
     case 10: psObj.getNextWorkPackage();   break;
     case 11: psObj.getNextWorkPackagePidString();   break;
     case 12: psObj.getWorkPackagePidStrings();   break;
     case 13: psObj.getWorkPackagePidStringsWithItem();   break;
     case 14: psObj.getWorkPackagePriority();   break;
     case 15: psObj.getWorkPackageOwner();   break;
     case 16: psObj.getCount();   break;
     case 17:psObj.getWorkNodeNames();   break;
     case 18:psObj.getWorkListNames();   break;
     case 19:psObj.getProcessNames();   break;
     case 20:psObj.getProcesses();   break;
     case 21:psObj.getWorkLists();   break;
     case 22:psObj.getWorkList();   break;
     case 23: psObj.getProcess();   break;
     case 24: psObj.getWorkPackage();   break;
     case 25: psObj.getWorkNode();   break;
     case 26: psObj.getUsers();   break;

     }
 }
  try {
     psObj.connBean.disconnect();
     System.out.println("Bye.");
     System.exit(0);
  }
  catch (Exception ex) {
     ex.printStackTrace();
  }
 }
}


