/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample submit a combined query (parametric query and text query) to DL datastore

/*
 * Filename       : TCQPQandTQDL.java
 * Description    : Connect to DL Datastore
 *                  Create parametric query string
 *                  Create text query string
 *                  Create a combined query object and submit query
 *                  get the results in a collection.
 *                  Disconnect from DL Datastore
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;

public class TCQPQandTQDL implements DKConstantDL
{
  // Main method
  public static void main(String argv[])
  {
          
    // Indicate this is a Console Application on Windows NT/95 for error
    // message display in dos console code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_CM_SS_WINDOWS is the default.
    DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //-------------------------------------------------------------- 
    if (argv.length < 4){
        System.out.println("Usage: " );
        System.out.println("  java TCQPQandTQDL <libSrv> <userid> <pw> <TMserver> " );
        return;
    } 

    String libSrv = argv[0];
    String userid = argv[1];
    String pw     = argv[2];
    String tmsrv  = argv[3];
      
       
    DKDatastoreDL dsDL = null;
    DKDatastoreTS dsTS = null;
    try {
       System.out.println(" *** Test Combined Query : Parametric and Text Query, no scope ");
       dsDL = new DKDatastoreDL();
       System.out.println("connecting to datastore DL"); 
       dsDL.connect(libSrv,userid,pw,"");
       System.out.println("datastore DL connected"); 

       dsTS = new DKDatastoreTS();
       System.out.println("connecting to datastore TS"); 
       dsTS.connect(tmsrv,"",' ');
       System.out.println("datastore TS connected"); 

       String pcmd = null;
       String tcmd = null;
       DKDDO pDDO = null;
       dkXDO pXDO = null;
       DKPidXDO pidXDO = null;
       int i = 0;
       int resultCounter = 0;
       int numDataItems = 0;
       short k = 0;
       short j = 0;
       String pidString;
       String strDataName;
       String className;  
       String className2;
       dkCollection pCol = null;
       dkIterator pIter = null;
       Object a = null;
       dkDataObjectBase pDO = null;
       DKNVPair[] par = new DKNVPair[3];    

       // prepare the parametric query string
       pcmd = "SEARCH=(INDEX_CLASS=DLSAMPLE,COND=(DLSEARCH_Volume > 1));";
       pcmd = pcmd + "OPTION=(CONTENT=NO;";
       pcmd = pcmd + "TYPE_QUERY=STATIC;";
       pcmd = pcmd + "TYPE_FILTER=FOLDERDOC)";
       System.out.println(" parametric query string " + pcmd);

       // create a parametric query object
       DKParametricQuery pq = (DKParametricQuery) dsDL.createQuery(pcmd,DK_CM_PARAMETRIC_QL_TYPE, null);
       par[0] = new DKNVPair(DK_CM_PARM_QUERY, pq);

       // prepare the text query string
       tcmd = "SEARCH=(COND=(IBM));";
       tcmd = tcmd + "OPTION=(SEARCH_INDEX=TMINDEX)";
       System.out.println("text query string " + tcmd);

       // create a text query object
       DKTextQuery     tq = (DKTextQuery) dsTS.createQuery(tcmd,DK_CM_TEXT_QL_TYPE, null);
       
       // create a combined query object
       DKCombinedQueryDL cq = new DKCombinedQueryDL(dsDL);
       par[1] = new DKNVPair(DK_CM_TEXT_QUERY, tq);
       par[2] = new DKNVPair(DK_CM_PARM_END, null);

       // execute the query object
       System.out.println("executing combined query, pq and tq");
       cq.execute(par);
       System.out.println("cq query executed");

       // retrieve the result
       DKResults res = (DKResults) cq.result();
       if (res == null) {
           System.out.println(" CQ results is null ");  
           return;
       } else if (res.cardinality() == 0) {
           System.out.println(" CQ results is empty ");  
           return;
       } else System.out.println(" CQ number of results = " + res.cardinality());  
      
       dkIterator iter = res.createIterator();
       System.out.println("process query results");
       while (iter.more())
       {
          pDDO = (DKDDO) iter.next();
    	  if (pDDO != null){
	        pDDO.retrieve();
    	     System.out.println("=====> Item " + ++resultCounter + " <=====");
             i++;
    	     numDataItems = pDDO.dataCount();
    	     k = pDDO.propertyId(DK_CM_PROPERTY_ITEM_TYPE);
    	     if (k > 0)  {
    	      	Short sVal = (Short)pDDO.getProperty(k);
    	      	j = sVal.shortValue();
    	      	switch (j) {
    	      	   case DK_CM_DOCUMENT :
    	   	        System.out.println("Item is document ");
    	   	        break;
    	      	   case DK_CM_FOLDER :
    	   	        System.out.println("Item is folder ");
    	   	        break;
     	      	} 
    	     }  
    	     System.out.println("Pid " + pDDO.getPidObject().pidString());
    	     System.out.println("Number of Data Items " + numDataItems);
    	     for (j = 1; j <= numDataItems; j++) {
    	      	 a = pDDO.getData(j);
    	      	 strDataName = pDDO.getDataName(j);
    	      	 System.out.println(j + ": Attribute Name " + strDataName );
    	      	 if (a instanceof dkCollection) {
    	      	    System.out.println("   Attribute Value is a collection");    
    	      	    pCol = (dkCollection)a;
    	      	    pIter = pCol.createIterator();
    	      	    i = 0;
    	      	    while (pIter.more() == true) {
    	   	           i++;
    	   	           a = pIter.next();
    	   	           pDO = (dkDataObjectBase)a;
    	   	           if (pDO.protocol() == DK_CM_PDDO) {
    	   		       System.out.println("    DKDDO object " + i + " in collection");
    	   		       System.out.println("      object's pid is " + 
                                ((DKDDO)pDO).getPidObject().pidString());
	   	           } else if (pDO.protocol() == DK_CM_XDO) {
    	   		       System.out.println("    dkXDO object " + i + " in collection");
    	   		       pXDO = (dkXDO)pDO;
    	   		       pidXDO = (DKPidXDO)pXDO.getPidObject();
    	   		       pidString = pidXDO.pidString();
    	   		       System.out.println("      object's pid is " + pidString);
	   	           } 
	      	    } // end while
    	      	 } // end if collection	   
                 else if (a != null)
	      	    System.out.println("   Attribute Value " + a);    
                 else 		
	      	    System.out.println("   Attribute Value  is NULL");    
    	     }       
    	  } // if pDDO == null
       }

       System.out.println("query results processed");
       dsTS.disconnect();
       dsDL.disconnect();
       System.out.println("datastore disconnected"); 
       dsTS.destroy();
       dsDL.destroy();

    }
    catch (DKException exc)
    {
     try {
         dsDL.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     try {
        if (dsTS != null)
         dsTS.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     try {
         dsDL.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     try {
        if (dsTS != null)
         dsTS.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }

  }
}

