/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample to submit a text query to Text Search datastore

/*
 * Filename       : TClientSampleTQryTS.java
 * Description    : connect to Text Search.
 *                  create query string
 *                  create a text query object  and submit query.
 *                  get the results in a collection.
 *                  display items with Text Search Information.
 *                  disconnect from Text Search
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.client.*;
import java.io.*;
import java.lang.*;

// TClientSampleTQryTS class
public class TClientSampleTQryTS implements DKConstantDL {

  // Main method
  public static void main(String argv[]){
    
      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (argv.length < 1){
          System.out.println("Usage: " );
          System.out.println("  java TClientSampleTQryTS <srchSrv> " );
          return;
      }
      
      String srchSrv = argv[0];
       
      DKDatastoreTS dsTS = null;
      try {
         dsTS = new DKDatastoreTS();
         dkQuery pQry = null;
         DKResults pResults = null;
         DKNVPair parms[] = null;
         System.out.println("connecting to datastore"); 
         dsTS.connect(srchSrv,"",' ');
         System.out.println("datastore connected"); 
         String cmd = "SEARCH=(COND=(internet OR UNIX));" +
                      "OPTION=(SEARCH_INDEX=TMINDEX)";
         System.out.println("query string " + cmd);
         System.out.println("create query");
         pQry = dsTS.createQuery(cmd,DK_TEXT_QL_TYPE,parms);
         System.out.println("executing query");
         pQry.execute(parms);
         System.out.println("query executed");
         System.out.println("number of results " + pQry.numberOfResults());
         System.out.println("process query results");
         pResults = (DKResults)pQry.result();
         processResults((dkCollection)pResults);
         System.out.println("query results processed");
         dsTS.disconnect();
         System.out.println("datastore disconnected"); 
         dsTS.destroy();
      }
      catch(DKException exc){
     	 try {
     	     dsTS.destroy();
     	 }
     	 catch (Exception e)
     	 {
     	  e.printStackTrace();
     	 }         
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         System.out.println("Exception error code " + exc.errorCode());
         System.out.println("Exception error state " + exc.errorState());
         exc.printStackTrace();
      }
      catch (Exception exc){
     	 try {
     	     dsTS.destroy();
     	 }
     	 catch (Exception e)
     	 {
     	  e.printStackTrace();
     	 }         
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
  }
 
  public static void processResults(dkCollection results) throws DKException, Exception
  {
      DKDDO    item = null;
      short    i = 0;
      short    numItems = 0;
      String   dataName;
      Object a = null;
 
      dkIterator iter = results.createIterator();
 
      while(iter.more()){
          item = (DKDDO) iter.next();
          if (item != null){
              numItems++;
              System.out.println("******** Item " + numItems + " ********");
              for (i = 1; i <= item.dataCount(); i++) {
                  a = item.getData(i);
                  dataName = item.getDataName(i);
                  System.out.println(i + ": Attribute Name " + dataName );
                  if (a != null)
                      System.out.println("   Attribute Value " + a);    
                  else 
                      System.out.println("   Attribute Value is null ");
              }
          }
      }
      System.out.println("======> Total Items is " + numItems + " <======");
    }
}

