/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TDataTypesDL.java
 * Description    : A sample program to add/retrieve/update/delete a document containing 
 *                  different type of attributes, that is, string, int, Date, Time, etc
 *                  It uses an existing index class "DataType_Index".
 *
 * Preparation to run this program:
 *
 *          	    - use System Administration GUI tools to create Index-Class "DataType_Index" 
 *                    with the following attributes:
 *	  
 *          		IndexClassName : DataType_Index
 *          		 ...attribute [1] :
 *          		     name          = Data_FString
 *          		     type          = 2
 *          		     size          = 20
 *          		     nullable      = true
 *          		     precision     = 20
 *          		     scale         = 1
 *          		     stringType    = 4
 *          		 ...attribute [2] :
 *          		     name          = Data_VString
 *          		     type          = 1
 *          		     size          = 32
 *          		     nullable      = true
 *          		     precision     = 32
 *          		     scale         = 1
 *          		     stringType    = 4
 *          		 ...attribute [3] :
 *          		     name          = Data_Short
 *          		     type          = 3
 *          		     size          = 6
 *          		     nullable      = true
 *          		     precision     = 32767
 *          		     scale         = -32768
 *          		     stringType    = 0
 *          		 ...attribute [4] :
 *          		     name          = Data_Long
 *          		     type          = 4
 *          		     size          = 11
 *          		     nullable      = true
 *          		     precision     = 2147483647
 *          		     scale         = -2147483648
 *          		     stringType    = 0
 *          		 ...attribute [5] :
 *          		     name          = Data_Date
 *          		     type          = 7
 *          		     size          = 10
 *          		     nullable      = true
 *          		     precision     = 0
 *          		     scale         = 0
 *          		     stringType    = 0
 *          		 ...attribute [6] :
 *          		     name          = Data_Time
 *          		     type          = 8
 *          		     size          = 8
 *          		     nullable      = true
 *          		     precision     = 0
 *          		     scale         = 0
 *          		     stringType    = 0
 *          		 ...attribute [7] :
 *          		     name          = Data_Timestamp
 *          		     type          = 9
 *          		     size          = 26
 *          		     nullable      = true
 *          		     precision     = 0
 *          		     scale         = 0
 *          		     stringType    = 0
 *          		 ...attribute [8] :
 *          		     name          = Data_Decimal
 *          		     type          = 6
 *          		     size          = 7
 *          		     id            = 125
 *          		     nullable      = true
 *          		     precision     = 5
 *          		     scale         = 2
 *          		     stringType    = 0
 *         	   
 *          		 This index class will be used in this sample program.
 *
 * Program's steps:
 *                  - connect to DL datastore
 *                  - create a document DDO
 *                  - add different type of attributes to this DDO
 *                  - set the value of these attributes
 *                  - display the content of this DDO
 *                  - add this DDO to the DL persistent store
 *                  - retrieve it back, and display the content
 *                  - update this DDO, retrieve it back and display the content
 *                  - delete this document DDO from the persistent store
 *                  - disconnect from DL datastore
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;

public class TDataTypesDL implements DKConstantDL 
{
  // Main method
  public static void main(String argv[])
  {
    System.out.println("begin...");
    
    // Checking for input parameters
    if (argv.length < 3) {
       System.out.println("Usage: " );
       System.out.println("  java TDataTypes " + "<LibServer> <userid> <password>" );
       return;
    }

    String libSrv = argv[0];
    String userid = argv[1];
    String pw     = argv[2];

    String indexclass = "DataType_Index";

    System.out.println("index class = " + indexclass );

    DKDatastoreDL dl = null;
    try 
    {
       dl = new DKDatastoreDL();
           
       System.out.println("Connecting to datastore DL : " + libSrv);
       dl.connect(libSrv, userid, pw, "");
       System.out.println(" datastore connected" );
                   
       DKPid pid = new DKPid();
       pid.setObjectType(indexclass);
       
       DKDDO ddo = new DKDDO(dl, pid);
       
       // set the item type to be a document
       ddo.addProperty(DK_CM_PROPERTY_ITEM_TYPE, new Short(DK_CM_DOCUMENT));
       
       short dataId;
       String[] attrName = { 
                                "Data_FString",
                                "Data_VString",
                                "Data_Short",
                                "Data_Long",
                                "Data_Date",
                                "Data_Time",
                                "Data_Timestamp",
                                "Data_Decimal"
                           };
       
       Object[] attrValue = new Object[8];
       attrValue[0] = new String("testchar");
       attrValue[1] = new String("vchar");
       attrValue[2] = new Short((short)32767);
       attrValue[3] = new Integer(500);
       attrValue[4] = new DKDate(1990-1900, 4-1, 24);
       attrValue[5] = new DKTime(12,12,12);
       attrValue[6] = new DKTimestamp(97, 9, 4, 12, 13, 14, 999);
       attrValue[7] = new String("5.2");
       
       short[] attr_prop_type = { 
                                  DK_CM_DATAITEM_TYPE_STRING,
                                  DK_CM_DATAITEM_TYPE_STRING,
                                  DK_CM_DATAITEM_TYPE_SHORT,
                                  DK_CM_DATAITEM_TYPE_LONG,
                                  DK_CM_DATAITEM_TYPE_DATE
                                 ,DK_CM_DATAITEM_TYPE_TIME
                                 ,DK_CM_DATAITEM_TYPE_TIMESTAMP
                                 ,DK_CM_DATAITEM_TYPE_STRING         // DK_CM_DATAITEM_TYPE_DECIMAL inside
                                }; 
       
       short attrCount = (short) attrName.length;
       
       int i;
       for (i=0; i < attrCount; i++)
       {
          dataId = ddo.addData(attrName[i]);
          ddo.addDataProperty(dataId, DK_CM_PROPERTY_TYPE, new Short(attr_prop_type[i]));                // mandatory
          ddo.addDataProperty(dataId, DK_CM_PROPERTY_NULLABLE, new Integer(DK_CM_TRUE));                    // optional
          ddo.addDataProperty(dataId, DK_CM_PROPERTY_ACCESS_RIGHT, new Integer(DK_CM_ATTRIBUTE_READWRITE)); // optional
          ddo.setData(dataId, attrValue[i]);
       }
       
       System.out.println("\n *** displaying the ddo before adding it ***\n ");
       displayDDO(ddo);
      
       // add the document DDO to the DL persistent store
       System.out.println("\n *** adding the ddo *** \n" );

       ddo.add();

       pid = ddo.getPidObject();
       System.out.println(" the new added ddo itemId = " + pid.getPrimaryId() );     
       
       // retrieve it back to see it is really the same one
       System.out.println(" retrieve the newly added ddo, with itemId = " + pid.getPrimaryId() );     
       
       // create a new DDO for retrieving it
       DKDDO nddo = new DKDDO(dl, pid);
       try {
          nddo.retrieve();
       } catch (DKException e) {
         System.out.println(" item does not exist " + e.toString() );
         return;
       } 
       System.out.println(" displaying the retrieved ddo" );
       displayDDO(nddo);

       System.out.println("\n *** updating this DDO ***\n");
       
       Object[] u_attr = new Object[8];
       u_attr[0] = new String("updated char");
       u_attr[1] = new String("updated vchar");
       u_attr[2] = new Short((short)-32768);
       u_attr[3] = new Integer((int)-2147483648);
       u_attr[4] = new DKDate(1997-1900, 05-1, 15);
       u_attr[5] = new DKTime(10,10,10);
       u_attr[6] = new DKTimestamp(96, 9, 3, 12, 30, 40, 88900);
       u_attr[7] = new String("10.5");
       
       short u_dataid;
       String aName;
       for (i=0; i < attrCount; i++) {
           aName = attrName[i];
           u_dataid = nddo.dataId(aName);
           System.out.println(" original " + aName + " value = " + nddo.getData(u_dataid) );
           nddo.setData(u_dataid, u_attr[i]);
           System.out.println("      new " + aName + " value = " + nddo.getData(u_dataid) );
       }
       
       nddo.update();

       pid = nddo.getPidObject();
       
       // retrieve back the updated DDO
       System.out.println(" retrieve the updated ddo, with itemId = " + pid.getPrimaryId() );
       DKDDO uddo = new DKDDO(dl, pid);
       uddo.retrieve();
       System.out.println("displaying the retrieved ddo" );
       displayDDO(uddo);
              
       // delete the DDO from the persistent store
       System.out.println("\n *** deleting the ddo ***\n");
       ddo.del(); 

       // disconnect from DL datastore
       dl.disconnect();
       System.out.println("datastore disconnected");
       dl.destroy();
    }
    catch(DKException exc)
    {
     try {
         dl.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }   
    catch (Exception exc)
    {
     try {
         dl.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     exc.printStackTrace();
    }

    System.out.println(" done" );
  }

  public static void displayDDO(DKDDO ddo)
  {
    short i, j;
    String dataName;

    try {
      System.out.println(" data count = " + ddo.dataCount());
      short itemType = ((Short)ddo.getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE)).shortValue();
      if (itemType == DK_CM_FOLDER)
          System.out.println(" this item is a Folder, itemId = " + ddo.getPidObject().getPrimaryId());
      else if (itemType == DK_CM_DOCUMENT)
          System.out.println(" this item is a Document, itemId = " + ddo.getPidObject().getPrimaryId());
      else 
          System.out.println(" this item is of unknown type, type = " + itemType +
                                             ", itemId = " + ddo.getPidObject().getPrimaryId() );
      // print the properties of this DDO
      System.out.println(" properties of this DDO: " + ddo.propertyCount());
      for (i = 1; i <= ddo.propertyCount(); i++) {
           System.out.println("   property_name(" + i + ") = " + 
                                  ddo.getPropertyName(i) + ", value = " +
                                  ddo.getProperty(i));
      }

      // print out the content
      Object value;
      DKParts  parts;
      DKFolder folder;
      DKBlobDL blob;
      dkIterator iter;
      
      for ( i=1; i <= ddo.dataCount(); i++) {
           dataName = ddo.getDataName(i);
           System.out.print(" data item name (" + i + ") = " + dataName + " ");
           value = ddo.getData(i);
           if (value != null) 
                System.out.println("  \t value (" + i + ") = " + value);
           else System.out.println("  \t value (" + i + ") = null" );
           System.out.println(" properties of this data item: " + ddo.dataPropertyCount(i));
           for ( j=1; j <= ddo.dataPropertyCount(i); j++) {
               System.out.println("   property_name(" + i + "," + j + ") = " + 
                                      ddo.getDataPropertyName(i,j) + ", value = " +
                                      ddo.getDataProperty(i,j));
           }
           
           if (dataName.equals(DKFOLDER)) {
	         if (value != null) {
                 folder = (DKFolder) value;
       		     iter = folder.createIterator();
       		     DKDDO mddo;
       		     while(iter.more()) {
       		        mddo = (DKDDO) iter.next();
       		        mddo.retrieve();               // retrieve a folder member
       		        displayDDO(mddo);
       		     }         
             }
           }
           
           if (dataName.equals(DKPARTS)) {
              if (value != null) {
                parts = (DKParts) value;
                iter  = parts.createIterator();

                while(iter.more()) {
                     blob = (DKBlobDL) iter.next();   
                     
                     System.out.println(" \t\t blob part id " +
                       ((DKPidXDODL) (blob.getPidObject())).getPartId() +
                            " content class = " +
                            blob.getContentClass() +
                            " rep type " +
                            blob.getRepType() +
                            " aff type " +
                            blob.getAffiliatedType() );
  
                     blob.retrieve();                // retrieve a part
                }// while
              } // if
           } // if
      }//while
   
    }//try
    catch (DKException exc)
    {
       System.out.println(" Exception name " + exc.name());
       System.out.println(" Exception message " + exc.getMessage());
       exc.printStackTrace();
    }
    catch (Exception exc)
    {
       exc.printStackTrace();
    }
  }
  
}
