/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample to connect to Text Search,  execute a query, display the result,
//           then disconnect 

/*
 * Filename       : TExecuteStructDocTS.java
 * Description    : Connect to Text Search Datastore.
 *                  Create query string.
 *                  Get results in a resultset cursor.
 *                  Loop through results.
 *                  Display items found.
 *                  Disconnect from Text Search Datastore.
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;

// TExecuteStructDocTS class
public class TExecuteStructDocTS implements DKConstantDL
{
  // Main method
  public static void main(String argv[])
  {
    // Indicate this is a Console Application on Windows NT/95 for error
    // message display in dos console code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_CM_SS_WINDOWS is the default.
    DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //-------------------------------------------------------------- 
    if (argv.length < 2)
    {
       System.out.println("Invoke Syntax: " );
       System.out.println("  java TExecuteStructDocTS " +
        "<srchSrv> <srchIndex>" );
       return;
    }
   
    String srchSrv = argv[0];
    String strIndex = argv[1];
       
    DKDatastoreTS dsTS = null;
    try {
    dsTS = new DKDatastoreTS();
    dkResultSetCursor pCur = null;
    DKNVPair parms[] = null;
    Integer iValObj = null;
    System.out.println("connecting to datastore"); 
    dsTS.connect(srchSrv,"","","");
    System.out.println("datastore connected srchSrv " + srchSrv); 
    String cmd = "SEARCH=(COND=($CCSID=850," + 
           "DOCMOD=(DOCMODNAME=SAMPCORPMOD," +
           "SECLIST=(SAMPCORPTITLE,SAMPCORPBODY))$ Corp));" +
           "OPTION=(SEARCH_INDEX=(" + strIndex + ");MAX_RESULTS=5)"; 
    Integer iVal;
    DKDDO p = null;
    DKPid pid = null;
    int i = 0;
    int numDataItems = 0;
    short j = 0;
    String strDataName;
    Object a = null;
    System.out.println("query string " + cmd);
    System.out.println("executing query");
    pCur = dsTS.execute(cmd,DK_TEXT_QL_TYPE,parms);
    System.out.println("datastore executed query");
    System.out.println("cardinality " + pCur.cardinality());
    System.out.println("process query results");
    while (pCur.isValid())
    {
     p = pCur.fetchNext();
     if (p != null)
     {
      i = pCur.getPosition();
      System.out.println("=====> Item " + i + " <=====");
      pid = p.getPidObject();
      System.out.println("pid is " + pid.pidString());
      System.out.println("pid id string " + pid.getPrimaryId());
      System.out.println("pid idString " + pid.getIdString());
      System.out.println("pid idString cnt " + pid.getIdStringCount());
      System.out.println("pid primary id string " + pid.getPrimaryId());
      numDataItems = p.dataCount();
      System.out.println("Number of Data Items " + numDataItems);
      for (j = 1; j <= numDataItems; j++)
      {
       a = p.getData(j);
       strDataName = p.getDataName(j);
       System.out.println(j + ": Attribute Name " + strDataName );
       if (a instanceof String)
       {
        System.out.println("   Attribute Value " + a);    
       }
       else if (a instanceof Integer)
       {
        System.out.println("   Attribute Value " + a);    
       }
       else if (a instanceof Short)
       {
        System.out.println("   Attribute Value " + a);    
       }
      }
     }
    }
    pCur.destroy(); // Finished with the cursor
    System.out.println("query results processed");
    dsTS.disconnect();
    System.out.println("datastore disconnected"); 
    dsTS.destroy();
    }
    catch (DKException exc)
    {
     try {
         dsTS.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     System.out.println("Exception error code " + exc.errorCode());
     System.out.println("Exception error state " + exc.errorState());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     try {
         dsTS.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }

  }
}

