/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TListCatalogFed.java
 * Description    : List Fed servers, entities and attributes via the DKDatastoreFed
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

public class TListCatalogFed implements DKConstantFed
{

  // Main method
  public static void main(String argv[])
  {
     // Indicate this is a Console Application on Windows NT/95 for error
     // message display in dos console code page. If this was a Windows
     // GUI (Graphic User Interface) Application this would not be necessary
     // because DK_CM_SS_WINDOWS is the default.
     DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

     //-------------------------------------------------------------- 
     // Checking for input parameters
     //-------------------------------------------------------------- 
     if (argv.length < 3) {
        System.out.println("Usage Syntax: ");
        System.out.println("\t java TListCatalogFed <cmbdb> <userid> <pw>");
        return;
      }
      
      String  cmbdb = argv[0];
      String  userid = argv[1];
      String  pw     = argv[2];

      try {
    	 DKSequentialCollection pCol = null;
    	 dkIterator pIter = null;
    	 DKServerDefFed pSV = null;
    	 String strServerName = "";
    	 String strRepType = "";
         int i = 0;
         dkDatastore dsFed = new DKDatastoreFed();
    	 System.out.println("list servers");
    	 pCol = (DKSequentialCollection)dsFed.listDataSources();
    	 pIter = pCol.createIterator();
    	 while (pIter.more() == true)
    	 {
    	  i++;
    	  pSV = (DKServerDefFed)pIter.next();
    	  strServerName = pSV.getName();
    	  strRepType = pSV.getRepositoryType();
    	  System.out.println("Server Name [" + i + "] - " + strServerName + " Server Repository Type - " + strRepType);
    	 }
    	 System.out.println(i + " servers listed");
         System.out.println(" *** connecting to datastore");

         dsFed.connect(cmbdb, userid, pw,"");
         System.out.println(" *** datastore connected"); 
         String userName = dsFed.userName();
         String dsName = dsFed.datastoreName();
         System.out.println("     user " + userName + " dsName " + dsName); 

         // get the Def and Admin
         dkDatastoreDef  defFed = dsFed.datastoreDef();
         DKDatastoreAdminFed    adm = (DKDatastoreAdminFed) defFed.datastoreAdmin();

         //=======================================================================================================
         // list back-end servers

         String servers[] = adm.listServerNames();
         System.out.println("List of servers in federated database");
         for (int index = 0; index < servers.length; index++)
            System.out.println("Server [" + index + "] = " + servers[index].trim());
      
         System.out.println("\nDetail servers description");
         // retrieve the definition of the servers
         DKServerMapDefFed serverdat;
         for (int index = 0; index < servers.length; index ++) {
            serverdat = adm.getServer(servers[index]);
            System.out.println("\n DKServerMapDefFed = " + serverdat.getName().trim());
            System.out.println("          name: " + serverdat.getName().trim());
            System.out.println("          type: " + serverdat.getServerType());
            System.out.println("   Connect Str: " + serverdat.getConnectString());
            System.out.println("         desc : " + serverdat.getDescription());
            System.out.println("      refTime : " + serverdat.getRefreshTime());
         }
	
         //=======================================================================================================
         // list federated entity and attribute names

	     String[] names = dsFed.listEntityNames();
	     for (i=0; i<names.length; i++) {
		    System.out.println("name[" + i + "] = " + names[i] );
		    dkEntityDef entityFed   = defFed.getEntity(names[i]);
		    System.out.println("\tDescription= " + entityFed.getDescription());

		    String[] attrNames   = entityFed.listAttrNames();
		    if (attrNames != null) {
			    for (int j=0; j<attrNames.length; j++) {	
				    System.out.println("\tattrName = " + attrNames[j]);
				    dkAttrDef attr = entityFed.getAttr(attrNames[j]);
				    System.out.println("\t\tentityName:   " + attr.getEntityName());
				    System.out.println("\t\tdsName:       " + attr.datastoreName());
				    System.out.println("\t\tdsType:       " + attr.datastoreType());
				    System.out.println("\t\ttype:         " + attr.getType());
				    System.out.println("\t\tdescription:  " + attr.getDescription());
				    System.out.println("\t\tsize:         " + attr.getSize());
				    System.out.println("\t\tid:           " + attr.getId());
				    System.out.println("\t\tprecision:    " + attr.getPrecision());
				    System.out.println("\t\tscale:        " + attr.getScale());
				    System.out.println("\t\tmax:          " + attr.getMax());
				    System.out.println("\t\tmin:          " + attr.getMin());
				    System.out.println("\t\tstringType:   " + attr.getStringType());

				    if (attr.isNullable())
					    System.out.println("\t\tisNullable:   true");
				    else 
					    System.out.println("\t\tisNullable:   false");
	
				    if (attr.isUpdatable())
					    System.out.println("\t\tisUpdatable:  true");
				    else 
					    System.out.println("\t\tisUpdatable:  false");
			
				    if (attr.isQueryable())
					    System.out.println("\t\tisQueryable:  true");
				    else 
					    System.out.println("\t\tisQueryable:  false");
				    System.out.println("\n");
			    }
		    }
	    }

	    dsFed.disconnect();
	    System.out.println("datastore disconnected"); 
	    userName = dsFed.userName();
	    dsName = dsFed.datastoreName();
	    System.out.println("user " + userName + " dsName " + dsName);   
    }
    catch (DKException exc)
    {
	    System.out.println("Exception name " + exc.name());
     	System.out.println("Exception message " + exc.getMessage());
     	exc.printStackTrace();
    }
    catch (Exception exc)
    {
    	System.out.println("Exception message " + exc.getMessage());
     	exc.printStackTrace();
    }

  }
}

