/* -------------------------------------------------------------------
 * @copyright(disclaimer) 
 * IBM Enterprise Information Portal for Multiplatforms V7.1  (program number 5697-G29) 
 * (c ) Copyright IBM Corp. 1998, 2000. All Rights Reserved. 
 * IBM Content Manager for Multiplatforms V6.1  (program number 5648-D03) 
 * (c ) Copyright IBM Corp. 1994, 2000. All Rights Reserved. 
 * 
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.  Licensed Materials - Property of IBM 
 * 
 * DISCLAIMER OF WARRANTIES : 
 * 
 * Permission is granted to copy and modify this code, and to 
 * distribute modified versions provided that both the copyright 
 * notice and this permission notice appear in all copies and modified 
 * versions. THIS SAMPLE CODE IS LICENSED TO YOU AS-IS. IBM DISCLAIMS 
 * ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE. IBM 
 * WILL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE 
 * OF THE CODE.
 * 
 * @endCopyright
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.client.*; // using client package
import java.io.*;
import java.lang.*;

public class TClientExecuteIC implements DKConstantIC
{

   // Main method
   public static void main(String argv[])
   {

      dkQuery pQry;
      dkResultSetCursor pCur = null;
      DKDDO item, p;
      short type, prop_id;


      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      if (argv.length < 3)
      {
         System.out.println("Usage: " );
         System.out.println("  java TClientExecuteIC <db> <userid> <pw> [<connectString>]" );
         return;
      }

      String db = argv[0];
      String userid = argv[1];
      String pw     = argv[2];
      String connString = "";
      
      if ( argv.length > 3 )
      {
         connString = argv[3];
      }
      
      try {
         DKDatastoreIC dsIC = new DKDatastoreIC();

	 System.out.println("--- connecting to datastore ---");
         dsIC.connect(db, userid, pw, connString);
         System.out.println("--- datastore connected ---");

// Parametric Query
// TABLE - DIMENSON - Dimensions within a multi-dimensional database
// NAME - Name
// SHRTDESC - Short description
         String qry = "SEARCH=(ENTITY=Dimensions within a multi-dimensional database," +
               "MAX_RESULTS=10,COND=( ('Name' LIKE 'M%')" +
               " OR ('Short description' = 'Product dimension')));" +
               "OPTION=(CONTENT=YES)";
         dkResultSetCursor rs = dsIC.execute( qry, DK_CM_PARAMETRIC_QL_TYPE, null );
         if( rs != null ) {
            while( rs.isValid() ) {
               DKDDO ddo = rs.fetchNext();
               if( ddo != null ) {
                  System.out.println( "\n******************************\n" );
                  for( short i = 1; i <= ddo.dataCount(); i++ ) {
                     String dName = ddo.getDataName( i ).trim();
                     Object dValue = ddo.getData( i );
                     System.out.println( dName + " = '" + dValue +
                              "'" );
                  }
               }
            }
         System.out.println( "\n******************************\n" );
         }
         //rs.close();
		 rs.destroy();


// SQL Query 
// Use LIKE and OR
// TABLE - DIMENSON - Dimensions within a multi-dimensional database
// NAME - Name
// SHRTDESC - Short description
         String cmd = "Select * from dimenson where name like 'M%'" +
	       "or shrtdesc = 'Product dimension'";
         rs = dsIC.execute( cmd, DK_CM_SQL_QL_TYPE, null );
         if( rs != null ) {
            while( rs.isValid() ) {
               DKDDO ddo = rs.fetchNext();
               if( ddo != null ) {
                  System.out.println( "\n******************************\n" );
                  for( short i = 1; i <= ddo.dataCount(); i++ ) {
                     String dName = ddo.getDataName( i ).trim();
                     Object dValue = ddo.getData( i );
                     System.out.println( dName + " = '" + dValue +
                              "'" );
                  }
               }
            }
         System.out.println( "\n******************************\n" );
         }
         //rs.close();
		 rs.destroy();


// execute with DKCQExpr

         DKNVPair soptions[] = new DKNVPair[1];
         DKNVPair qoptions[] = new DKNVPair[1];
         DKNVPair tnv = null;
         DKCQExpr cqe = new DKCQExpr();
         DKQExpr qe = new DKQExpr();
         DKQTerm qterm = new DKQTerm("Short description", DKOpCode.EQ, "'Product dimension'");  // Modified query to make sense for IC
         DKQSubExpr sqe = new DKQSubExpr();
         String strTmp = "";
         strTmp = "5";
         tnv = new DKNVPair("MAX_RESULTS",strTmp);
         soptions[0] = tnv;
         sqe.setOptionList(soptions);

         cqe.setQLType(DK_PARAMETRIC_QL_TYPE);
         qe.setQLType(DK_PARAMETRIC_QL_TYPE);
         sqe.setTranslation(false);
         sqe.setEntityName("Dimensions within a multi-dimensional database");
         sqe.setQueryTerm(qterm);
         strTmp = "YES";
         tnv = new DKNVPair("CONTENT", strTmp);
         qoptions[0] = tnv;
         qe.setOptionList(qoptions);
         qe.addSubQuery(sqe);            //  Have to set the subquery!
         cqe.setQueryExpr(qe);

         pCur = dsIC.execute(cqe);

         int count = 0;
         while (pCur.isValid())
         {
            p = pCur.fetchNext();
            if (p != null)
            {
               count++;
            }
         }

         System.out.println("  Num of query results: " + count);
		pCur.destroy();



         dsIC.disconnect();
         System.out.println("--- datastore disconnected ---");
         dsIC.destroy();
      }
      catch (DKException exc)
      {
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
      catch (Exception exc)
      {
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
   }
}

