/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Attribute Groups
    Groups of attributes can be created and used to easily add, retrieve,
    and access entire groups of attributes to items and sub-components.
    Attributes may belong to any number of Attribute Groups, from zero to 
    many.  The same attribute name may appear in multiple attribute groups, 
    and since a DDO may contain multiple attribute groups, the same attribute
    name may appear in the DDO, but each attribute is completely seperate.

    When adding attributes, use the individual attribute definition names.  However, 
    once they are part of an attribute group, in order to access a particular 
    attribute of a group, the correct attribute name follows the following format:
    
        Grouped Attribute's Name Format:  <Attribute Group Name>.<Attribute Name>

    When an attribute is added to an attribute group, this will only impact Component 
    Types that are created after the addion.  Existing component types will remain 
    unchanged.
    
    Names
        Attribute names are limited to 15 characters.  Use the description
        field to add more detailed information.

        Names only need to be unique among those of its own namespace.  The 
        Attribute namespace is seperate from the Entity Type namespace, Link Type 
        Namespace, etc.

Reference Attribute (Group):
    Please consult the SReferenceAttrDefCreationICM Sample.

---------------------------------
Insurance Model Attribute Groups:
---------------------------------
NOTE: In this sample, the prefix "S_" will be used to indicate that the attribute
      group is used by and was create by the samples.  This helps avoid using
      attribute group names that may already be in use.

    Attribute Group    Attributes     Part of Insurance Model Used In
    ---------------  ---------------  -------------------------------
    S_carAccident    S_accidentDate   Adjuster Report
                     S_licensePlate

---------------------------------------
Attribute Groups Defined in this Sample
---------------------------------------

    Attribute Group    Attributes 
    ---------------  ---------------
    S_carAccident    S_accidentDate
                     S_licensePlate
    --------------------------------
    S_address        S_street
                     S_city
                     S_state
                     S_zipCode
    --------------------------------

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SAttributeGroupDefCreationICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Defining & Explaining Attribute Groups, Insurance Model Attribute Groups
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Creating Attribute Groups
 *                    Attribute Group Name Requirements
 *                    Defining attribute groups used by samples & Insurance Model
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SAttributeGroupDefCreationICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: If the attribute have not yet been defined, please run
 *                    SAttributeDefinitionCreationICM first.  If the attribute groups are defined
 *                    already, please run SAttributeGroupDefDeletionICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SAttributeGroupDefCreationICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SAttributeGroupDefCreationICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("================================================");
        System.out.println("IBM Information Integrator for Content      v"+ver);
        System.out.println("Sample Program:  SAttributeGroupDefCreationICM");
        System.out.println("------------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("================================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //--------------------------------------------------------------
            // Define an Attribute Group Definition
            //--------------------------------------------------------------
            // Note, in this sample, the prefix "S_" will be used to indicate
            // that it is used by and was create by the samples.  This helps 
            // avoid using attribute group names that may already be in use.
            //
            // For more information on Retrieving Attribute Definitions, please
            // consult Sample SAttributeDefinitionRetrievalICM.
            //
            // Defining "Car Accident" attribute group for the Insurance Model's
            // Adjuster Report.  It contains two attributes, Accident Date, and
            // License Plate.
            System.out.println("Defining Attribute Group 'S_carAccident'...");

                DKAttrGroupDefICM attrGroup = new DKAttrGroupDefICM(dsICM); // create a new attribute group
                attrGroup.setName("S_carAccident");               // Set a name, maximum 15 characters
                attrGroup.setDescription("Car Accident");         // For more detail than name
                
                // Retrieve Attribute Definitions For Those to Add to Group
                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKAttrDefICM licensePlate  = (DKAttrDefICM) dsDefICM.retrieveAttr("S_licensePlate"); // Retrieve the definition.
                DKAttrDefICM accidentDate  = (DKAttrDefICM) dsDefICM.retrieveAttr("S_accidentDate"); // Retrieve the definition.
                
                attrGroup.addAttr(licensePlate);  // Add the Attribute Definitions to the Attribute Group
                attrGroup.addAttr(accidentDate);
                
                attrGroup.add();                  // add it to the persistent datastore

            System.out.println("Defined Attribute Group 'S_carAccident'.");

            //--------------------------------------------------------------
            // Define another Attribute Group
            //--------------------------------------------------------------
            // Defining "Address" attribute group.  It contains attributes 
            // that make up an address.
            System.out.println("Defining Attribute Group 'S_address'...");

                attrGroup = new DKAttrGroupDefICM(dsICM);     // create a new attribute group
                attrGroup.setName("S_address");               // Set a name, maximum 15 characters
                attrGroup.setDescription("Address");          // For more detail than name
                
                // Retrieve Attribute Definitions For Those to Add to Group
                DKAttrDefICM street  = (DKAttrDefICM) dsDefICM.retrieveAttr("S_street"); // Retrieve the definition.
                DKAttrDefICM city    = (DKAttrDefICM) dsDefICM.retrieveAttr("S_city"); 
                DKAttrDefICM state   = (DKAttrDefICM) dsDefICM.retrieveAttr("S_state"); 
                DKAttrDefICM zipCode = (DKAttrDefICM) dsDefICM.retrieveAttr("S_zipCode"); 
                
                attrGroup.addAttr(street);  // Add the Attribute Definitions to the Attribute Group
                attrGroup.addAttr(city);
                attrGroup.addAttr(state);
                attrGroup.addAttr(zipCode);
                
                attrGroup.add();                  // add it to the persistent datastore

            System.out.println("Defined Attribute Group 'S_address'.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
    
}//end class SAttributeGroupDefCreationICM
              
