/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Deletes All Items Created of Item Types of the Sample Data Model.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SSampleModelItemDeletionICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deletes All Items Created of the Item Types of the Sample Data Model.  
 *                    Cleans up to wipe data base clean of Sample Items.  The Definitions 
 *                    will remain.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Cleans up after Samples.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SSampleModelItemDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: None
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SItemTypeRetrievalICM.java
 *                    SItemCreationICM.java
 *                    SItemDeletionICM.java
 ************************************************************************************************/
public class SSampleModelItemDeletionICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE; // Use defaults found in SConnectDisconnectICM
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD; 

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SSampleModelItemDeletionICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Test Program: SSampleModelItemDeletionICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================\n");
        
        try{

            java.lang.String args[] = new java.lang.String[3];
            // specify the arguments to pass to object's main method
            args[0] = database;
            args[1] = userName;
            args[2] = password;

            // Run Samples
            
            // Delete Items
            DKDatastoreICM dsICM = SConnectDisconnectICM.connect(database,userName,password,"");
            SItemDeletionICM.deleteAllItems(dsICM, "S_simple");
            SItemDeletionICM.deleteAllItems(dsICM, "S_withMultAttrs");
            SItemDeletionICM.deleteAllItems(dsICM, "S_autoClaim");
            SItemDeletionICM.deleteAllItems(dsICM, "S_adjustrReport");
            SItemDeletionICM.deleteAllItems(dsICM, "S_policeReport");
            SItemDeletionICM.deleteAllItems(dsICM, "S_insurePolicy");
            SItemDeletionICM.deleteAllItems(dsICM, "S_autoPhoto");
            SItemDeletionICM.deleteAllItems(dsICM, "S_docModel");
            SItemDeletionICM.deleteAllItems(dsICM, "S_lob");
            SItemDeletionICM.deleteAllItems(dsICM, "S_text");
            SItemDeletionICM.deleteAllItems(dsICM, "S_image");
            SItemDeletionICM.deleteAllItems(dsICM, "S_withChild");
            SItemDeletionICM.deleteAllItems(dsICM, "S_videoType");
            SConnectDisconnectICM.disconnect(dsICM);
            
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Test program completed.");
            System.out.println("==========================================\n");

        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        } catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

}//end class SSampleModelDeleteICM
              
