
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Description    : Delete the documents from the sample folder, then delete
 *                  the folder itself.
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;

public class TDeleteSampIP implements DKConstantIP
{

   // Main method
   public static void main(String argv[])
   {


      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      if (argv.length < 3)
      {
         System.out.println("Invoke Syntax: " );
         System.out.println("  java TDeleteSampIP <libsrv> <userid> <pw>" );
         return;
      }

      String libSrv = argv[0];
      String userid = argv[1];
      String pw     = argv[2];

      DKDatastoreIP dsIP = null;
      try
      {
         System.out.println(" *** ImagePlus Connector *** ");

         dsIP = new DKDatastoreIP();
         dsIP.setOption( DK_IP_OPT_INCLUDE_PENDING_DOCS, new Integer(DK_IP_PENDING_YES) );
         System.out.println("---Connecting to ImagePlus server---");

         String connectString = new String("APPL=01;FAFIP=9.67.43.83;IODMIP=9.67.43.83;FAFPORT=3061;IODMPORT=3082;FAFPROT=4000;IODMPROT=4000;FAFSITE=CS61");
         dsIP.connect(libSrv, userid, pw, connectString);
         System.out.println("datastore ImagePlus connected");

         String pcmd = null;
         DKDDO pDDO  = null;
         DKPid pid = null;
         short i = 0;
         short j = 0;
         short k = 0;
         int numDataItems = 0;
         Object a = null;
         dkCollection pCol = null;
         String strDataName;
         dkIterator pIter = null;
         dkDataObjectBase pDO = null;

         byte[] tmpIn;
         tmpIn = new byte[3];

         // Build the folder pid...
         pDDO = new DKDDO( dsIP, "FOLDER" );
         pid = pDDO.getPidObject();
         pid.setIdStringCount( DK_IP_PID_COUNT_FOLD );
         pid.setIdString( DK_IP_PID_FOLD_ID, "IPSAMP" );
         pDDO.setPidObject(pid);
         pDDO.retrieve();

         numDataItems = pDDO.dataCount();
         k = pDDO.propertyId(DK_PROPERTY_ITEM_TYPE);
         if (k > 0)
         {
            Short sVal = (Short)pDDO.getProperty(k);
            j = sVal.shortValue();
            switch (j)
            {
               case DK_DOCUMENT :
               {
                  System.out.println("Item is document ");
                  break;
               }
               case DK_FOLDER :
               {
                  System.out.println("Item is folder ");
                  break;
               }
            }  // end switch...
         }  // end valid property

         System.out.println("Pid                  : " + pDDO.getPidObject().getPrimaryId());
         System.out.println("Number of Data Items : " + numDataItems);
         for (j = 1; j <= numDataItems; j++)
         {
            a = pDDO.getData(j);
            strDataName = pDDO.getDataName(j);
            System.out.println(j + ": Attribute Name   : " + strDataName );

            if (a instanceof Integer)
            {
               System.out.println("   Integer Value    : " + a);
            }
            else if (a instanceof Short)
            {
               System.out.println("   Short Value      : " + a);
            }
            else if (a instanceof DKTimestamp)
            {
               System.out.println("   Timestamp value  : " + a);
            }
            else if (a instanceof DKDate)
            {
               System.out.println("   Date value       : " + a);
            }
            else if (a instanceof String)
            {
               System.out.println("   String Value     : " + a);
            }
            else if (a instanceof dkCollection)
            {
               System.out.println("   Collection Values: ");
               pCol = (dkCollection)a;
               pIter = pCol.createIterator();
               i = 0;
               while (pIter.more() == true)
               {
                  i++;
                  a = pIter.next();
                  pDO = (dkDataObjectBase)a;
                  if (pDO.protocol() == DK_PDDO)
                  {
                     System.out.println( "    " + i + ": DKDDO primary id  = "
                                       + ((DKDDO)pDO).getPid().getPrimaryId());

                     System.out.print( "Do you want to delete this document? (y/n)");
                     System.in.read( tmpIn );
                     if (  tmpIn[0] == 'y'
                        || tmpIn[0] == 'Y' )
                     {
                        // DELETE:  Now delete the doc...
                        dsIP.deleteObject( (DKDDO)pDO );
                        System.out.println("Document deleted.");
                     }
                  }
               } // end while
            } // end if collection
            else if (a != null)
            {
               System.out.println("   Attribute Value  : " + a.toString());
            }
            else
            {
               System.out.println("   Attribute Value  : <null>");
            }
         }  // endfor

         System.out.print( "Do you want to delete this folder? (y/n)");
         System.in.read( tmpIn );
         if (  tmpIn[0] == 'y'
            || tmpIn[0] == 'Y' )
         {
            // DELETE:  Now delete the folder...
            dsIP.deleteObject( pDDO );
            System.out.println("Folder deleted.");
         }

         dsIP.disconnect();
         System.out.println("datastore ImagePlus disconnected");
         dsIP.destroy();
      }
      catch (DKException exc)
      {
     	  try
         {
            if ( dsIP != null )
               dsIP.destroy();
     	  }
     	  catch (Exception e)
     	  {
     	     e.printStackTrace();
     	  }
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
      catch (Exception exc)
      {
     	  try
         {
            if ( dsIP != null )
               dsIP.destroy();
     	  }
     	  catch (Exception e)
     	  {
     	     e.printStackTrace();
     	  }
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
   }
}

