/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************
 */

// sample connect/execute/disconnect to OD

/*
 * Filename       : TClientCQMapExecuteOD.java
 * Description    : Connect to OD Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/
 *                   folders.
 *                  Disconnect from OD Datastore.
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.client.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TClientCQMapExecuteOD class
public class TClientCQMapExecuteOD implements DKConstant
{
  static String strEntityName = new String("");
  static String strAttrName   = new String("");
  static short  sOpCode       = 0;
  static String strAttrValue  = new String("");

  // Main method
  public static void main(String argv[])
  {
    int MaxCount = 5;

    // Indicate this is a Console Application on Windows NT/95 for error
    // message display in dos console code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_CM_SS_WINDOWS is the default.
    DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    DKDatastoreOD dsOD = null;
    try
    {
      //*** for input parameters
      String libSrv = new String(""),     // library server
             userid = new String(""),     // userid
             pw     = new String(""),     // pw
             constr = new String("");     // connection string

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length < 3)
      {
        System.out.println("Invoke Syntax: " );
        System.out.println("  java TClientCQMapExecuteOD " +
                           "<libSrv> <userid> <pw> <connect_string>" );
        return;
      }
      else
      {
        libSrv = argv[0];
        userid = argv[1];
        pw = argv[2];
        if (argv.length > 3)
          constr = argv[3];
      }
      dsOD = new DKDatastoreOD();
      dkResultSetCursor pCur = null;
      dkCollection pCol = null;
      dkIterator pIter = null;
      DKNVPair soptions[] = new DKNVPair[1];
      DKNVPair qoptions[] = new DKNVPair[1];
      DKNVPair tnv = null;
      dkEntityDef entDef = null;
      dkAttrDef attrDef = null;
      dkDatastoreDef dsDef = null;

      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      // Select a testcase by changing this number!!!
      int testcase = 1;
      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      setAGQueryValues(testcase, libSrv);

      String strMappedEntityName = "FedEntity";
      String strMappedAttrName = "FedAttrName";
      DKSchemaMappingFed mapFed = new DKSchemaMappingFed();
      DKAttrMappingFed attrMapFed = new DKAttrMappingFed();
      mapFed.setServerName(libSrv);
      mapFed.setServerType(DK_DSTYPE_OD);
      mapFed.setEntityName(strEntityName);
      mapFed.setMappedEntityName(strMappedEntityName);
      mapFed.setMappedEntityType(DKConstantFed.MAPPED);
      DKCQExpr cqe = new DKCQExpr();
      DKQExpr qe = new DKQExpr();
      DKQTerm qterm = new DKQTerm(strMappedAttrName,sOpCode,strAttrValue);
      DKQSubExpr sqe = new DKQSubExpr();
      int i = 0;
      String strTmp = "";

      System.out.println("\nstrEntityName =>" + strEntityName + "<=");
      System.out.println("qterm =>" + qterm + "<=\n");

      System.out.println("connecting to datastore");
      dsOD.connect(libSrv,userid,pw,constr);
      System.out.println("datastore connected libSrv " + libSrv + " userid " +
                         userid );

      dsDef = dsOD.datastoreDef();
      entDef = dsDef.getEntity(strEntityName);
      if (entDef == null)
      {
        System.out.println("Native entity "+strEntityName+" does not exist!!!");
        return;
      }
      attrDef = entDef.createAttr();
      if (attrDef == null)
      {
        System.out.println("attrDef is null");
        return;
      }
      attrDef.setName(strMappedAttrName);
      attrDef.setType(DK_CM_VSTRING);
      attrMapFed.addMappedAttr(attrDef); // Add LHS

      pCol = dsDef.listEntityAttrs(strEntityName);
      pIter = pCol.createIterator();
      i = 0;
      boolean fDone = false;
      String strAttrDefName = "";
      while (pIter.more() == true)
      {
        i++;
        attrDef = (dkAttrDef)pIter.next();
        strAttrDefName = attrDef.getName();
        if (strAttrDefName.equals(strAttrName))
        {
          attrMapFed.addMapToAttr(attrDef); // Add RHS
        }
      }

      mapFed.addAttrMapping(attrMapFed);

      DKNVPair sourceMap = new DKNVPair("SCHEMA",mapFed);
      dsOD.registerMapping(sourceMap);
      cqe.setQLType(DK_CM_PARAMETRIC_QL_TYPE);
      qe.setQLType(DK_CM_PARAMETRIC_QL_TYPE);
      sqe.setTranslation(true);
      sqe.setMappingName(mapFed.getName());
      sqe.setEntityName(strMappedEntityName);
      sqe.setQueryTerm(qterm);

      // Display list- uncomment to display a set
      //sqe.setDisplayList(new String[]{strMappedAttrName,strEntityName});

      tnv = new DKNVPair("MAX_RESULTS",new String(Integer.toString(MaxCount)));
      soptions[0] = tnv;
      sqe.setOptionList(soptions);
      qe.addSubQuery(sqe);

      tnv = new DKNVPair("CONTENT","YES");
      qoptions[0] = tnv;
      qe.setOptionList(qoptions);
      cqe.setQueryExpr(qe);

      DKDDO p = null;
      long  t = 0;
      int cnt = 0;

      System.out.println("executing query");
      Date date;
      date  = new Date();
      System.out.println("before query time " + date.toString());
      pCur = dsOD.execute(cqe);
      date  = new Date();
      System.out.println("After query time " + date.toString());
      System.out.println("cardinality " + pCur.cardinality());
      System.out.println("datastore executed query");

      if (pCur == null)
      {
        System.out.println("cursor null");
      }

      Vector hitListVector = new Vector();
      t = (new Date()).getTime();
      while (pCur.isValid())
      {
        p = pCur.fetchNext();
        if (p != null)
        {
          i = pCur.getPosition();
          hitListVector.addElement((Object)p);
          displayHit(i, p);
        }
      } // while (pCur.isValid())

      t = (new Date()).getTime() - t;
      System.out.println("\nFetch elapsed time: " + t + " msec\n");

      t = (new Date()).getTime();
      for (i=0; i<hitListVector.size(); i++)
      {
        cnt++;
        p = (DKDDO)hitListVector.elementAt(i);
        String idstr = ((DKPid)p.getPidObject()).pidString();
        System.out.println(" pidString : " + idstr);
        showDDO(cnt, p);
      }
      System.out.println("Total number of items processed: " + cnt );
      hitListVector.removeAllElements();

      pCur.destroy(); // Finished with the cursor
      date  = new Date();
      System.out.println("After results processed " + date.toString());
      System.out.println("query results processed");
      System.out.println("Number of items " + cnt );
      dsOD.disconnect();
      System.out.println("datastore disconnected");

      dsOD.destroy(); // Finished with the datstore
    }
    catch (DKException exc)
    {
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      System.out.println("Exception error code " + exc.errorCode());
      System.out.println("Exception error state " + exc.errorState());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }
  }

  // --------------------------------------------------------------------
  // Convert regular month, day, year values to an OnDemand integer date
  // Asumming (and hopefully) the year is in the yyyy format
  // --------------------------------------------------------------------
  public static String ODdate( int month, int day, int year )
  {
    int[] days = {31,28,31,30,31,30,31,31,30,31,30,31};
    int OD_days = 0;
    String od_days_str;
    Integer OD_DAY = new Integer(0);

    // Just in case someone entered a 2 digit year
    if ( year < 100 )
    {
      // 21st century
      if (year < 70)
        year += 2000;
      else
        year += 1900;
    }

    // Get the numbers of days since 1/1/1970 to 1/1/yyyy
    OD_days = ( year - 1970 ) * 365;

    // Add days from leap years
    for ( int yr = 1970; yr < year; yr++ )
    {
      if ((((yr) % 4) == 0) && ((((yr) % 100) != 0) || (((yr) % 400) == 0)))
      {
        OD_days++;
      }
    }

    // Add days sicne 1/1 of this year
    for ( int mon = 0; mon < month-1; mon++ )
    {
      OD_days += days[mon];
    }

    // Add the current day
    OD_days += day;

    // Additional Leap year adjustment
    if ((((year) % 4) == 0) && ((((year) % 100) != 0) || (((year) % 400) == 0)) && month > 2)
      OD_days++;

    od_days_str = OD_DAY.toString(OD_days);
    return od_days_str;
  }

  // --------------------------------------------------------------------
  // display hits from the search
  // --------------------------------------------------------------------
  public static void displayHit(int cnt, DKDDO p)
  {
    short j = 0;
    if (p != null)
    {
      try
      {
        int numDataItems = numDataItems = p.dataCount();
        // display attribute (field) names (1st time only)
        if (cnt == 1)
        {
          System.out.println("");
          for (j = 1; j <= numDataItems; j++)
          {
            String strDataName = p.getDataName(j);
            if (strDataName.equals(DK_CM_DKPARTS) == false &&
                strDataName.equals("DKResource") == false &&
                strDataName.equals("DKViews") == false &&
                strDataName.equals("DKLargeObject") == false &&
                strDataName.equals("DKFixedView") == false &&
                strDataName.equals("DKAnnotations") == false)
            {
              System.out.print("  \t\""+strDataName+"\"");
            }
          }
          System.out.println("");
        }
        // display attribute (field) values
        for (j = 1; j <= numDataItems; j++)
        {
          Object a = p.getData(j);
          String strDataName = p.getDataName(j);
          if (strDataName.equals(DK_CM_DKPARTS) == false &&
              strDataName.equals("DKResource") == false &&
              strDataName.equals("DKViews") == false &&
              strDataName.equals("DKLargeObject") == false &&
              strDataName.equals("DKFixedView") == false &&
              strDataName.equals("DKAnnotations") == false)
          {
            if (j == 1)
              System.out.print(" "+cnt+".\t");
            else
              System.out.print("\t");

            if ((a instanceof String)  ||
                (a instanceof Integer) ||
                (a instanceof Short)   ||
                (a instanceof Long)    ||
                (a instanceof Float)   ||
                (a instanceof Double)  ||
                (a instanceof DKDate)  ||
                (a instanceof DKTime)  ||
                (a instanceof DKTimestamp))
            {
              System.out.print("\"" + a + "\"");
            }
            else
              System.out.print("");
          }
        }
        System.out.println("");
      }
      catch (DKException exc)
      {
        System.out.println("Exception name " + exc.name());
        System.out.println("Exception message " + exc.getMessage());
        System.out.println("Exception error code " + exc.errorCode());
        System.out.println("Exception error state " + exc.errorState());
        exc.printStackTrace();
      }
      catch (Exception exc)
      {
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
      }
    }
  }

  // --------------------------------------------------------------------
  // display results
  // --------------------------------------------------------------------
  public static void showDDO(int cnt, DKDDO p)
  {
    DKDDO pDDO = null;
    dkXDO pXDO = null;
    int i = 0;
    int numDataItems = 0;
    short k = 0;
    short j = 0;
    String strDataName;
    dkCollection pCol = null;
    dkIterator pIter = null;
    Object a = null;
    dkDataObjectBase pDO = null;

    try
    {
      System.out.println("\n=====> Item #" + cnt + " <=====");
      numDataItems = p.dataCount();
      DKPid pid = p.getPidObject();
      System.out.println("  pid string: " + pid.pidString());
      k = p.propertyId(DK_CM_PROPERTY_ITEM_TYPE);

      if (k > 0)
      {
        Short sVal = (Short)p.getProperty(k);
        j = sVal.shortValue();
        switch (j)
        {
          case DK_CM_DOCUMENT :
            System.out.println("  Item is a document ");
            break;
          case DK_CM_ITEM :
            System.out.println("  Item is an item ");
            break;
          default:
            System.out.println("  Unknown property: "+k);
        }
      }

      System.out.println("  Number of Data Items: " + numDataItems);

      for (j = 1; j <= numDataItems; j++)
      {
        a = p.getData(j);
        strDataName = p.getDataName(j);
        System.out.println("    " + j + ". Attribute Name: " + strDataName );
        System.out.println("       type: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_TYPE));
        System.out.println("       nullable: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_NULLABLE));

        if (strDataName.equals(DK_CM_DKPARTS) == false &&
            strDataName.equals("DKResource") == false &&
            strDataName.equals("DKViews") == false &&
            strDataName.equals("DKLargeObject") == false &&
            strDataName.equals("DKFixedView") == false &&
            strDataName.equals("DKAnnotations") == false)
        {
          System.out.println("       attribute id: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_ATTRIBUTE_ID));
        }

        if (a instanceof String)
        {
          System.out.println("       Attribute Value(String): =>" + a + "<=");
        }
        else if (a instanceof Integer)
        {
          System.out.println("       Attribute Value(Integer): " + a);
        }
        else if (a instanceof Short)
        {
          System.out.println("       Attribute Value(Short): " + a);
        }
        else if (a instanceof Long)
        {
          System.out.println("       Attribute Value(Long): " + a);
        }
        else if (a instanceof Float)
        {
          System.out.println("       Attribute Value(Float): " + a);
        }
        else if (a instanceof Double)
        {
          System.out.println("       Attribute Value(Double): " + a);
        }
        else if (a instanceof DKDate)
        {
          System.out.println("      Attribute Value(DKDate): " + a);
        }
        else if (a instanceof DKTime)
        {
          System.out.println("       Attribute Value(DKTime): " + a);
        }
        else if (a instanceof DKTimestamp)
        {
          System.out.println("       Attribute Value(DKTimestamp): " + a);
        }
        else if (a instanceof dkCollection)
        {
          System.out.println("       Attribute Value is collection");
          pCol = (dkCollection)a;
          pIter = pCol.createIterator();
          i = 0;

          while (pIter.more() == true)
          {
            i++;
            a = pIter.next();
            pDO = (dkDataObjectBase)a;

            if (pDO.protocol() == DK_CM_XDO)
            {
              System.out.println("         dkXDO object " + i + " in collection");
              pXDO = (dkXDO)pDO;
              DKPid pid2 = pXDO.getPidObject();
              System.out.println("           XDO pid string: " + pid2.pidString());

              // Retrieve and open instance handler for an XDO
              pXDO.retrieve();
              // pXDO.open();
            }
          }
        }
        else if (a != null)
        {
          System.out.println("       Attribute Value: " + a.toString());
          if (strDataName.equals("DKResource") ||
              strDataName.equals("DKFixedView") ||
              strDataName.equals("DKLargeObject"))
          {
            pDO = (dkDataObjectBase)a;

            if (pDO.protocol() == DK_CM_XDO)
            {
              System.out.println("         dkXDO object ");
              pXDO = (dkXDO)pDO;
              DKPid pid2 = pXDO.getPidObject();
              System.out.println("           XDO pid string: " + pid2.pidString());

              // Retrieve and open instance handler for an XDO
              pXDO.retrieve();
              // pXDO.open();
            }
          }
        }
        else System.out.println("       Attribute Value is null");
      }
    }
    catch (DKException exc)
    {
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      System.out.println("Exception error code " + exc.errorCode());
      System.out.println("Exception error state " + exc.errorState());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }
  }

  //--------------------------------------------------------------
  // set the application group query criteria
  //--------------------------------------------------------------
  public static void setAGQueryValues(int testcase, String libSrv)
  {
    // Application Group and SQL string
    // OD/390 v2.1
    if (libSrv.equalsIgnoreCase("9.91.121.31") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // AFP
          strEntityName = "INSURE  01 ";
          strAttrName   = "POLICY_NUMBER";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'324-1443255%'";
          break;
        case 2:
          strEntityName = "CHKSTMN301 ";
          strAttrName   = "ACCOUNT_NUMBER";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'%'";
          break;
        case 3:  // line/large object
          strEntityName = "DELINQCY01 ";
          strAttrName   = "LOAN_NUMBER";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'%'";
          break;
        case 4:  // pdf
          strEntityName = "KEYBANK 01 ";
          strAttrName   = "ACCOUNT_NUMBER";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'%'";
      }
    }
    // OD/390 v7.1
    else if (libSrv.equalsIgnoreCase("9.91.121.34") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // AFP data
          strEntityName = "Credit Card Statements";
          strAttrName   = "name";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'%'";
          break;
        case 2:
          strEntityName = "CHKSTMNT";
          strAttrName   = "ACCOUNT_NUMBER";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'%'";
          break;
        case 3:  // pdf
          strEntityName = "PDF landscape";
          strAttrName   = "region";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'%'";
          break;
      }
    }
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("everest"))
    {
      switch (testcase)
      {
        default:
        case 1:
          // AFP data
          strEntityName = "CRD";
          strAttrName   = "name";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'%'";
          break;
        case 2:
          // Large Object line data
          strEntityName = "sp-ebcdic-lo";
          strAttrName   = "rpt_date";
          sOpCode       = DKOpCode.EQ;
          strAttrValue  = "2000-12-15";
          break;
      }
    }
    else if (libSrv.equalsIgnoreCase("gunnar") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // Large object AFP - gunnar
          strEntityName = "OnDemand Publications";
          strAttrName   = "bookname";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'%'";
          break;
      }
    }
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("rdr400c") == true)
    {
      switch (testcase)
      {
        default:
        case 1:
          strEntityName = "CHECKSTMTS ";
          strAttrName   = "KY1VAL";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'%'";
          break;
      }
    }
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("rdr400g") == true)
    {
      switch (testcase)
      {
        default:
        case 1:
          strEntityName = "CheckStatements";
          strAttrName   = "name";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'%'";
          break;
      }
    }
    else if (libSrv.length() >= 5 &&
             libSrv.substring(0,5).equalsIgnoreCase("akira") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // AFP
          strEntityName = "Few Credit Card Statements";
          strAttrName   = "name";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'%'";
          break;
      }
    }
    else if (libSrv.length() >= 6 &&
             libSrv.substring(0,6).equalsIgnoreCase("reagan") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // AFP
          strEntityName = "load-db2-CRD";
          strAttrName   = "account";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'%'";
          break;
      }
    }
    else // can be any server!
    {
      switch (testcase)
      {
        default:
        case 1: // AFP
          strEntityName = "CRD";
          strAttrName   = "account";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'000-000-00%'";
          break;
        case 2:  // line
          strEntityName = "Balance";
          strAttrName   = "hiercode";
          sOpCode       = DKOpCode.LIKE;
          strAttrValue  = "'..21...002'";
          break;
      }
    }
  }
}

