/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************
 */

// sample connect/execute/disconnect to OnDemand

/*
 * Filename       : TClientExecuteOD.java
 * Description    : Connect to OnDemand Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                  if folders found display them and their documents/
 *                  folders.
 *                  Disconnect from OD Datastore.
 *
 *                  configuration string can be
 *                      CC2MIMEURL=(File:///c:\cmbroot\cmbcc2mime.ini);
 *                    or
 *                      CC2MIMEURL=(HTTP://www.my.com/cmbcc2mime.ini);
 *                    or
 *                      CC2MIMEFILE=(c:\cmbroot\cmbcc2mime.ini);
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.client.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TClientExecuteOD class
public class TClientExecuteOD implements DKConstant
{
  static String appgrp = new String("");
  static String cmd = new String("");

  // Main method
  public static void main(String argv[])
  {
    int     MaxCount = 5;
    DKDatastoreOD dsOD = null;
    try
    {
      //*** for input parameters
      String libSrv = new String(""),     // library server
             userid = new String(""),     //userid
             pw     = new String(""),     // pw
             connect_string = new String(""),
             configuration_string = new String("");

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length < 3)
      {
        System.out.println("Invoke Syntax: " );
        System.out.println("  java TClientExecuteOD " + "<libSrv> <userid> <pw> <connect string> <configuration string>" );
        return;
      }
      else
      {
        libSrv = argv[0];
        userid = argv[1];
        pw = argv[2];
        if (argv.length > 3)
          connect_string = argv[3];
        if (argv.length > 4)
          configuration_string = argv[4];
      }

      System.out.println("connection string=>" + connect_string + "<=");
      System.out.println("configuration string=>" + configuration_string + "<=");

      if (configuration_string.length() > 0)
      {
        dsOD = new DKDatastoreOD(configuration_string);
      }
      else
      {
        dsOD = new DKDatastoreOD();
      }
      dkResultSetCursor pCur = null;

      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      // Select a testcase by changing this number!!!
      int testcase = 10;
      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      setAGQueryCriteria(testcase, libSrv);

      DKNVPair[] parms = new DKNVPair[3];
      parms[0] = new DKNVPair("APPL_GROUP", appgrp);
      parms[1] = new DKNVPair("MAX_RESULTS",
                 new String(Integer.toString(MaxCount)));
      parms[2] = new DKNVPair("CONTENT", new String("YES"));

      System.out.println("\nApplication group =>" + appgrp + "<=");
      System.out.println("Query string =>" + cmd + "<=\n");

      System.out.println("connecting to datastore");
      dsOD.connect(libSrv,userid,pw,connect_string);
      System.out.println("datastore connected, libSrv: " + libSrv + ", userid: " +
                         userid );

      DKDDO p = null;
      int i = 0;
      long  t = 0;
      int cnt = 0;
      System.out.println("executing query");

      Date date;
      date  = new Date();
      System.out.println("Before query, time is " + date.toString());
      pCur = dsOD.execute(cmd,DK_CM_SQL_QL_TYPE,parms);
      date  = new Date();
      System.out.println("After query, time is " + date.toString());
      System.out.println("cardinality: " + pCur.cardinality());
      System.out.println("datastore executed query");

      if (pCur == null)
      {
        System.out.println("cursor is null!!!");
      }

      Vector hitListVector = new Vector();
      t = (new Date()).getTime();
      while (pCur.isValid())
      {
        p = pCur.fetchNext();

        if (p != null)
        {
          i = pCur.getPosition();
          hitListVector.addElement((Object)p);
          displayHit(i, p);
        }
      }
      t = (new Date()).getTime() - t;
      System.out.println("\nFetch elapsed time: " + t + " msec\n");

      t = (new Date()).getTime();
      for (i=0; i<hitListVector.size(); i++)
      {
        cnt++;
        p = (DKDDO)hitListVector.elementAt(i);
        String pidStr = ((DKPid)p.getPidObject()).pidString();
        System.out.println(" pidString : " + pidStr);
        showDDO(cnt, p);
      }
      System.out.println("Total number of items processed: " + cnt );
      hitListVector.removeAllElements();

      pCur.destroy(); // Finished with the cursor

      date  = new Date();
      System.out.println("\nAfter results processed " + date.toString());
      System.out.println("\nquery results processed");
      System.out.println("Number of items retrieved: " + cnt );
      dsOD.disconnect();
      System.out.println("\ndatastore disconnected");

      dsOD.destroy(); // Finished with the datstore
    }
    catch (DKException exc)
    {
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      System.out.println("Exception error code " + exc.errorCode());
      System.out.println("Exception error state " + exc.errorState());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }
  }

  // --------------------------------------------------------------------
  // display hits from the search
  // --------------------------------------------------------------------
  public static void displayHit(int cnt, DKDDO p)
  {
    short j = 0;
    if (p != null)
    {
      try
      {
        int numDataItems = numDataItems = p.dataCount();
        // display attribute (field) names (1st time only)
        if (cnt == 1)
        {
          System.out.println("");
          for (j = 1; j <= numDataItems; j++)
          {
            String strDataName = p.getDataName(j);
            if (strDataName.equals(DK_CM_DKPARTS) == false &&
                strDataName.equals("DKResource") == false &&
                strDataName.equals("DKViews") == false &&
                strDataName.equals("DKLargeObject") == false &&
                strDataName.equals("DKFixedView") == false &&
                strDataName.equals("DKAnnotations") == false)
            {
              System.out.print("  \t\""+strDataName+"\"");
            }
          }
          System.out.println("");
        }
        // display attribute (field) values
        for (j = 1; j <= numDataItems; j++)
        {
          Object a = p.getData(j);
          String strDataName = p.getDataName(j);
          if (strDataName.equals(DK_CM_DKPARTS) == false &&
              strDataName.equals("DKResource") == false &&
              strDataName.equals("DKViews") == false &&
              strDataName.equals("DKLargeObject") == false &&
              strDataName.equals("DKFixedView") == false &&
              strDataName.equals("DKAnnotations") == false)
          {
            if (j == 1)
              System.out.print(" "+cnt+".\t");
            else
              System.out.print("\t");

            if ((a instanceof String)  ||
                (a instanceof Integer) ||
                (a instanceof Short)   ||
                (a instanceof Long)    ||
                (a instanceof Float)   ||
                (a instanceof Double)  ||
                (a instanceof DKDate)  ||
                (a instanceof DKTime)  ||
                (a instanceof DKTimestamp))
            {
              System.out.print("\"" + a + "\"");
            }
            else
              System.out.print("");
          }
        }
        System.out.println("");
      }
      catch (DKException exc)
      {
        System.out.println("Exception name " + exc.name());
        System.out.println("Exception message " + exc.getMessage());
        System.out.println("Exception error code " + exc.errorCode());
        System.out.println("Exception error state " + exc.errorState());
        exc.printStackTrace();
      }
      catch (Exception exc)
      {
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
      }
    }
  }

  // --------------------------------------------------------------------
  // display results
  // --------------------------------------------------------------------
  public static void showDDO(int cnt, DKDDO p)
  {
    DKDDO pDDO = null;
    dkXDO pXDO = null;
    int i = 0;
    int numDataItems = 0;
    short k = 0;
    short j = 0;
    String strDataName;
    dkCollection pCol = null;
    dkIterator pIter = null;
    Object a = null;
    dkDataObjectBase pDO = null;

    try
    {
      System.out.println("\n=====> Item #" + cnt + " <=====");
      numDataItems = p.dataCount();
      DKPid pid = p.getPidObject();
      System.out.println("  pid string: " + pid.pidString());
      k = p.propertyId(DK_CM_PROPERTY_ITEM_TYPE);

      if (k > 0)
      {
        Short sVal = (Short)p.getProperty(k);
        j = sVal.shortValue();
        switch (j)
        {
          case DK_CM_DOCUMENT :
            System.out.println("  Item is a document ");
            break;
          default:
            System.out.println("  Unknown property: "+k);
        }
      }

      System.out.println("  Number of Data Items: " + numDataItems);

      for (j = 1; j <= numDataItems; j++)
      {
        a = p.getData(j);
        strDataName = p.getDataName(j);
        System.out.println("    " + j + ". Attribute Name: " + strDataName );
        System.out.println("       type: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_TYPE));
        System.out.println("       nullable: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_NULLABLE));

        if (strDataName.equals(DK_CM_DKPARTS) == false &&
            strDataName.equals("DKResource") == false &&
            strDataName.equals("DKViews") == false &&
            strDataName.equals("DKLargeObject") == false &&
            strDataName.equals("DKFixedView") == false &&
            strDataName.equals("DKAnnotations") == false)
        {
          System.out.println("       attribute id: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_ATTRIBUTE_ID));
        }

        if (a instanceof String)
        {
          System.out.println("       Attribute Value(String): =>" + a + "<=");
        }
        else if (a instanceof Integer)
        {
          System.out.println("       Attribute Value(Integer): " + a);
        }
        else if (a instanceof Short)
        {
          System.out.println("       Attribute Value(Short): " + a);
        }
        else if (a instanceof Long)
        {
          System.out.println("       Attribute Value(Long): " + a);
        }
        else if (a instanceof Float)
        {
          System.out.println("       Attribute Value(Float): " + a);
        }
        else if (a instanceof Double)
        {
          System.out.println("       Attribute Value(Double): " + a);
        }
        else if (a instanceof DKDate)
        {
          System.out.println("      Attribute Value(DKDate): " + a);
        }
        else if (a instanceof DKTime)
        {
          System.out.println("       Attribute Value(DKTime): " + a);
        }
        else if (a instanceof DKTimestamp)
        {
          System.out.println("       Attribute Value(DKTimestamp): " + a);
        }
        else if (a instanceof dkCollection)
        {
          System.out.println("       Attribute Value is collection");
          pCol = (dkCollection)a;
          pIter = pCol.createIterator();
          i = 0;

          while (pIter.more() == true)
          {
            i++;
            a = pIter.next();
            pDO = (dkDataObjectBase)a;

            if (pDO.protocol() == DK_CM_XDO)
            {
              System.out.println("         dkXDO object " + i + " in collection");
              pXDO = (dkXDO)pDO;
              DKPid pid2 = pXDO.getPidObject();
              System.out.println("           XDO pid string: " + pid2.pidString());

              // Retrieve and open instance handler for an XDO
              pXDO.retrieve();
              // pXDO.open();
            }
          }
        }
        else if (a != null)
        {
          System.out.println("       Attribute Value: " + a.toString());
          if (strDataName.equals("DKResource") ||
              strDataName.equals("DKFixedView") ||
              strDataName.equals("DKLargeObject"))
          {
            pDO = (dkDataObjectBase)a;

            if (pDO.protocol() == DK_CM_XDO)
            {
              System.out.println("         dkXDO object ");
              pXDO = (dkXDO)pDO;
              DKPid pid2 = pXDO.getPidObject();
              System.out.println("           XDO pid string: " + pid2.pidString());

              // Retrieve and open instance handler for an XDO
              pXDO.retrieve();
              // pXDO.open();
            }
          }
        }
        else System.out.println("       Attribute Value is null");
      }
    }
    catch (DKException exc)
    {
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      System.out.println("Exception error code " + exc.errorCode());
      System.out.println("Exception error state " + exc.errorState());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }
  }

  // --------------------------------------------------------------------
  // Convert regular month, day, year values to an OnDemand integer date
  // Asumming (and hopefully) the year is in the yyyy format
  // --------------------------------------------------------------------
  public static String ODdate( int month, int day, int year )
  {
    int[] days = {31,28,31,30,31,30,31,31,30,31,30,31};
    int OD_days = 0;
    String od_days_str;
    Integer OD_DAY = new Integer(0);

    // Just in case someone entered a 2 digit year
    if ( year < 100 )
    {
      // 21st century
      if (year < 70)
        year += 2000;
      else
        year += 1900;
    }

    // Get the numbers of days since 1/1/1970 to 1/1/yyyy
    OD_days = ( year - 1970 ) * 365;

    // Add days from leap years
    for ( int yr = 1970; yr < year; yr++ )
    {
      if ((((yr) % 4) == 0) && ((((yr) % 100) != 0) || (((yr) % 400) == 0)))
      {
        OD_days++;
      }
    }

    // Add days sicne 1/1 of this year
    for ( int mon = 0; mon < month-1; mon++ )
    {
      OD_days += days[mon];
    }

    // Add the current day
    OD_days += day;

    // Additional Leap year adjustment
    if ((((year) % 4) == 0) && ((((year) % 100) != 0) || (((year) % 400) == 0)) && month > 2)
      OD_days++;

    od_days_str = OD_DAY.toString(OD_days);
    return od_days_str;
  }

  // --------------------------------------------------------------------
  // set application group and query SQL
  // --------------------------------------------------------------------
  public static void setAGQueryCriteria(int testcase, String libSrv)
  {
    int month = 6;
    int day   = 2;
    int year  = 1997;
    Integer temp_int = new Integer(0);
    String  mon_str, day_str, yr_str, date_str;
    String od_date = "";

    // Application Group and SQL string
    // OD/390 v2.1
    if (libSrv.equalsIgnoreCase("9.91.121.31") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // AFP
          appgrp = "INSURE  01 ";
          cmd = "WHERE (POLICY_NUMBER LIKE '324-1443255%')";  //'324-1443255-11' ";
          break;
        case 2:
          appgrp = "CHKSTMN301 ";
          cmd = "WHERE ACCOUNT_NUMBER LIKE '%' ";
          break;
        case 3:  // line/large object
          appgrp = "DELINQCY02 ";
          cmd = "WHERE ( LOAN_NUMBER LIKE '%' )";
          break;
        case 4:  // pdf
          appgrp = "KEYBANK 01 ";
          cmd = "WHERE ( ACCOUNT_NUMBER LIKE '%' )";
          break;
      }
    }
    // OD/390 v7.1
    else if (libSrv.equalsIgnoreCase("9.91.121.34") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // AFP data
          appgrp = "Credit Card Statements";
          cmd = "WHERE ( name LIKE '%' )";
          break;
        case 2:
          appgrp = "CHKSTMNT";
          cmd = "WHERE ACCOUNT_NUMBER LIKE '%' ";
          break;
        case 3:  // line data
          appgrp = "Delinqcy-AG";
          cmd = "WHERE ( pagenum != 0 )";
          break;
        case 4:  // line data large object
          appgrp = "LOAN LO";
          od_date = ODdate( 10, 1, 1994 );
          cmd = "WHERE ( load_date = " + od_date + " )";
          //od_date = ODdate( 2, 2, 1970 );
          //cmd  = "WHERE (( load_date > " + od_date + ") AND ";
          //cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
          break;
        case 5:  // pdf
          appgrp = "PDF landscape";
          cmd = "WHERE ( region LIKE '%' )";
        case 6:  // AFP
          appgrp = "afpdata";
          cmd = "WHERE (TaxYear = 1995)";
          break;
      }
    }
    else if (libSrv.equalsIgnoreCase("gunnar") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // Large object AFP - gunnar
          appgrp = "OnDemand Publications";
          cmd = "where bookname LIKE '%'";
          break;
      }
    }
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("everest"))
    {
      switch (testcase)
      {
        default:
        case 1:
          // AFP data
          appgrp = "CRD";
          cmd = "WHERE ( name LIKE '%' )";
          break;
        case 2:
          // Large Object line data
          appgrp = "sp-ebcdic-lo";
          //od_date = ODdate( 1, 1, 1990 );
          //od_date2 = ODdate( 12, 31, 2001 );
          //cmd = "WHERE ( rpt_date BETWEEN " + od_date + " AND " + od_date2 + " )";
          od_date = ODdate( 12, 15, 2000 );
          cmd = "WHERE ( rpt_date > 0 )"; //= " + od_date + " )";
          break;
        case 3: // Loan Delinquency Reports - Large Objects
          appgrp = "LOAN LO";
          od_date = ODdate( 2, 2, 1970 );   // 2/2/1970
          cmd  = "WHERE (( load_date > " + od_date + ") AND ";
          cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
          break;
        case 4:
          // Large Object line data
          appgrp = "eliz-largeline";
          od_date = ODdate( 3, 19, 1999 );
          cmd = "WHERE ( rptdate = " + od_date + " )";
          break;
      case 5:
        // Large Object line data
        appgrp = "43MBPDF";
        cmd = "WHERE ( rtitle LIKE '%' )";
        break;
      }
    }
    // OD/400 v4.5
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("rdr400c") == true)
    {
      switch (testcase)
      {
        default:
        case 1:
          appgrp = "CHECKSTMTS ";
          cmd = "WHERE ( KY1VAL LIKE '%' )";
          break;
      }
    }
    // OD/400 v5.1
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("rdr400g") == true)
    {
      switch (testcase)
      {
        default:
        case 1:
          appgrp = "CheckStatements";
          cmd = "WHERE ( name LIKE '%' )";
          break;
      }
    }
    else if (libSrv.length() >= 5 &&
             libSrv.substring(0,5).equalsIgnoreCase("akira") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // AFP
          appgrp = "Few Credit Card Statements";
          cmd = "WHERE ( name LIKE '%' )";
          break;
        case 2: // Line - Loan Delinquency Reports
          appgrp = "Loan Delinquency Reports";
          od_date = ODdate( 2, 2, 1970 );   // 2/2/1970
          cmd  = "WHERE (( load_date > " + od_date + ") AND ";
          cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
          break;
      }
    }
    else // can be any server!
    {
      switch (testcase)
      {
        default:
        case 1: // AFP
          mon_str = temp_int.toString( month );
          day_str = temp_int.toString( day );
          yr_str  = temp_int.toString( year );
          date_str = mon_str + '/' + day_str + '/' + yr_str;
          // native OD integer date
          od_date = ODdate( month, day, year );
          System.out.println("The date '"+ date_str +"' equals to the OnDemand integer date '"+ od_date+"'" );

          appgrp = "CRD";
          //cmd = "WHERE ( crd_date = " + od_date + " )";
          cmd = "WHERE ( name LIKE '%')";
          break;
        case 2:  // line
          appgrp = "Balance";
          cmd = "WHERE ( hiercode LIKE '..21...002')"; //'%.....006')";
          break;
      }
    }
  }
}

