/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   *
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************
 */

// sample connect/execute/retrieve/disconnect to OnDemand

/*
 * Filename       : TRetrieveOD.java
 * Description    : Connect to OnDemand Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                  if folders found display them and their documents/
 *                  folders.
 *                  Disconnect from OD Datastore.
 *
 *                  configuration string can be
 *                      CC2MIMEURL=(File:///c:\cmbroot\cmbcc2mime.ini);
 *                    or
 *                      CC2MIMEURL=(HTTP://www.my.com/cmbcc2mime.ini);
 *                    or
 *                      CC2MIMEFILE=(c:\cmbroot\cmbcc2mime.ini);
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TRetrieveOD class
public class TRetrieveOD implements DKConstantOD
{
  static String appgrp = new String("");
  static String cmd = new String("");

  // Main method
  public static void main(String argv[])
  {
    int     MaxCount = 20;
    boolean retrieveToFile = false;

    DKDatastoreOD dsOD = null;
    try
    {
      //*** for input parameters
      String libSrv = new String(""),     // library server
             userid = new String(""),     //userid
             pw     = new String(""),     // pw
             connect_string       = new String(""),
             configuration_string = new String(""),
             testcase_string = new String("");


      // Select a testcase by changing this number!!!
      int testcase = 1;

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length < 3)
      {
        System.out.println("Invoke Syntax: " );
        System.out.println("  java TRetrieveOD " +
        "<libSrv> <userid> <pw> <connect string> <config string> <testcase>" );
        return;
      }
      else
      {
        libSrv = argv[0];
        userid = argv[1];
        pw = argv[2];
        if (argv.length > 3)
          connect_string = argv[3];
        if (argv.length > 4)
          configuration_string = argv[4];
        if (argv.length > 5)
        {
          testcase_string = argv[5];
          testcase = Integer.parseInt(testcase_string);
        }
      }
      System.out.println("connection string=>" + connect_string + "<=");
      System.out.println("configuration string=>" + configuration_string + "<=");

      if (configuration_string.length() > 0)
      {
        dsOD = new DKDatastoreOD(configuration_string);
      }
      else
      {
        dsOD = new DKDatastoreOD();
      }
      dkResultSetCursor pCur = null;

      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      // Set a testcase
      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      setAGQueryCriteria(testcase, libSrv);

      DKNVPair[] parms = new DKNVPair[3];
      parms[0] = new DKNVPair("APPL_GROUP", appgrp);
      parms[1] = new DKNVPair("MAX_RESULTS", new String(Integer.toString(MaxCount)));
      parms[2] = new DKNVPair("CONTENT", new String("ATTRONLY"));

      System.out.println("\nApplication group =>" + appgrp + "<=");
      System.out.println("Query string =>" + cmd + "<=\n");

      System.out.println("connecting to datastore");
      dsOD.connect(libSrv,userid,pw,connect_string);
      System.out.println("datastore connected, libSrv: " + libSrv + ", userid: " +
                         userid );

      DKDDO p = null;
      int i = 0;
      long  t = 0;
      int cnt = 0;
      System.out.println("executing query");

      Date date;
      date  = new Date();
      System.out.println("Before query, time is " + date.toString());
      t = (new Date()).getTime();
      pCur = dsOD.execute(cmd,DK_CM_SQL_QL_TYPE,parms);
      t = (new Date()).getTime() - t;
      System.out.println("Execute elapsed time: " + t + " msec");
      date  = new Date();
      System.out.println("After query, time is " + date.toString());
      System.out.println("cardinality: " + pCur.cardinality());
      System.out.println("datastore executed query");

      if (pCur == null)
      {
        System.out.println("cursor is null!!!");
      }

      Vector hitListVector = new Vector();
      t = (new Date()).getTime();
      while (pCur.isValid())
      {
        p = pCur.fetchNext();
        if (p != null)
        {
          i = pCur.getPosition();
          hitListVector.addElement((Object)p);
          displayHit(i, p);
        }
      } // while (pCur.isValid())

      t = (new Date()).getTime() - t;
      System.out.println("\nFetch elapsed time: " + t + " msec\n");

      t = (new Date()).getTime();
      for (i=0; i<hitListVector.size(); i++)
      {
        cnt++;
        p = (DKDDO)hitListVector.elementAt(i);

        // the retrieval action
        String idstr = ((DKPid)p.getPidObject()).pidString();
        System.out.println(" pidString : " + idstr);
        DKPid pid = new DKPid (idstr);
        DKDDO ddoold = p;
        short id, docType = 0;
        if ((id = ddoold.propertyId(DK_CM_PROPERTY_ITEM_TYPE)) > 0)
          docType = ((Short)ddoold.getProperty(id)).shortValue();
        if (docType == DK_CM_DOCUMENT)
        {
          System.out.println("create a new DDO with a cloned pid to retrieve!");
          p = dsOD.createDDO(ddoold.getObjectType(), DK_CM_DOCUMENT);
          p.setPidObject(pid);
          try
          {
            dsOD.retrieveObject((dkDataObject)p);
            displayDDO(cnt, dsOD, p, retrieveToFile);
          }
          catch (DKException exc)
          {
            System.out.println("Exception name " + exc.name());
            System.out.println("Exception message " + exc.getMessage());
            System.out.println("Exception error code " + exc.errorCode());
            System.out.println("Exception error state " + exc.errorState());
            exc.printStackTrace();
          }
        }
      }
      hitListVector.removeAllElements();

      pCur.destroy(); // Finished with the cursor

      t = (new Date()).getTime() - t;
      System.out.println("Retrieval elapsed time: " + t + " msec");

      date  = new Date();
      System.out.println("\nAfter results processed " + date.toString());
      System.out.println("\nquery results processed");
      System.out.println("Number of items retrieved: " + cnt );
      dsOD.disconnect();
      System.out.println("\ndatastore disconnected");

      dsOD.destroy(); // Finished with the datstore
    }
    catch (DKException exc)
    {
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      System.out.println("Exception error code " + exc.errorCode());
      System.out.println("Exception error state " + exc.errorState());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }
  }
  // --------------------------------------------------------------------
  // Convert regular month, day, year values to an OnDemand integer date
  // Asumming (and hopefully) the year is in the yyyy format
  // --------------------------------------------------------------------
  public static String ODdate( int month, int day, int year )
  {
    int[] days = {31,28,31,30,31,30,31,31,30,31,30,31};
    int OD_days = 0;
    String od_days_str;
    Integer OD_DAY = new Integer(0);

    // Just in case someone entered a 2 digit year
    if ( year < 100 )
    {
      // 21st century
      if (year < 70)
        year += 2000;
      else
        year += 1900;
    }

    // Get the numbers of days since 1/1/1970 to 1/1/yyyy
    OD_days = ( year - 1970 ) * 365;

    // Add days from leap years
    for ( int yr = 1970; yr < year; yr++ )
    {
      if ((((yr) % 4) == 0) && ((((yr) % 100) != 0) || (((yr) % 400) == 0)))
      {
        OD_days++;
      }
    }

    // Add days sicne 1/1 of this year
    for ( int mon = 0; mon < month-1; mon++ )
    {
      OD_days += days[mon];
    }

    // Add the current day
    OD_days += day;

    // Additional Leap year adjustment
    if ((((year) % 4) == 0) && ((((year) % 100) != 0) || (((year) % 400) == 0)) && month > 2)
      OD_days++;

    od_days_str = OD_DAY.toString(OD_days);
    return od_days_str;
  }

  // --------------------------------------------------------------------
  // display hits from the search
  // --------------------------------------------------------------------
  public static void displayHit(int cnt, DKDDO p)
  {
    short j = 0;
    if (p != null)
    {
      try
      {
        int numDataItems = numDataItems = p.dataCount();
        // display attribute (field) names (1st time only)
        if (cnt == 1)
        {
          System.out.println("");
          for (j = 1; j <= numDataItems; j++)
          {
            String strDataName = p.getDataName(j);
            if (strDataName.equals(DK_CM_DKPARTS) == false &&
                strDataName.equals("DKResource") == false &&
                strDataName.equals("DKViews") == false &&
                strDataName.equals("DKLargeObject") == false &&
                strDataName.equals("DKFixedView") == false &&
                strDataName.equals("DKAnnotations") == false)
            {
              System.out.print("  \t\""+strDataName+"\"");
            }
          }
          System.out.println("");
        }
        // display attribute (field) values
        for (j = 1; j <= numDataItems; j++)
        {
          Object a = p.getData(j);
          String strDataName = p.getDataName(j);
          if (strDataName.equals(DK_CM_DKPARTS) == false &&
              strDataName.equals("DKResource") == false &&
              strDataName.equals("DKViews") == false &&
              strDataName.equals("DKLargeObject") == false &&
              strDataName.equals("DKFixedView") == false &&
              strDataName.equals("DKAnnotations") == false)
          {
            if (j == 1)
              System.out.print(" "+cnt+".\t");
            else
              System.out.print("\t");

            if ((a instanceof String)  ||
                (a instanceof Integer) ||
                (a instanceof Short)   ||
                (a instanceof Long)    ||
                (a instanceof Float)   ||
                (a instanceof Double)  ||
                (a instanceof DKDate)  ||
                (a instanceof DKTime)  ||
                (a instanceof DKTimestamp))
            {
              System.out.print("\"" + a + "\"");
            }
            else
              System.out.print("");
          }
        }
        System.out.println("");
      }
      catch (DKException exc)
      {
        System.out.println("Exception name " + exc.name());
        System.out.println("Exception message " + exc.getMessage());
        System.out.println("Exception error code " + exc.errorCode());
        System.out.println("Exception error state " + exc.errorState());
        exc.printStackTrace();
      }
      catch (Exception exc)
      {
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
      }
    }
  }

  // --------------------------------------------------------------------
  // Retrieve and display results
  // --------------------------------------------------------------------
  public static void displayDDO(int           cnt,
                                DKDatastoreOD dsOD,
                                DKDDO         p,
                                boolean       retrieveToFile)
  {
    String DocFile, DataFile, LineDataFile, ResourceFile,
           AnnotationFile, TocFile, LogViewFile, FixedViewFile;

    FileOutputStream myLogViewFile, myFixedViewFile, myAnnotationFile,
                     myResourceFile, myTocFile, myDataFile, myDocFile;
    String extension = new String("");
    String MimeType = new String("");
    String noteString = new String("");
    String segmentFile;
    String cnt_str;
    DKAnnotationOD annotation = null;
    int numNotes = 0;
    int numBytes = 0;

    DKDDO pDDO = null;
    dkXDO pXDO = null;
    int i = 0;
    int numDataItems = 0;
    short k = 0;
    short j = 0;
    String strDataName;
    dkCollection pCol = null;
    dkIterator pIter = null;
    Object a = null;
    dkDataObjectBase pDO = null;
    DKPid pid = null;

    try
    {
      // create the output directory if it does not exist
      String OutDir;
      if (File.separator.equals("\\"))
        OutDir = new String("c:\\cmbuild\\output\\");
      else
        OutDir = new String("/archive/bulldog/eip/samples/output/");

      File outputPath = new File(OutDir);
      if (outputPath.exists() == false)
      {
        // try this first
        if (outputPath.mkdirs() == false)
        {
          // If mkdirs did not work, try this
          outputPath.mkdir();
        }
      }

      if (p != null)
      {
        System.out.println("\n=====> Item " + cnt + " <=====");

        numDataItems = p.dataCount();
        pid = p.getPidObject();

        // Retrieve privileges
        DKDatastoreExtOD dsODExt = (DKDatastoreExtOD)dsOD.getExtension(DK_CM_DATASTORE_EXT);
        DKCommonPrivilege priv = dsODExt.getCommonItemPrivilege(p);
        System.out.println("privileges are:");
        if (priv.checkAuthorization(DK_CM_PRIV_PRINT) == DK_CM_PRIV_YES)
          System.out.println(" +++can print document");
        else
          System.out.println(" ---cannot print document");
        if (priv.checkAuthorization(DK_CM_PRIV_VIEW_ANNOTATION) == DK_CM_PRIV_YES)
          System.out.println(" +++can view annotations");
        else
          System.out.println(" ---cannot view annotations");
        if (priv.checkAuthorization(DK_CM_PRIV_CREATE_ANNOTATION) == DK_CM_PRIV_YES)
          System.out.println(" +++can create annotations");
        else
          System.out.println(" ---cannot create annotations");
        if (priv.checkAuthorization(DK_CM_PRIV_DELETE_ANNOTATION) == DK_CM_PRIV_YES)
          System.out.println(" +++can delete annotations");
        else
          System.out.println(" ---cannot delete annotations");
        if (priv.checkAuthorization(DK_CM_PRIV_EDIT_TEXT_ANNOTATION) == DK_CM_PRIV_YES)
          System.out.println(" +++can edit annotations");
        else
          System.out.println(" ---cannot edit annotations");
        if (priv.checkAuthorization(DK_CM_PRIV_MODIFY_MULTI_VIEWDATA) == DK_CM_PRIV_YES)
          System.out.println(" +++can change logical views");
        else
          System.out.println(" ---cannot change logical views");

        System.out.println("object type is: "+pid.getObjectType());

        System.out.println("\npid string: " + pid.pidString());
        k = p.propertyId(DK_CM_PROPERTY_ITEM_TYPE);

        if (k > 0)
        {
          Short sVal = (Short)p.getProperty(k);
          j = sVal.shortValue();
          switch (j)
          {
            case DK_CM_DOCUMENT :
              System.out.println("  Item is a document ");
              break;
            default:
              System.out.println("  Unknown property: "+k);
          }
        }

        cnt_str = new String(Integer.toString(cnt));
        DataFile     = OutDir + new String("Docdata")  + cnt_str;
        LineDataFile = OutDir + new String("Linedata") + cnt_str;
        ResourceFile = OutDir + new String("resource") + cnt_str + new String(".dat");
        AnnotationFile = OutDir + new String("annotation") + cnt_str + new String(".dat");
        TocFile = OutDir + new String("Toc") + cnt_str + new String(".dat");
        LogViewFile    = OutDir + new String("LogView") + cnt_str + new String(".dat");
        FixedViewFile  = OutDir + new String("FixedView") + cnt_str + new String(".dat");

        System.out.println("  DataFile=>"+DataFile+"<=");
        System.out.println("  ResourceFile=>"+ResourceFile+"<=");
        System.out.println("  AnnotationFile=>"+AnnotationFile+"<=");
        System.out.println("  LineDataFile=>"+LineDataFile+"<=");
        System.out.println("  TocFile=>"+TocFile+"<=");
        System.out.println("  LogViewFile=>"+LogViewFile+"<=");
        System.out.println("  FixedViewFile=>"+FixedViewFile+"<=");

        // Removed existing output files
        File LineFile = new File(LineDataFile + new String(".lin"));
        if (LineFile.exists())
          LineFile.delete();
        File ResFile = new File(ResourceFile);
        if (ResFile.exists())
          ResFile.delete();
        File AnnFile = new File(AnnotationFile);
        if (AnnFile.exists())
          AnnFile.delete();
        File TOCFile = new File(TocFile);
        if (TOCFile.exists())
          TOCFile.delete();
        File ViewFile = new File(LogViewFile);
        if (ViewFile.exists())
          ViewFile.delete();
        File FixViewFile = new File(FixedViewFile);
        if (FixViewFile.exists())
          FixViewFile.delete();

        System.out.println("  Number of Data Items: " + numDataItems);

        for (j = 1; j <= numDataItems; j++)
        {
          a = p.getData(j);
          strDataName = p.getDataName(j);
          System.out.println("    " + j + ". Attribute Name: " + strDataName );
          System.out.println("       type: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_TYPE));
          System.out.println("       nullable: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_NULLABLE));

          if (strDataName.equals(DK_CM_DKPARTS) == false &&
              strDataName.equals("DKResource") == false &&
              strDataName.equals("DKViews") == false &&
              strDataName.equals("DKLargeObject") == false &&
              strDataName.equals("DKFixedView") == false &&
              strDataName.equals("DKAnnotations") == false)
          {
            System.out.println("       attribute id: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_ATTRIBUTE_ID));
          }

          if (a instanceof String)
          {
            System.out.println("       Attribute Value(String): =>" + a + "<=");
          }
          else if (a instanceof Integer)
          {
            System.out.println("       Attribute Value(Integer): " + a);
          }
          else if (a instanceof Short)
          {
            System.out.println("       Attribute Value(Short): " + a);
          }
          else if (a instanceof Long)
          {
            System.out.println("       Attribute Value(Long): " + a);
          }
          else if (a instanceof Float)
          {
            System.out.println("       Attribute Value(Float): " + a);
          }
          else if (a instanceof Double)
          {
            System.out.println("       Attribute Value(Double): " + a);
          }
          else if (a instanceof DKDate)
          {
            System.out.println("      Attribute Value(DKDate): " + a);
          }
          else if (a instanceof DKTime)
          {
            System.out.println("       Attribute Value(DKTime): " + a);
          }
          else if (a instanceof DKTimestamp)
          {
            System.out.println("       Attribute Value(DKTimestamp): " + a);
          }
          else if (a instanceof dkCollection)
          {
            System.out.println("       Attribute Value is collection");
            pCol = (dkCollection)a;
            pIter = pCol.createIterator();
            i = 0;

            while (pIter.more() == true)
            {
              i++;
              a = pIter.next();
              pDO = (dkDataObjectBase)a;

              if (pDO.protocol() == DK_CM_XDO)
              {
                System.out.println("         dkXDO object " + i + " in collection");
                pXDO = (dkXDO)pDO;
                DKPid pid2 = pXDO.getPidObject();
                System.out.println("           XDO pid string: " + pid2.pidString());

                // Get the MIME type
                MimeType = pXDO.getMimeType();
                System.out.println("MimeType = " + MimeType);
                if ( MimeType.equals("application/ondemand line") == true ||
                     MimeType.equals("application/lin") == true )
                {
                  // Set the EBCDIC->ASCII conversion
                  a = (new Integer(DK_CM_TEXT_ASCII));
                  pXDO.setOption( DK_CM_OPT_XDO_TEXT_CONVERSION, a);
                  System.out.println("EBCDIC->ASCII conversion set!");
                }
                byte[] DataBytes = null;
                String DataString = "";
                long   DataSize  = 0;
                // Retrieve and open instance handler for an XDO
                if (!retrieveToFile)
                {
                  pXDO.retrieve();
                  DataBytes = pXDO.getContent();
                  DataString = new String(DataBytes);
                  DataSize = DataString.length();
                }
                System.out.println("Content Begin:");
                long item_type = ((DKPidXDOOD)pXDO.getPidObject()).getType();
                long part_id = ((DKPidXDOOD)pXDO.getPidObject()).getPartId();
                switch ((short)item_type)
                {
                  case DK_CM_ANNOTATION:
                    // binary
                    System.out.println("======DK_OD_ANNOTATION (Notes)======");
                    if (retrieveToFile)
                    {
                      pXDO.retrieve(AnnotationFile);
                      DataSize = pXDO.size();
                    }
                    else
                    {
                      annotation = (DKAnnotationOD)((DKBlobOD)pXDO).getExtension("dkAnnotationExt");
                      noteString  = "NOTE: " + annotation.getTimestamp();
                      noteString += " by " + annotation.getUserID();
                      noteString += " for page " + Integer.toString(annotation.getPageNumber());
                      noteString += "\n";
                      noteString += annotation.isPublic() ? "Public" : "Private";
                      noteString += "--";
                      noteString += annotation.isOkToCopy() ? "Can" : "Cannot";
                      noteString += " be copied to another server";
                      noteString += new String("\n") + annotation.getText();
                      System.out.println( noteString );
                    }
                    break;
                  case DK_OD_VIEW:
                    // binary
                    System.out.println("======DK_OD_VIEW (Logical Views)======");
                    if (retrieveToFile)
                    {
                      pXDO.retrieve(LogViewFile);
                      DataSize = pXDO.size();
                    }
                    else
                    {
                      myLogViewFile = new FileOutputStream(LogViewFile, true);  // append
                      myLogViewFile.write(DataBytes);
                      myLogViewFile.close();
                    }
                    break;
                  case DK_OD_SEGMENT:
                    // binary
                    if ( extension.equals("") == true )
                    {
                      if ( MimeType.equals("application/afp") == true )
                        extension = new String("afp");
                      else if ( MimeType.equals("application/ondemand line") == true )
                        extension = new String("lin");
                      else if ( MimeType.equals("application/lin") == true )
                        extension = new String("lin");
                      else if ( MimeType.equals("text/plain") == true )
                        extension = new String("txt");
                      else if ( MimeType.equals("image/bmp") == true )
                        extension = new String("bmp");
                      else if ( MimeType.equals("image/gif") == true )
                        extension = new String("gif");
                      else if ( MimeType.equals("image/jpeg") == true )
                        extension = new String("jpg");
                      else if ( MimeType.equals("image/pcx") == true )
                        extension = new String("pcx");
                      else if ( MimeType.equals("image/tif") == true )
                        extension = new String("tif");
                      else if ( MimeType.equals("application/pdf") == true )
                        extension = new String("pdf");
                      else
                        extension = new String("dat");
                    }
                    System.out.println("======DK_OD_SEGMENT (Doc Data) part_id=" + part_id + "======");
                    if (extension.equals("lin"))
                      segmentFile = LineDataFile + new String(".") + extension;
                    else
                      segmentFile = DataFile + new String(".") + extension;
                    if ( part_id == 0 )
                    {
                      File SegFile = new File(segmentFile);
                      if (SegFile.exists())
                        SegFile.delete();
                    }
                    // The first segment starts at part_id 0
                    if ( part_id >= 0)
                    {
                      if (retrieveToFile)
                      {
                        pXDO.retrieve(segmentFile);
                        DataSize = pXDO.size();
                      }
                      else
                      {
                        myDataFile = new FileOutputStream(segmentFile, true);  // append
                        myDataFile.write(DataBytes);
                        myDataFile.close();
                      }
                    }
                    break;
                }
                System.out.println("\nContent End");
              }
            }
          }
          else if (a != null)
          {
            System.out.println("       Attribute Value: " + a.toString());
            if (strDataName.equals("DKResource")  ||
                strDataName.equals("DKFixedView") ||
                strDataName.equals("DKLargeObject"))
            {
              pDO = (dkDataObjectBase)a;

              if (pDO.protocol() == DK_CM_XDO)
              {
                System.out.println("         dkXDO object ");
                pXDO = (dkXDO)pDO;
                DKPid pid2 = pXDO.getPidObject();
                System.out.println("           XDO pid string: " + pid2.pidString());

                byte[] DataBytes = null;
                String DataString = "";
                long DataSize = 0;
                if (!retrieveToFile)
                {
                  // Retrieve and open instance handler for an XDO
                  pXDO.retrieve();
                  // pXDO.open();

                  DataBytes = pXDO.getContent();
                  DataString = new String(DataBytes);
                  DataSize = DataString.length();
                }
                long item_type = ((DKPidXDOOD)pXDO.getPidObject()).getType();

                if (strDataName.equals("DKResource") )
                {
                  // binary
                  System.out.println("======DK_OD_RESOURCE (AFP Resource Group)======");
                  if (retrieveToFile)
                  {
                    pXDO.retrieve(ResourceFile);
                    DataSize = pXDO.size();
                  }
                  else
                  {
                    myResourceFile = new FileOutputStream(ResourceFile, true);  // append
                    myResourceFile.write(DataBytes);
                    myResourceFile.close();
                  }
                }
                else if (strDataName.equals("DKFixedView"))
                {
                  // string
                  System.out.println("======DK_OD_FIXED_VIEW (Fixed View)======");
                  if (retrieveToFile)
                  {
                    pXDO.retrieve(FixedViewFile);
                  }
                  else
                  {
                    System.out.println( DataString );
                  }
                }
                else if (item_type == DK_OD_TOC)
                {
                  // binary
                  System.out.println("======DK_OD_TOC (Large Object table of contents)======");
                  if (retrieveToFile)
                  {
                    pXDO.retrieve(TocFile);
                    DataSize = pXDO.size();
                  }
                  else
                  {
                    myTocFile = new FileOutputStream(TocFile, true);  // append
                    myTocFile.write(DataBytes);
                    myTocFile.close();
                  }
                }
              }
            }
          }
          else System.out.println("       Attribute Value is null");
        }

        // If the resource file exists, concatenate the resource with data
        ResFile = new File(ResourceFile);
        if (ResFile.exists())
        {
          DocFile = OutDir + new String("sample") + cnt_str + new String(".") + extension;
          File DocumentFile = new File(DocFile);
          if (DocumentFile.exists())
            DocumentFile.delete();

          int BytesTotal = 0;
          byte[] buffer = new byte[100000];
          myDocFile = new FileOutputStream(DocFile);
          segmentFile = DataFile + new String(".") + extension;

          FileInputStream myResFile = new FileInputStream(ResourceFile);
          FileInputStream mySegFile = new FileInputStream(segmentFile);
          int BytesExpected = myResFile.available() + mySegFile.available();
          //System.out.println("Expecting "+BytesExpected+" bytes.");
          Vector v = new Vector();
          v.addElement(myResFile);
          v.addElement(mySegFile);
          SequenceInputStream afpStream = new SequenceInputStream(v.elements());

          // copy data
          if ( afpStream != null)
          {
            int DataBytes = 0;
            while ( BytesTotal < BytesExpected )
            {
              try
              {
                numBytes = afpStream.read(buffer);
                if ( numBytes == -1  )
                {
                  // this is a quick way to stop
                  break;
                }
                DataBytes += numBytes;
                myDocFile.write(buffer, 0, numBytes);
                BytesTotal += numBytes;
              }
              catch (Exception e)
              {
                System.out.println("exception: e="+e);
                break;
              }
            }
          }
          //System.out.println("BytesTotal="+BytesTotal);

          myDocFile.close();
          afpStream.close();

          File SegFile = new File(segmentFile);
          SegFile.delete();
          ResFile.delete();

        } // if (ResFile.exists())

      }  // if (p != null)

    }
    catch (DKException exc)
    {
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      System.out.println("Exception error code " + exc.errorCode());
      System.out.println("Exception error state " + exc.errorState());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }
  }

  // --------------------------------------------------------------------
  // set application group and query SQL
  // --------------------------------------------------------------------
  public static void setAGQueryCriteria(int testcase, String libSrv)
  {
    int month = 6;
    int day   = 2;
    int year  = 1997;
    Integer temp_int = new Integer(0);
    String  mon_str, day_str, yr_str, date_str;
    String od_date = "";

    // Application Group and SQL string
    // OD/390 v2.1
    if (libSrv.equalsIgnoreCase("9.91.121.31") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // AFP
          appgrp = "INSURE  01 ";
          cmd = "WHERE (POLICY_NUMBER LIKE '324-1443255%')";  //'324-1443255-11' ";
          break;
        case 2:
          appgrp = "CHKSTMN301 ";
          cmd = "WHERE ACCOUNT_NUMBER LIKE '%' ";
          break;
        case 3:  // line/large object
          appgrp = "DELINQCY02 ";
          cmd = "WHERE ( LOAN_NUMBER LIKE '%' )";
          break;
        case 4:  // pdf
          appgrp = "KEYBANK 01 ";
          cmd = "WHERE ( ACCOUNT_NUMBER LIKE '%' )";
          break;
      }
    }
    // OD/390 v7.1
    else if (libSrv.equalsIgnoreCase("9.91.121.34") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // AFP data
          appgrp = "Credit Card Statements";
          cmd = "WHERE ( name LIKE '%' )";
          break;
        case 2:
          appgrp = "CHKSTMNT";
          cmd = "WHERE ACCOUNT_NUMBER LIKE '%' ";
          break;
        case 3:  // line data
          appgrp = "Delinqcy-AG";
          cmd = "WHERE ( pagenum != 0 )";
          break;
        case 4:  // line data large object
          appgrp = "LOAN LO";
          od_date = ODdate( 10, 1, 1994 );
          cmd = "WHERE ( load_date = " + od_date + " )";
          //od_date = ODdate( 2, 2, 1970 );
          //cmd  = "WHERE (( load_date > " + od_date + ") AND ";
          //cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
          break;
        case 5:  // pdf
          appgrp = "PDF landscape";
          cmd = "WHERE ( region LIKE '%' )";
        case 6:  // AFP
          appgrp = "afpdata";
          cmd = "WHERE (TaxYear = 1995)";
          break;
      }
    }
    else if (libSrv.equalsIgnoreCase("gunnar") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // Large object AFP - gunnar
          appgrp = "OnDemand Publications";
          cmd = "where bookname LIKE '%'";
          break;
      }
    }
    else if (libSrv.length() >= 6 &&
             libSrv.substring(0,6).equalsIgnoreCase("reagan"))
    {
      switch (testcase)
      {
        default:
        case 1:
          appgrp = "load-db2-CRD";
          cmd = "where Account LIKE '%001'";
          //cmd = "where name LIKE '%'";
          break;
        case 2:
          appgrp = "load-db2-afpdata";
          //cmd = "where BranchID LIKE '%'";
          cmd = "where TaxpayeraAcctNo LIKE '%'";
          //cmd = "where TaxpayerID = '2222222'"; //LIKE '2%'";
          break;
        case 3:
          appgrp = "load-db2-Retain";
          cmd = "where pmr_num LIKE '%'";
          break;
      }
    }
    else if (libSrv.equalsIgnoreCase("elric") == true)
    {
      switch (testcase)
      {
        default:
        case 1:
          appgrp = "Customer Information";
          cmd = "where Application LIKE 'CI%'";
          break;
      }
    }
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("everest"))
    {
      switch (testcase)
      {
        default:
        case 1:
          // AFP data
          appgrp = "CRD";
          cmd = "WHERE ( name LIKE '%' )";
          break;
        case 2:
          // Large Object line data
          appgrp = "sp-ebcdic-lo";
          //od_date = ODdate( 1, 1, 1990 );
          //od_date2 = ODdate( 12, 31, 2001 );
          //cmd = "WHERE ( rpt_date BETWEEN " + od_date + " AND " + od_date2 + " )";
          od_date = ODdate( 12, 15, 2000 );
          cmd = "WHERE ( rpt_date > 0 )"; //= " + od_date + " )";
          break;
        case 3: // Loan Delinquency Reports - Large Objects
          appgrp = "LOAN LO";
          od_date = ODdate( 2, 2, 1970 );   // 2/2/1970
          cmd  = "WHERE (( load_date > " + od_date + ") AND ";
          cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
          break;
        case 4:
          // Large Object line data
          appgrp = "eliz-largeline";
          od_date = ODdate( 3, 19, 1999 );
          cmd = "WHERE ( rptdate = " + od_date + " )";
          break;
      case 5:
        // Large Object line data
        appgrp = "43MBPDF";
        cmd = "WHERE ( rtitle LIKE '%' )";
        break;
      }
    }
    // OD/400 v4.5
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("rdr400c") == true)
    {
      switch (testcase)
      {
        default:
        case 1:
          appgrp = "CHECKSTMTS ";
          cmd = "WHERE ( KY1VAL LIKE '%' )";
          break;
      }
    }
    // OD/400 v5.1
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("rdr400g") == true)
    {
      switch (testcase)
      {
        default:
        case 1:
          appgrp = "CheckStatements";
          cmd = "WHERE ( name LIKE '%' )";
          break;
      }
    }
    else if (libSrv.length() >= 5 &&
             libSrv.substring(0,5).equalsIgnoreCase("akira") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // AFP
          appgrp = "Few Credit Card Statements";
          cmd = "WHERE ( name LIKE '%' )";
          break;
        case 2: // Line - Loan Delinquency Reports
          appgrp = "Loan Delinquency Reports";
          od_date = ODdate( 2, 2, 1970 );   // 2/2/1970
          cmd  = "WHERE (( load_date > " + od_date + ") AND ";
          cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
          break;
        case 3: // User defined types
          appgrp = "User Defined Data Types";
          cmd = "WHERE ( Docname LIKE '%' )";
          break;
        case 4:
          appgrp = "LARpmr94348z";
          cmd = "WHERE ( cuid LIKE '%' )";
          break;
        case 5:
          appgrp = "GFAC";
          cmd = "WHERE ( nomcli LIKE 'MR PEREZ ERIC' )";
          break;
      }
    }
    else if (libSrv.length() >= 5 &&
             libSrv.substring(0,5).equalsIgnoreCase("elric") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // AFP
          appgrp = "Customer Information";
          cmd = "WHERE ( Application LIKE '%' )";
          break;
      }
    }
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("atwater") == true)
    {
      switch (testcase)
      {
        default:
        case 1:
          appgrp = "load-db2-Retain";
          cmd = "WHERE ( pmr_num LIKE '%' )";
          break;
      }
    }
    else // can be any server!
    {
      switch (testcase)
      {
        default:
        case 1: // AFP
          mon_str = temp_int.toString( month );
          day_str = temp_int.toString( day );
          yr_str  = temp_int.toString( year );
          date_str = mon_str + '/' + day_str + '/' + yr_str;
          // native OD integer date
          od_date = ODdate( month, day, year );
          System.out.println("The date '"+ date_str +"' equals to the OnDemand integer date '"+ od_date+"'" );

          appgrp = "CRD-LO";
          //cmd = "WHERE ( crd_date = " + od_date + " )";
          cmd = "WHERE ( name LIKE '%')";
          break;
        case 2:  // line
          appgrp = "Balance";
          cmd = "WHERE ( hiercode LIKE '..21...002')"; //'%.....006')";
          break;
      }
    }
  }
}

