/**
@copyright(disclaimer)
Licensed Materials - Property of IBM

IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE
IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR
SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE
SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS
BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE
DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS HAVE BEEN
ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import java.util.*;
import java.io.*;
import com.ibm.mm.viewer.*;

/**
 * This sample performs document conversions using CMBStreamingDocServices and the document engines.
 * This sample also illustrates loading a multipart document and also passing an annotation stream to
 * the CMBStreamingDocServices to display the annotations along with the document.
 *<br>
 *>br>Usage: Syntax: java TStreamingDocServices <docfilename> <docmimetype> <numberOfParts> <annofilename>
 *            <docfilename> - the full path of the first part of the document.
 *                              For multipart documents, provide the other parts in the same location
 *                              and use the same name for all the parts appended by a number 1, 2, 3
 *                              and so on to name  part1, part2 and part3.
 *            <docmimetype>   - the document mimetype
 *            <numberOfParts> - the number of content parts
 *            <annofilename>  - the annotation file name or just 'null' if no annotations
 *
 *            Example1:     "java TStreamingDocServices d:\testdocs\testpart image/tiff 3 d:\testdocs\testanno.dat"
 *                          - this would look for testpart1, testpart2, testpart3 files in the same dir as tespart
 *
 *            Example2:     "java TStreamingDocServices d:\testdocs\ibm.gif image/gif 1 null"
 */
public class TStreamingDocServices
{
    private String partName = new String();
    private String ext = new String();

    private boolean traceEnabled = true;

	public static void main(String args[])
	{
		TStreamingDocServices app = new TStreamingDocServices();
		app.run(args);
		System.exit(0);
	}

    public void run(String[] args) {
        try {

            if (args.length != 4) {
                System.out.println("Syntax: java TStreamingDocServices <docfilename> <docmimetype> <numberOfParts> <annofilename>");
                System.out.println("<docfilename> - the full path of the first part of the document.");
                System.out.println("                For multipart documents, provide the other parts in the same location ");
                System.out.println("                and use the same name for all the parts ");
                System.out.println("                appended by a number 1, 2, 3 and so on to name  part1, part2 and part3");
                System.out.println("<docmimetype>   - the document mimetype");
                System.out.println("<numberOfParts> - the number of content parts");
                System.out.println("<annofilename>  - the annotation file name or just 'null' if no annotations");
                System.exit(0);
            }

            System.out.println("Creating streaming doc services");
            Properties engineProperties = new Properties();
            engineProperties.put("ENGINES","4");
            engineProperties.put("ENGINE1_CLASSNAME","com.ibm.mm.viewer.mstech.CMBMSTechDocumentEngine");
            engineProperties.put("ENGINE2_CLASSNAME","com.ibm.mm.viewer.inso.CMBMSTechInsoEngine");
            engineProperties.put("ENGINE3_CLASSNAME","com.ibm.mm.viewer.CMBODDocumentEngine");
            engineProperties.put("ENGINE4_CLASSNAME","com.ibm.mm.viewer.CMBJavaDocumentEngine");
            CMBStreamingDocServices docServices = new CMBStreamingDocServices(new Callbacks(),engineProperties);


            System.out.println("Loading doc " + args[0] + ":" + args[1] );
            String doc = args[0];
            FileInputStream inStream = new FileInputStream(doc);
            int extIndex = doc.lastIndexOf('.');
            System.out.println("doc string:" + doc + " length:" + doc.length() + " extindex:" + extIndex);
            if( extIndex != -1 ){
                partName = doc.substring(0, extIndex);
                ext = doc.substring(extIndex,doc.length());
                System.out.println("ext:" + ext );
            }
            else {
                partName = doc;
            }

            System.out.println("part name:" + partName);
            int nParts = Integer.parseInt( args[2] );
            System.out.println("Number of parts:" + nParts);
            FileInputStream annoStream = null;
            if( args[3].equalsIgnoreCase("null") )
                annoStream = null;
            else
                annoStream = new FileInputStream(args[3]);
            System.out.println("Annotation file:" + args[3]);
            CMBDocument document = docServices.loadDocument(inStream, nParts, args[1], args[1], annoStream,null);
            if( document.getCanShowAnnotations() )
            {
                document.setShowAnnotations( true );
            }
            if (document.getCanWrite()) {
                String outfilename = "document.dat";
                System.out.println("Converting document to "+document.getWriteMimeType()+" as file "+outfilename);
                FileOutputStream outStream = new FileOutputStream(outfilename);
                document.write(outStream);
                outStream.close();
            }
            if (document.getCanPaginate()) {
                int pagecount = document.getPageCount();
                System.out.println("Document has "+pagecount+" pages.");
                for (int npage = 0; npage < pagecount; npage++) {
                    String outfilename = Thread.currentThread().getName()+"page"+npage+".gif";
                    System.out.println("Converting page "+npage+" to image/gif as file "+outfilename);
                    CMBPage page = document.getPages(npage);
                    FileOutputStream outStream = new FileOutputStream(outfilename);
                    page.write(outStream);
                    outStream.close();
                }
            }
            System.out.println("Dropping doc");
            docServices.dropDocument(document);
            document = null;
            inStream.close();
            inStream = null;
            if( annoStream != null )
            {
                annoStream.close();
                annoStream = null;
            }

            System.out.println("Terminating streaming doc services");
            docServices.terminate();

        } catch (Throwable e) {
            e.printStackTrace();
        }

    }

    /** Callbacks used by docservices.  Stubbed. */
    class Callbacks extends CMBStreamingDocServicesCallbacks {
        public InputStream getForm(CMBDocument document, String formName) {
            System.out.println("Callback for form "+formName+" made.  Returned null.");
            return null;
        }
        public InputStream getPart(CMBDocument document, int partNumber, StringBuffer mimetype) {
            System.out.println("Callback for " + partName + partNumber+ ext + " made.");
            FileInputStream inStream = null;
            try {
              inStream = new FileInputStream(partName + partNumber + ext);
            }catch(Exception e)
            {
            }
            return inStream;
        }
        public InputStream getAnnotation(CMBDocument document, int annotationNumber) {
            System.out.println("Callback for annotation "+annotationNumber+" made.  Returned null.");
            return null;
        }
        public InputStream getResources(CMBDocument document) {
            System.out.println("Callback for resources made.  Returned null.");
            return null;
        }
        public boolean traceEnabled() {
            return traceEnabled;
        }
        public void trace(String message) {
            System.out.println("Callback for trace: "+message);
        }
    }
}
